@@ -27,17 +27,18 @@ Z* ------------------------------------------------------------------- */
 #include"MemoryCache.h"
 #include"Base.h"
 
-static MapType *_MapNew(PyMOLGlobals * G, float range, float *vert, int nVert,
-                        float *extent, int *flag, int group_id, int block_id);
+static MapType *_MapNew(PyMOLGlobals * G, float range, const float *vert, int nVert,
+                        const float *extent, const int *flag, int group_id, int block_id);
 
 float MapGetDiv(MapType * I)
 {
   return I->Div;
 }
 
-void MapFree(MapType * I)
+MapType::~MapType()
 {
-  if(I) {
+  auto I = this;
+  {
     CacheFreeP(I->G, I->Head, I->group_id, I->block_base + cCache_map_head_offset, false);
     CacheFreeP(I->G, I->Link, I->group_id, I->block_base + cCache_map_link_offset, false);
     CacheFreeP(I->G, I->EHead, I->group_id, I->block_base + cCache_map_ehead_offset,
@@ -47,7 +48,6 @@ void MapFree(MapType * I)
     VLACacheFreeP(I->G, I->EList, I->group_id, I->block_base + cCache_map_elist_offset,
                   false);
   }
-  OOFreeP(I);
 }
 
 int MapCacheInit(MapCache * M, MapType * I, int group_id, int block_base)
@@ -423,8 +423,8 @@ int MapSetupExpressXYVert(MapType * I, float *vert, int n_vert, int negative_sta
   return ok;
 }
 
-int MapSetupExpressPerp(MapType * I, float *vert, float front, int nVertHint,
-			int negative_start, int *spanner)
+int MapSetupExpressPerp(MapType * I, const float *vert, float front, int nVertHint,
+			int negative_start, const int *spanner)
 {
   PyMOLGlobals *G = I->G;
   int n = 0;
@@ -443,7 +443,7 @@ int MapSetupExpressPerp(MapType * I, float *vert, float front, int nVertHint,
   float min0 = I->Min[0] * iDiv;
   float min1 = I->Min[1] * iDiv;
   float base0, base1;
-  float perp_factor, premult, *v0;
+  float perp_factor, premult;
   int *emask, dim1, *link, *ptr1, *ptr2;
 
   PRINTFD(G, FB_Map)
@@ -478,7 +478,7 @@ int MapSetupExpressPerp(MapType * I, float *vert, float front, int nVertHint,
 
         i = *MapFirst(I, a, b, c);
         while(i >= 0) {
-          v0 = vert + 3 * i;
+          const float* v0 = vert + 3 * i;
           perp_factor = premult / v0[2];
           base0 = v0[0] * perp_factor;
           base1 = v0[1] * perp_factor;
@@ -733,6 +733,18 @@ int MapExclLocus(MapType * I, const float *v, int *a, int *b, int *c)
   return (1);
 }
 
+/**
+ * Return EList start index for points in proximity to `v`.
+ * Return 0 if `v` is outside the grid or there are no points.
+ */
+int MapExclLocusEStart(MapType* map, const float* v)
+{
+  int h, k, l;
+  if (!MapExclLocus(map, v, &h, &k, &l))
+    return 0;
+  return *(MapEStart(map, h, k, l));
+}
+
 float MapGetSeparation(PyMOLGlobals * G, float range, const float *mx, const float *mn,
                        float *diagonal)
 {
@@ -802,38 +814,36 @@ float MapGetSeparation(PyMOLGlobals * G, float range, const float *mx, const flo
   return (divSize);
 }
 
-MapType *MapNew(PyMOLGlobals * G, float range, float *vert, int nVert, float *extent)
+MapType *MapNew(PyMOLGlobals * G, float range, const float *vert, int nVert, const float *extent)
 {
   return (_MapNew(G, range, vert, nVert, extent, NULL, -1, 0));
 }
 
-MapType *MapNewCached(PyMOLGlobals * G, float range, float *vert, int nVert,
-                      float *extent, int group_id, int block_id)
+MapType *MapNewCached(PyMOLGlobals * G, float range, const float *vert, int nVert,
+                      const float *extent, int group_id, int block_id)
 {
   return (_MapNew(G, range, vert, nVert, extent, NULL, group_id, block_id));
 }
 
-MapType *MapNewFlagged(PyMOLGlobals * G, float range, float *vert, int nVert,
-                       float *extent, int *flag)
+MapType *MapNewFlagged(PyMOLGlobals * G, float range, const float *vert, int nVert,
+                       const float *extent, const int *flag)
 {
   return (_MapNew(G, range, vert, nVert, extent, flag, -1, 0));
 }
 
-static MapType *_MapNew(PyMOLGlobals * G, float range, float *vert, int nVert,
-                        float *extent, int *flag, int group_id, int block_base)
+static MapType *_MapNew(PyMOLGlobals * G, float range, const float *vert, int nVert,
+                        const float *extent, const int *flag, int group_id, int block_base)
 {
   int a, c;
   int mapSize;
   int h, k, l;
   int *list;
-  float *v, tmp_f;
+  const float *v;
   int firstFlag;
   Vector3f diagonal;
   int ok = true;
 
-  /* allocate space for a MapType 
-   * "I" is now defined */
-  OOAlloc(G, MapType);
+  auto I = new MapType();
   PRINTFD(G, FB_Map)
     " MapNew-Debug: entered.\n" ENDFD;
   CHECKOK(ok, I);
@@ -844,12 +854,6 @@ static MapType *_MapNew(PyMOLGlobals * G, float range, float *vert, int nVert,
   I->G = G;
   I->group_id = group_id;
   I->block_base = block_base;
-  I->Head = NULL;
-  I->Link = NULL;
-  I->EHead = NULL;
-  I->EList = NULL;
-  I->EMask = NULL;
-  I->NEElem = 0;
 
   /* initialize an empty cache for the map */
   I->Link = CacheAlloc(G, int, nVert, group_id, block_base + cCache_map_link_offset);
@@ -930,9 +934,7 @@ static MapType *_MapNew(PyMOLGlobals * G, float range, float *vert, int nVert,
   /* sanity check */
   for(a = 0; a < 3; a++) {
     if(I->Min[a] > I->Max[a]) {
-      tmp_f = I->Min[a];
-      I->Max[a] = I->Min[a];
-      I->Min[a] = tmp_f;
+      std::swap(I->Max[a], I->Min[a]);
     }
 
     // empirical limit to avoid crash in PYMOL-3002
