@@ -93,7 +93,7 @@ static int TetsurfCodeVertices(CTetsurf * II);
 
 static int TetsurfFindActiveBoxes(CTetsurf * II, int mode, int *n_strip, int n_vert,
                                   int **strip_l, float **vert,
-                                  MapType * voxelmap, float *a_vert,
+                                  MapType * voxelmap, const float* a_vert,
                                   float carvebuffer, int side);
 
 #define TetsurfSubSize		50
@@ -309,7 +309,7 @@ static CTetsurf *TetsurfNew(PyMOLGlobals * G)
 #define cM_101_111 0x20000
 #define cM_110_111 0x40000
 
-  CTetsurf *I = Calloc(CTetsurf, 1);
+  CTetsurf *I = pymol::calloc<CTetsurf>(1);
   int c;
   int nv = 1;
   int last_nv;
@@ -516,7 +516,7 @@ void TetsurfGetRange(PyMOLGlobals * G,
 
 /*===========================================================================*/
 int TetsurfVolume(PyMOLGlobals * G, Isofield * field, float level, int **num,
-                  float **vert, int *range, int mode, MapType * voxelmap, float *a_vert,
+                  float **vert, int *range, int mode, MapType * voxelmap, const float* a_vert,
                   float carvebuffer, int side)
 {
 
@@ -563,9 +563,9 @@ int TetsurfVolume(PyMOLGlobals * G, Isofield * field, float level, int **num,
        }
      */
 
-    I->Coord = field->points;
-    I->Grad = field->gradients;
-    I->Data = field->data;
+    I->Coord = field->points.get();
+    I->Grad = field->gradients.get();
+    I->Data = field->data.get();
     I->Level = level;
     if(ok)
       ok = TetsurfAlloc(I);
@@ -641,13 +641,13 @@ static int TetsurfAlloc(CTetsurf * II)
     dim4[a] = I->CurDim[a];
   dim4[3] = 3;
 
-  I->VertexCodes = FieldNew(G, I->CurDim, 3, sizeof(int), cFieldInt);
+  I->VertexCodes = CField::make<int>(G, I->CurDim, 3);
   ErrChkPtr(G, I->VertexCodes);
-  I->ActiveEdges = FieldNew(G, I->CurDim, 3, sizeof(int), cFieldInt);
+  I->ActiveEdges = CField::make<int>(G, I->CurDim, 3);
   ErrChkPtr(G, I->ActiveEdges);
 
   dim4[3] = 7;                  /* seven different ways now... */
-  I->Point = FieldNew(G, dim4, 4, sizeof(PointType), cFieldOther);
+  I->Point = CField::make<PointType>(G, dim4, 4);
   ErrChkPtr(G, I->Point);
 
   I->Tri = VLAlloc(TriangleType, 50000);
@@ -674,25 +674,14 @@ static void TetsurfPurge(CTetsurf * II)
   if(I->PtLink) {
     VLAFreeP(I->PtLink);
   }
-  if(I->VertexCodes) {
-    FieldFree(I->VertexCodes);
-    I->VertexCodes = NULL;
-  }
-  if(I->ActiveEdges) {
-    FieldFree(I->ActiveEdges);
-    I->ActiveEdges = NULL;
-  }
-  if(I->Point) {
-    FieldFree(I->Point);
-    I->Point = NULL;
-  }
+  DeleteP(I->VertexCodes);
+  DeleteP(I->ActiveEdges);
+  DeleteP(I->Point);
 }
 
 
 /*===========================================================================*/
-#ifdef _PYMOL_INLINE
-__inline__
-#endif
+inline
 static void TetsurfInterpolate2(float *pt, float *v0, float l0, float *v1, float l1,
                                 float level)
 {
@@ -705,9 +694,6 @@ static void TetsurfInterpolate2(float *pt, float *v0, float l0, float *v1, float
 
 
 /*===========================================================================*/
-#ifdef _PYMOL_INLINE
-__inline__
-#endif
 static void TetsurfInterpolate4(float *pt, float *v0, float l0, float *v1, float l1,
                                 float l2, float l3, float level)
 {
@@ -730,9 +716,6 @@ static void TetsurfInterpolate4(float *pt, float *v0, float l0, float *v1, float
 
 
 /*===========================================================================*/
-#ifdef _PYMOL_INLINE
-__inline__
-#endif
 static void TetsurfInterpolate8(float *pt, float *v0, float l0, float *v1, float l1,
                                 float l2, float l3, float l4,
                                 float l5, float l6, float l7, float level)
@@ -758,7 +741,7 @@ static void TetsurfInterpolate8(float *pt, float *v0, float l0, float *v1, float
 /*===========================================================================*/
 static int TetsurfFindActiveBoxes(CTetsurf * II, int mode, int *n_strip, int n_vert,
                                   int **strip_l, float **vert,
-                                  MapType * voxelmap, float *a_vert,
+                                  MapType * voxelmap, const float *a_vert,
                                   float carvebuffer, int side)
 {
   CTetsurf *I = II;
