@@ -264,6 +264,11 @@ int PConvPyStrToStr(PyObject * obj, char *ptr, int size)
   int ok = true;
   if(!obj) {
     ok = false;
+#if PY_MAJOR_VERSION >= 3
+  } else if(PyBytes_Check(obj)) {
+    auto strval = PyBytes_AsSomeString(obj);
+    UtilNCopy(ptr, strval.c_str(), size);
+#endif
   } else if(!PyString_Check(obj)) {
     ok = false;
     if(size)
@@ -534,7 +539,7 @@ int PConvPyListToFloatArrayImpl(PyObject * obj, float **f, bool as_vla)
     if (as_vla) {
       (*f) = VLAlloc(float, l);
     } else {
-      (*f) = Alloc(float, l);
+      (*f) = pymol::malloc<float>(l);
     }
 
     auto strval = PyBytes_AsSomeString(obj);
@@ -552,7 +557,7 @@ int PConvPyListToFloatArrayImpl(PyObject * obj, float **f, bool as_vla)
     if (as_vla) {
       (*f) = VLAlloc(float, l);
     } else {
-      (*f) = Alloc(float, l);
+      (*f) = pymol::malloc<float>(l);
     }
 
     ff = (*f);
@@ -648,7 +653,7 @@ int PConvPyListToDoubleArray(PyObject * obj, double **f)
       ok = -1;
     else
       ok = l;
-    (*f) = Alloc(double, l);
+    (*f) = pymol::malloc<double>(l);
     ff = (*f);
     for(a = 0; a < l; a++)
       *(ff++) = PyFloat_AsDouble(PyList_GetItem(obj, a));
@@ -672,7 +677,7 @@ int PConvPyListToIntArrayImpl(PyObject * obj, int **f, bool as_vla)
     if (as_vla) {
       (*f) = VLAlloc(int, l);
     } else {
-      (*f) = Alloc(int, l);
+      (*f) = pymol::malloc<int>(l);
     }
 
     auto strval = PyBytes_AsSomeString(obj);
@@ -690,7 +695,7 @@ int PConvPyListToIntArrayImpl(PyObject * obj, int **f, bool as_vla)
     if (as_vla) {
       (*f) = VLAlloc(int, l);
     } else {
-      (*f) = Alloc(int, l);
+      (*f) = pymol::malloc<int>(l);
     }
 
     ff = (*f);
@@ -1097,6 +1102,9 @@ PyObject *PConvStringListToPyList(int l, const char * const *str)
   return (PConvAutoNone(result));
 }
 
+/**
+ * Converts concatenated null-terminated strings to a list of strings.
+ */
 PyObject *PConvStringVLAToPyList(const char *vla)
 {
   int a, c, n = 0;
@@ -1118,6 +1126,10 @@ PyObject *PConvStringVLAToPyList(const char *vla)
   return (PConvAutoNone(result));
 }
 
+/**
+ * Concatenates a list of null-terminated strings (the null-terminator serves as
+ * the delimiter).
+ */
 int PConvPyListToStringVLA(PyObject * obj, char **vla_ptr)
 {
   int a, l, ll;
