@@ -37,6 +37,7 @@ Z* -------------------------------------------------------------------
 #include"Executive.h"
 #include"CGO.h"
 #include"Selector.h"
+#include"vla.h"
 
 void ObjectPurgeSettings(CObject * I)
 {
@@ -246,7 +247,7 @@ int ObjectMotion(CObject * I, int action, int first,
       nFrame = -nFrame;
 
     if(!I->ViewElem) {
-      I->ViewElem = VLACalloc(CViewElem, 0);
+      I->ViewElem = pymol::vla<CViewElem>(0);
     }
     
     if((action == 7) || (action == 8)) { /* toggle */
@@ -511,7 +512,7 @@ int ObjectMotion(CObject * I, int action, int first,
       if(I->ViewElem) {
         VLAFreeP(I->ViewElem);
       }
-      I->ViewElem = VLACalloc(CViewElem, 0);
+      I->ViewElem = pymol::vla<CViewElem>(0);
       break;
     case 6:                      /* uninterpolate */
       if(I->ViewElem) {
@@ -604,8 +605,15 @@ void ObjectAdjustStateRebuildRange(CObject * I, int *start, int *stop)
   }
 }
 
-void ObjectMakeValidName(char *name)
+/**
+ * Replaces invalid characters in the given object name with an underscore,
+ * or strips them if they are terminal or sequential.
+ * @param[in,out] name Object name to validate
+ * @return true if name was modified, false otherwise
+ */
+bool ObjectMakeValidName(char *name)
 {
+  bool modified = false;
   char *p = name, *q;
   if(p) {
     /* currently legal are A to Z, a to z, 0 to 9, -, _, + */
@@ -624,6 +632,7 @@ void ObjectMakeValidName(char *name)
             break;
         /* must be an ASCII-visible character */
         *p = 1;                 /* placeholder for non-printable */
+        modified = true;
       }
       p++;
     }
@@ -656,22 +665,47 @@ void ObjectMakeValidName(char *name)
       p++;
     }
   }
+  return modified;
 }
 
-/*
- * Strip invalid characters from `name`, and if `name` equals a reserved
+/**
+ * Replaces invalid characters in `name` with an underscore,
+ * or strips them if they are terminal or sequential - if `name` equals a reserved
  * selection keyword, then also append an underscore.
+ *
+ * @param[in,out] name Object name to validate
+ * @param quiet If false, print warnings if the name gets modified.
  */
-void ObjectMakeValidName(PyMOLGlobals * G, char *name)
+void ObjectMakeValidName(PyMOLGlobals * G, char *name, bool quiet)
 {
-  ObjectMakeValidName(name);
+  if (ObjectMakeValidName(name) && !quiet) {
+    PRINTFB(G, FB_Executive, FB_Warnings)
+      " Warning: Invalid characters in '%s' have been replaced or stripped\n",
+      name ENDFB(G);
+  }
 
   if (SelectorNameIsKeyword(G, name)) {
-    PRINTFB(G, FB_Executive, FB_Warnings)
+    if (!quiet) {
+      PRINTFB(G, FB_Executive, FB_Warnings)
       " Warning: '%s' is a reserved keyword, appending underscore\n", name
       ENDFB(G);
+    }
     strcat(name, "_");
-  } else if (strcmp(name, "protein") == 0 || strcmp(name, "nucleic") == 0) {
+    return;
+  }
+
+  static bool once_protein = false;
+  static bool once_nucleic = false;
+
+  if (!once_protein && strcmp(name, "protein") == 0) {
+    once_protein = true;
+  } else if (!once_nucleic && strcmp(name, "nucleic") == 0) {
+    once_nucleic = true;
+  } else {
+    return;
+  }
+
+  {
     // Warn the user if "protein" or "nucleic" are used as names, but
     // don't modify the name (yet).
     PRINTFB(G, FB_Executive, FB_Warnings)
@@ -680,8 +714,52 @@ void ObjectMakeValidName(PyMOLGlobals * G, char *name)
   }
 }
 
+/**
+ * Get a pointer to an object state.
+ * @param state State (0-indexed) or -2/-3 for current state
+ * @return NULL if state is out of bounds or empty
+ */
+CObjectState* CObject::getObjectState(int state)
+{
+  if (state == -2 /* cSelectorUpdateTableCurrentState */ ||
+      state == -3 /* cSelectorUpdateTableEffectiveStates */) {
+    state = getCurrentState();
+  }
+  if (state < 0 || state >= getNFrame()) {
+    return nullptr;
+  }
+  return _getObjectState(state);
+}
+
+/**
+ * Get the effective state (0-indexed) of an object, based on the `state` and
+ * `static_singletons` settings. Will not validate the value of the `state`
+ * setting, it could be `<0` or `>=getNFrame()`.
+ */
+int CObject::getCurrentState() const
+{
+  if (getNFrame() == 1 &&
+      SettingGet<bool>(G, Setting, nullptr, cSetting_static_singletons))
+    return 0;
+  return SettingGet<int>(G, Setting, nullptr, cSetting_state) - 1;
+}
+
+/**
+ * Like CObject::getCurrentState() but will return `-1` if the `all_states`
+ * setting is set.
+ *
+ * Note: Clamps negative values at `-1` (all states). The usefulness of this
+ * should be questioned, in particular with `ignore_all_states=true` a caller
+ * is likely to discard all negative values, including -1.
+ *
+ * @param ignore_all_states Boolean flag, should be false. You most likely
+ * should use CObject::getCurrentState() instead of setting `ignore_all_states`
+ * to true.
+ */
 int ObjectGetCurrentState(CObject * I, int ignore_all_states)
 {
+  assert("use CObject::getCurrentState()" && !ignore_all_states);
+
   // the previous implementation (up to PyMOL 1.7.6) ignored
   // object-level state=0 (all states)
 
@@ -689,15 +767,7 @@ int ObjectGetCurrentState(CObject * I, int ignore_all_states)
       SettingGet_b(I->G, I->Setting, NULL, cSetting_all_states))
     return -1;
 
-  if (I->getNFrame() == 1 &&
-      SettingGet_b(I->G, I->Setting, NULL, cSetting_static_singletons))
-    return 0;
-
-  int state = SettingGet_i(I->G, I->Setting, NULL, cSetting_state) - 1;
-
-  if(state < -1)
-    state = -1;
-  return (state);
+  return std::max(-1, I->getCurrentState());
 }
 
 PyObject *ObjectAsPyList(CObject * I)
@@ -820,7 +890,7 @@ int ObjectCopyHeader(CObject * I, const CObject * src)
 
 
 /*========================================================================*/
-void ObjectCombineTTT(CObject * I, float *ttt, int reverse_order, int store)
+void ObjectCombineTTT(CObject * I, const float *ttt, int reverse_order, int store)
 {
   if(I->type == cObjectGroup) {
     ExecutiveGroupCombineTTT(I->G, I, ttt, reverse_order,store);
@@ -841,7 +911,7 @@ void ObjectCombineTTT(CObject * I, float *ttt, int reverse_order, int store)
       store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
-        I->ViewElem = VLACalloc(CViewElem, 0);
+        I->ViewElem = pymol::vla<CViewElem>(0);
       if(I->ViewElem) { /* update motion path waypoint, if active */
         int frame = SceneGetFrame(I->G);
         if(frame >= 0) {
@@ -854,7 +924,7 @@ void ObjectCombineTTT(CObject * I, float *ttt, int reverse_order, int store)
   }
 }
 /*========================================================================*/
-void ObjectTranslateTTT(CObject * I, float *v, int store)
+void ObjectTranslateTTT(CObject * I, const float *v, int store)
 {
   if(I->type == cObjectGroup) {
     ExecutiveGroupTranslateTTT(I->G, I, v, store);
@@ -872,7 +942,7 @@ void ObjectTranslateTTT(CObject * I, float *v, int store)
       store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
-        I->ViewElem = VLACalloc(CViewElem, 0);
+        I->ViewElem = pymol::vla<CViewElem>(0);
       if(I->ViewElem) { /* update motion path waypoint, if active */
         int frame = SceneGetFrame(I->G);
         if(frame >= 0) {
@@ -901,7 +971,7 @@ void ObjectSetTTT(CObject * I, const float *ttt, int state, int store)
       store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
-        I->ViewElem = VLACalloc(CViewElem, 0);
+        I->ViewElem = pymol::vla<CViewElem>(0);
       if(I->ViewElem) { /* update motion path waypoint, if active */
         int frame = SceneGetFrame(I->G);
         if(frame >= 0) {
@@ -942,7 +1012,7 @@ void ObjectResetTTT(CObject * I,int store)
     store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
   if(store && MovieDefined(I->G)) {
     if(!I->ViewElem)  
-      I->ViewElem = VLACalloc(CViewElem, 0);
+      I->ViewElem = pymol::vla<CViewElem>(0);
     if(I->ViewElem) { /* update motion path waypoint, if active */
       int frame = SceneGetFrame(I->G);
       if(frame >= 0) {
@@ -957,6 +1027,15 @@ void ObjectResetTTT(CObject * I,int store)
 
 
 /*========================================================================*/
+/**
+ * Get the combined transformation of TTT and state matrix. State matrix is
+ * only included if `history=true` or `matrix_mode > 0`.
+ *
+ * @param state See CObject::getObjectState
+ * @param history Boolean flag
+ * @param[out] matrix Homogeneous 4x4 matrix
+ * @return True if `matrix` was populated
+ */
 int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
 {
   int result = false;
@@ -965,15 +1044,17 @@ int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
     result = true;
   }
 
-  {
-    int use_matrices = SettingGet_i(I->G, I->Setting, NULL, cSetting_matrix_mode);
-    if(use_matrices<0) use_matrices = 0;
-    if(use_matrices || history) {
-      if(I->fGetObjectState) {
-        CObjectState *obj_state = I->fGetObjectState(I, state);
-        if(obj_state) {
-          double *state_matrix = obj_state->Matrix;
-          if(state_matrix) {
+  if (!history) {
+    history =
+        SettingGet<int>(I->G, I->Setting, nullptr, cSetting_matrix_mode) > 0;
+  }
+
+  if (history) {
+    {
+      CObjectState* obj_state = I->getObjectState(state);
+      if (obj_state) {
+          if(!obj_state->Matrix.empty()) {
+            const double *state_matrix = obj_state->Matrix.data();
             if(result) {
               right_multiply44d44d(matrix, state_matrix);
             } else {
@@ -981,7 +1062,6 @@ int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
             }
             result = true;
           }
-        }
       }
     }
   }
@@ -1094,15 +1174,14 @@ void ObjectSetTTTOrigin(CObject * I, float *origin)
 
 
 /*========================================================================*/
-static
-CSetting **ObjectGetSettingHandle(CObject * I, int state)
+CSetting **CObject::getSettingHandle(int state)
 {
-  return (&I->Setting);
+  return &Setting;
 }
 
 
 /*========================================================================*/
-static void ObjectDescribeElement(CObject * I, int index, char *buffer)
+void CObject::describeElement(int index, char* buffer) const
 {
   buffer[0] = 0;
 }
@@ -1148,37 +1227,10 @@ void ObjectSetName(CObject * I, const char *name)
 
 
 /*========================================================================*/
-static
-void ObjectUpdate(CObject * I)
+CObject::~CObject()
 {
-
-}
-
-
-/*========================================================================*/
-void ObjectPurge(CObject * I)
-{
-  if(I) {
-    SceneObjectDel(I->G, I, false);
-    SettingFreeP(I->Setting);
-    VLAFreeP(I->ViewElem);
-  }
-}
-
-
-/*========================================================================*/
-void ObjectFree(CObject * I)
-{
-  if(I)
-    ObjectPurge(I);
-}
-
-
-/*========================================================================*/
-static
-int ObjectGetNFrames(CObject * I)
-{
-  return 1;
+  SceneObjectDel(this->G, this, false);
+  SettingFreeP(this->Setting);
 }
 
 
@@ -1200,16 +1252,11 @@ void ObjectUseColorCGO(CGO *cgo, CObject * I)
 }
 
 /*========================================================================*/
-static void ObjectInvalidate(CObject * this_, int rep, int level, int state)
+/**
+ * Render a unit box (dummy representation)
+ */
+void CObject::render(RenderInfo * info)
 {
-
-}
-
-
-/*========================================================================*/
-static void ObjectRenderUnitBox(CObject * this_, RenderInfo * info)
-{
-  PyMOLGlobals *G = this_->G;
   if(G->HaveGUI && G->ValidContext) {
 #ifdef PURE_OPENGL_ES_2
     /* TODO */
@@ -1243,131 +1290,84 @@ static void ObjectRenderUnitBox(CObject * this_, RenderInfo * info)
 
 
 /*========================================================================*/
-void ObjectInit(PyMOLGlobals * G, CObject * I)
+CObject::CObject(PyMOLGlobals * G) : G(G)
 {
-  UtilZeroMem(I, sizeof(CObject));
-
-  I->G = G;
-  I->fFree = ObjectFree;
-  I->fRender = ObjectRenderUnitBox;
-  I->fUpdate = ObjectUpdate;
-  I->fGetNFrame = ObjectGetNFrames;
-  I->fDescribeElement = ObjectDescribeElement;
-  I->fGetSettingHandle = ObjectGetSettingHandle;
-  I->fInvalidate = ObjectInvalidate;
-
-  /*
-     I->fGetCaption = NULL;
-     I->fGetObjectState = NULL;
-     I->Name[0]=0;
-     I->Color=0;
-     I->ExtentFlag=false;
-     I->Setting=NULL;
-     I->TTTFlag=false;
-     I->Enabled=false;
-     zero3f(I->ExtentMin);
-     zero3f(I->ExtentMax);
-     for(a=0;a<16;a++) I->TTT[a]=0.0F;
-     I->Context=0;
-     I->ViewElem = NULL;
-   */
-
   OrthoRemoveSplash(G);         /* HMM... this seems like an inappropriate sideeffect */
-  I->visRep = cRepBitmask & ~(cRepCellBit | cRepExtentBit);
+  visRep = cRepBitmask & ~(cRepCellBit | cRepExtentBit);
 }
 
+/*========================================================================*/
+
 void ObjectStateInit(PyMOLGlobals * G, CObjectState * I)
 {
   I->G = G;
-  I->Matrix = NULL;
-  I->InvMatrix = NULL;
-}
-
-/* ObjectStateCopy -- deep copy the State struct from src to dst */
-void ObjectStateCopy(CObjectState * dst, const CObjectState * src)
-{
-  /* State is a ptr to globals and an array of matrices. */
-  /* Shallow cpy; good enough for G */
-  *dst = *src;
-  /* deep copy matrices if necessary */
-  if(src->Matrix) {
-    dst->Matrix = Alloc(double, 16);
-    if(dst->Matrix) {
-      copy44d(src->Matrix, dst->Matrix);
-    }
-  }
-  dst->InvMatrix = NULL;
 }
 
 void ObjectStatePurge(CObjectState * I)
 {
-  FreeP(I->Matrix);
-  FreeP(I->InvMatrix);
 }
 
 int ObjectStateSetMatrix(CObjectState * I, double *matrix)
 {
   int ok = true;
   if(matrix) {
-    if(!I->Matrix)
-      I->Matrix = Alloc(double, 16);
-    CHECKOK(ok, I->Matrix);
-    if(I->Matrix) {
-      copy44d(matrix, I->Matrix);
-    }
-  } else if(I->Matrix) {
-    FreeP(I->Matrix);
-    I->Matrix = NULL;
+    I->Matrix.resize(16);
+    copy44d(matrix, I->Matrix.data());
+  } else {
+    I->Matrix.clear();
   }
-  FreeP(I->InvMatrix);
+  I->InvMatrix.clear();
   return ok;
 }
 
 void ObjectStateRightCombineMatrixR44d(CObjectState * I, double *matrix)
 {
   if(matrix) {
-    if(!I->Matrix) {
-      I->Matrix = Alloc(double, 16);
-      copy44d(matrix, I->Matrix);
+    if(I->Matrix.empty()) {
+      I->Matrix = std::vector<double>(16);
+      copy44d(matrix, I->Matrix.data());
     } else {
-      right_multiply44d44d(I->Matrix, matrix);
+      right_multiply44d44d(I->Matrix.data(), matrix);
     }
   }
-  FreeP(I->InvMatrix);
+  I->InvMatrix.clear();
 }
 
 void ObjectStateLeftCombineMatrixR44d(CObjectState * I, double *matrix)
 {
   if(matrix) {
-    if(!I->Matrix) {
-      I->Matrix = Alloc(double, 16);
-      copy44d(matrix, I->Matrix);
+    if(I->Matrix.empty()) {
+      I->Matrix = std::vector<double>(16);
+      copy44d(matrix, I->Matrix.data());
     } else {
-      left_multiply44d44d(matrix, I->Matrix);
+      left_multiply44d44d(matrix, I->Matrix.data());
     }
   }
-  FreeP(I->InvMatrix);
+  I->InvMatrix.clear();
 }
 
 void ObjectStateCombineMatrixTTT(CObjectState * I, float *matrix)
 {
 
   if(matrix) {
-    if(!I->Matrix) {
-      I->Matrix = Alloc(double, 16);
-      convertTTTfR44d(matrix, I->Matrix);
+    if(I->Matrix.empty()) {
+      I->Matrix = std::vector<double>(16);
+      convertTTTfR44d(matrix, I->Matrix.data());
     } else {
       double tmp[16];
       convertTTTfR44d(matrix, tmp);
-      right_multiply44d44d(I->Matrix, tmp);
+      right_multiply44d44d(I->Matrix.data(), tmp);
     }
   }
-  FreeP(I->InvMatrix);
+  I->InvMatrix.clear();
 }
 
 double *ObjectStateGetMatrix(CObjectState * I)
 {
-  return I->Matrix;
+  if(!I->Matrix.empty()) {
+    return I->Matrix.data();
+  }
+  return nullptr;
 }
 
 /*
@@ -1375,31 +1375,34 @@ double *ObjectStateGetMatrix(CObjectState * I)
  */
 double *ObjectStateGetInvMatrix(CObjectState * I)
 {
-  if(I->Matrix && !I->InvMatrix) {
-    I->InvMatrix = Alloc(double, 16);
-    xx_matrix_invert(I->InvMatrix, I->Matrix, 4);
+  if(!I->Matrix.empty() && I->InvMatrix.empty()) {
+    I->InvMatrix = std::vector<double>(16);
+    xx_matrix_invert(I->InvMatrix.data(), I->Matrix.data(), 4);
   }
-  return I->InvMatrix;
+  return I->InvMatrix.data();
 }
 
 void ObjectStateTransformMatrix(CObjectState * I, double *matrix)
 {
-  if(!I->Matrix) {
-    I->Matrix = Alloc(double, 16);
-    if(I->Matrix) {
-      copy44d(matrix, I->Matrix);
+  if(I->Matrix.empty()) {
+    I->Matrix = std::vector<double>(16);
+    if(!I->Matrix.empty()) {
+      copy44d(matrix, I->Matrix.data());
     }
   } else {
-    right_multiply44d44d(I->Matrix, matrix);
+    right_multiply44d44d(I->Matrix.data(), matrix);
   }
-  FreeP(I->InvMatrix);
+  I->InvMatrix.clear();
 }
 
 int ObjectStatePushAndApplyMatrix(CObjectState * I, RenderInfo * info)
 {
   PyMOLGlobals *G = I->G;
   float matrix[16];
-  double *i_matrix = I->Matrix;
+  const double *i_matrix = nullptr;
+  if(!I->Matrix.empty()) {
+    i_matrix = I->Matrix.data();
+  }
   int result = false;
   if(i_matrix) {
     if(info->ray) {
@@ -1455,10 +1458,10 @@ void ObjectStatePopMatrix(CObjectState * I, RenderInfo * info)
   }
 }
 
-void ObjectStateResetMatrix(CObjectState * I)
+void ObjectStateResetMatrix(CObjectState* I)
 {
-  FreeP(I->Matrix);
-  FreeP(I->InvMatrix);
+  I->Matrix.clear();
+  I->InvMatrix.clear();
 }
 
 PyObject *ObjectStateAsPyList(CObjectState * I)
@@ -1468,8 +1471,8 @@ PyObject *ObjectStateAsPyList(CObjectState * I)
   if(I) {
     result = PyList_New(1);
 
-    if(I->Matrix) {
-      PyList_SetItem(result, 0, PConvDoubleArrayToPyList(I->Matrix, 16));
+    if(!I->Matrix.empty()) {
+      PyList_SetItem(result, 0, PConvDoubleArrayToPyList(I->Matrix.data(), 16));
     } else {
       PyList_SetItem(result, 0, PConvAutoNone(Py_None));
     }
@@ -1494,7 +1497,7 @@ int ObjectStateFromPyList(PyMOLGlobals * G, PyObject * list, CObjectState * I)
     if(ok) {
       tmp = PyList_GetItem(list, 0);
       if(tmp != Py_None)
-        ok = PConvPyListToDoubleArray(tmp, &I->Matrix);
+        ok = PConvFromPyObject(G, tmp, I->Matrix);
     }
   }
   return (ok);
