@@ -38,6 +38,7 @@ Z* -------------------------------------------------------------------
 #include"Setting.h"
 #include"Executive.h"
 #include "Lex.h"
+#include "pymol/zstring_view.h"
 
 #include <map>
 
@@ -175,12 +176,13 @@ void BondTypeInit2(BondType *bond, int i1, int i2, int order)
   bond->index[1] = i2;
   bond->order = order;
   bond->id = -1;
+  bond->stereo = 0;
 }
 
 int AtomInfoInit(PyMOLGlobals * G)
 {
   CAtomInfo *I = NULL;
-  if((I = (G->AtomInfo = Calloc(CAtomInfo, 1)))) {
+  if((I = (G->AtomInfo = pymol::calloc<CAtomInfo>(1)))) {
     AtomInfoPrimeColors(G);
     I->NextUniqueID = 1;
     return 1;
@@ -1156,10 +1158,22 @@ void AtomInfoCombine(PyMOLGlobals * G, AtomInfoType * dst, AtomInfoType&& src_,
 
 
 /*========================================================================*/
+/**
+ * Make atom names in `atInfo1` unique w.r.t.\ `atInfo0` (and to `atInfo1` itself).
+ * @param atInfo0 List of reference atoms
+ * @param n0 Size of atInfo0 list
+ * @param atInfo1 List of atoms which need to be made unique
+ * @param flag1 Optional whitelist mask for `atInfo1` or NULL
+ * @param n1 Size of atInfo1 list
+ * @param mol Optional reference molecule to limit to atoms with coordinates
+ * @return Number of renamed atoms
+ */
 int AtomInfoUniquefyNames(PyMOLGlobals * G, const AtomInfoType * atInfo0, int n0,
-                          AtomInfoType * atInfo1, int *flag1, int n1)
+                          AtomInfoType * atInfo1, int *flag1, int n1,
+                          const ObjectMolecule * mol)
 {
   /* makes sure all names in atInfo1 are unique WRT 0 and 1 */
+  auto ignore_case = SettingGet<bool>(G, cSetting_ignore_case);
 
   /* tricky optimizations to avoid n^2 dependence in this operation */
   int result = 0;
@@ -1205,7 +1219,7 @@ int AtomInfoUniquefyNames(PyMOLGlobals * G, const AtomInfoType * atInfo0, int n0
 
       ai0 = atInfo1 + st1;
       for(a = st1; a <= nd1; a++) {
-        if(!WordMatchExact(G, ai1->name, ai0->name, true))
+        if(!WordMatchExact(G, ai1->name, ai0->name, ignore_case))
           ai0++;
         else if(!AtomInfoSameResidue(G, ai1, ai0))
           ai0++;
@@ -1221,27 +1235,20 @@ int AtomInfoUniquefyNames(PyMOLGlobals * G, const AtomInfoType * atInfo0, int n0
       if(atInfo0) {
         /* check within object 2 */
 
-        if(!lai0)
-          bracketFlag = true;
-        else if(!AtomInfoSameResidue(G, lai0, ai1))
-          bracketFlag = true;
-        else
-          bracketFlag = false;
-        if(bracketFlag) {
+        if (!lai0 || !AtomInfoSameResidue(G, lai0, ai1)) {
           AtomInfoBracketResidue(G, atInfo0, n0, ai1, &st0, &nd0);
           lai0 = ai1;
         }
-        ai0 = atInfo0 + st0;
-        for(a = st0; a <= nd0; a++) {
-          if(!WordMatchExact(G, ai1->name, ai0->name, true))
-            ai0++;
-          else if(!AtomInfoSameResidue(G, ai1, ai0))
-            ai0++;
-          else if(ai1 != ai0) {
+
+        for (a = st0; a <= nd0; ++a) {
+          ai0 = atInfo0 + a;
+
+          if (WordMatchExact(G, ai1->name, ai0->name, ignore_case) &&
+              AtomInfoSameResidue(G, ai1, ai0) && ai1 != ai0 &&
+              (!mol || mol->atomHasAnyCoordinates(a))) {
             matchFlag = true;
             break;
-          } else
-            ai0++;
+          }
         }
       }
     }
@@ -1266,6 +1273,19 @@ int AtomInfoUniquefyNames(PyMOLGlobals * G, const AtomInfoType * atInfo0, int n0
   return result;
 }
 
+/**
+ * Make atom names in `atoms` unique w.r.t.\ atoms-with-coordinates in `mol`.
+ * @param mol Reference molecule
+ * @param atoms List of atoms which need to be made unique
+ * @param natoms Size of atoms list
+ * @return Number of renamed atoms
+ */
+int AtomInfoUniquefyNames(
+    const ObjectMolecule* mol, AtomInfoType* atoms, size_t natoms)
+{
+  return AtomInfoUniquefyNames(
+      mol->G, mol->AtomInfo, mol->NAtom, atoms, nullptr, natoms, mol);
+}
 
 /*========================================================================*/
 void AtomInfoBracketResidue(PyMOLGlobals * G, const AtomInfoType * ai0, int n0,
@@ -2124,7 +2144,7 @@ int AtomInfoSameResidue(PyMOLGlobals * G, const AtomInfoType * at1, const AtomIn
       at1->discrete_state == at2->discrete_state &&
       at1->inscode == at2->inscode &&
       at1->segi == at2->segi &&
-      WordMatchExact(G, at1->resn, at2->resn, true));
+      WordMatchExact(G, at1->resn, at2->resn, SettingGet<bool>(G, cSetting_ignore_case)));
 }
 
 int AtomInfoSameResidueP(PyMOLGlobals * G, const AtomInfoType * at1, const AtomInfoType * at2)
@@ -2350,7 +2370,7 @@ int AtomInfoGetExpectedValence(PyMOLGlobals * G, const AtomInfoType * I)
 static int get_protons(const char * symbol)
 {
   char titleized[4];
-  static std::map<const char *, int, cstrless_t> lookup;
+  static std::map<pymol::zstring_view, int> lookup;
 
   if (lookup.empty()) {
     for (int i = 0; i < ElementTableSize; i++)
@@ -2479,8 +2499,14 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
     case 'Q':
       *(e + 1) = 0;
       break;
+    case 'p':
+      if (p_strstartswith(n, "pseudo")) {
+        strcpy(e, "PS");
+      }
     }
-    if(*(e + 1) && (e[1] != 'P' || e[0] != 'L'))
+    if(*(e + 1) &&
+        (/* e != "LP" */ e[1] != 'P' || e[0] != 'L') &&
+        (/* e != "PS" */ e[1] != 'S' || e[0] != 'P'))
       *(e + 1) = tolower(*(e + 1));
   }
 
