@@ -81,11 +81,11 @@ int CoordSetValidateRefPos(CoordSet * I)
     VLACheck(I->RefPos, RefPosType, I->NIndex);
     return true;
   } else {
-    int ok = true && (I->RefPos = VLACalloc(RefPosType, I->NIndex));
+    int ok = true && (I->RefPos = pymol::vla<RefPosType>(I->NIndex));
     if(ok) {
       int a;
       for(a = 0; a < I->NIndex; a++) {
-        float *src = I->Coord + 3 * a;
+        const float* src = I->coordPtr(a);
         copy3f(src, I->RefPos[a].coord);
         I->RefPos[a].specified = true;
       }
@@ -160,13 +160,22 @@ int CoordSetFromPyList(PyMOLGlobals * G, PyObject * list, CoordSet ** cs)
     if(ok)
       ok = PConvPyListToIntVLA(PyList_GetItem(list, 3), &I->IdxToAtm);
     if(ok && (ll > 5))
-      ok = PConvPyStrToStr(PyList_GetItem(list, 5), I->Name, sizeof(WordType));
-    if(ok && (ll > 6))
-      ok = ObjectStateFromPyList(G, PyList_GetItem(list, 6), &I->State);
-    if(ok && (ll > 7))
-      I->Setting = SettingNewFromPyList(G, PyList_GetItem(list, 7));
-    if(ok && (ll > 8))
-      ok = PConvPyListToLabPosVLA(PyList_GetItem(list, 8), &I->LabPos);
+      ok = CPythonVal_PConvPyStrToStr_From_List(G, list, 5, I->Name, sizeof(WordType));
+    if(ok && (ll > 6)){
+      CPythonVal *val = CPythonVal_PyList_GetItem(G, list, 6);
+      ok = ObjectStateFromPyList(G, val, I);
+      CPythonVal_Free(val);
+    }
+    if(ok && (ll > 7)){
+      CPythonVal *val = CPythonVal_PyList_GetItem(G, list, 7);
+      I->Setting = SettingNewFromPyList(G, val);
+      CPythonVal_Free(val);
+    }
+    if(ok && (ll > 8)){
+      CPythonVal *val = CPythonVal_PyList_GetItem(G, list, 8);
+      ok = CPythonVal_PConvPyListToLabPosVLA(G, val, &I->LabPos);
+      CPythonVal_Free(val);
+    }
 
 #ifdef _PYMOL_IP_PROPERTIES
 #endif
@@ -186,8 +195,8 @@ int CoordSetFromPyList(PyMOLGlobals * G, PyObject * list, CoordSet ** cs)
 	CPythonVal *val = CPythonVal_PyList_GetItem(G, list, 11);
 	if (!CPythonVal_IsNone(val)){
 	  int a;
-	  I->atom_state_setting_id = VLACalloc(int, I->NIndex);
-	  I->has_atom_state_settings = VLACalloc(char, I->NIndex);
+	  I->atom_state_setting_id = pymol::vla<int>(I->NIndex);
+	  I->has_atom_state_settings = pymol::vla<char>(I->NIndex);
 	  for (a=0; a<I->NIndex; a++){
 	    CPythonVal *val2 = CPythonVal_PyList_GetItem(G, val, a);
 	    if (!CPythonVal_IsNone(val2)){
@@ -233,8 +242,8 @@ int CoordSetFromPyList(PyMOLGlobals * G, PyObject * list, CoordSet ** cs)
 PyObject *CoordSetAsNumPyArray(CoordSet * cs, short copy)
 {
 #ifndef _PYMOL_NUMPY
-  PRINTFB(cs->State.G, FB_CoordSet, FB_Errors)
-    "No numpy support\n" ENDFB(cs->State.G);
+  PRINTFB(cs->G, FB_CoordSet, FB_Errors)
+    "No numpy support\n" ENDFB(cs->G);
   return NULL;
 #else
 
@@ -261,7 +270,7 @@ PyObject *CoordSetAsNumPyArray(CoordSet * cs, short copy)
     if((result = PyArray_SimpleNew(2, dims, typenum)))
       memcpy(PyArray_DATA((PyArrayObject *)result), cs->Coord, cs->NIndex * 3 * base_size);
   } else {
-    result = PyArray_SimpleNewFromData(2, dims, typenum, cs->Coord);
+    result = PyArray_SimpleNewFromData(2, dims, typenum, cs->Coord.data());
   }
 
   return result;
@@ -273,8 +282,9 @@ PyObject *CoordSetAsPyList(CoordSet * I)
   PyObject *result = NULL;
 
   if(I) {
-    int pse_export_version = SettingGetGlobal_f(I->State.G, cSetting_pse_export_version) * 1000;
-    bool dump_binary = SettingGetGlobal_b(I->State.G, cSetting_pse_binary_dump) && (!pse_export_version || pse_export_version >= 1765);
+    auto G = I->G;
+    int pse_export_version = SettingGet<float>(G, cSetting_pse_export_version) * 1000;
+    bool dump_binary = SettingGet<bool>(G, cSetting_pse_binary_dump) && (!pse_export_version || pse_export_version >= 1765);
     result = PyList_New(12);
     PyList_SetItem(result, 0, PyInt_FromLong(I->NIndex));
     PyList_SetItem(result, 1, PyInt_FromLong(I->NAtIndex));
@@ -286,7 +296,7 @@ PyObject *CoordSetAsPyList(CoordSet * I)
     else
       PyList_SetItem(result, 4, PConvAutoNone(NULL));
     PyList_SetItem(result, 5, PyString_FromString(I->Name));
-    PyList_SetItem(result, 6, ObjectStateAsPyList(&I->State));
+    PyList_SetItem(result, 6, ObjectStateAsPyList(I));
     PyList_SetItem(result, 7, SettingAsPyList(I->Setting));
     PyList_SetItem(result, 8, PConvLabPosVLAToPyList(I->LabPos, I->NIndex));
 
@@ -329,9 +339,6 @@ void CoordSetAdjustAtmIdx(CoordSet * I, int *lookup, int nAtom)
   char *new_has_atom_state_settings_by_atom = NULL;
   int *new_atom_state_setting_id_by_atom = NULL;
   int nIndex = I->NIndex;
-  PRINTFD(I->State.G, FB_CoordSet)
-    " CoordSetAdjustAtmIdx-Debug: entered NAtIndex: %d NIndex %d\n I->AtmToIdx %p\n",
-    I->NAtIndex, I->NIndex, (void *) I->AtmToIdx ENDFD;
 
   if (I->has_atom_state_settings){
     new_has_atom_state_settings_by_atom = VLACalloc(char, nIndex);
@@ -344,7 +351,7 @@ void CoordSetAdjustAtmIdx(CoordSet * I, int *lookup, int nAtom)
     if (a0 < 0){
       if (I->has_atom_state_settings){
 	if (I->has_atom_state_settings[a]){
-	  SettingUniqueDetachChain(I->State.G, I->atom_state_setting_id[a]);
+	  SettingUniqueDetachChain(I->G, I->atom_state_setting_id[a]);
 	  I->has_atom_state_settings[a] = 0;
 	  I->atom_state_setting_id[a] = 0;
 	}
@@ -378,12 +385,20 @@ void CoordSetAdjustAtmIdx(CoordSet * I, int *lookup, int nAtom)
     VLAFreeP(new_has_atom_state_settings_by_atom);
     VLAFreeP(new_atom_state_setting_id_by_atom);
   }
-  PRINTFD(I->State.G, FB_CoordSet)
+  PRINTFD(I->G, FB_CoordSet)
     " CoordSetAdjustAtmIdx-Debug: leaving... NAtIndex: %d NIndex %d\n",
     I->NAtIndex, I->NIndex ENDFD;
 
 }
 
+CoordSet* CoordSetCopy(const CoordSet* src)
+{
+  if(!src) {
+    return nullptr;
+  }
+  return new CoordSet(*src);
+}
+
 
 /*========================================================================*/
 int CoordSetMerge(ObjectMolecule *OM, CoordSet * I, CoordSet * cs)
@@ -409,13 +424,13 @@ int CoordSetMerge(ObjectMolecule *OM, CoordSet * I, CoordSet * cs)
       } else {
 	I->AtmToIdx[cs->IdxToAtm[a]] = i0;
       }
-      copy3f(cs->Coord + a * 3, I->Coord + i0 * 3);
+      copy3f(cs->coordPtr(a), I->coordPtr(i0));
     }
   }
   if (ok){
     if(cs->LabPos) {
       if(!I->LabPos)
-	I->LabPos = VLACalloc(LabPosType, nIndex);
+	I->LabPos = pymol::vla<LabPosType>(nIndex);
       else
 	VLACheck(I->LabPos, LabPosType, nIndex);
       if(I->LabPos) {
@@ -428,7 +443,7 @@ int CoordSetMerge(ObjectMolecule *OM, CoordSet * I, CoordSet * cs)
   if (ok){
     if(cs->RefPos) {
       if(!I->RefPos)
-	I->RefPos = VLACalloc(RefPosType, nIndex);
+	I->RefPos = pymol::vla<RefPosType>(nIndex);
       else
 	VLACheck(I->RefPos, RefPosType, nIndex);
       if(I->RefPos) {
@@ -450,6 +465,7 @@ void CoordSetPurge(CoordSet * I)
 
 /* performs first half of removal  */
 {
+  auto G = I->G;
   int offset = 0;
   int a, a1, ao;
   AtomInfoType *ai;
@@ -461,14 +477,14 @@ void CoordSetPurge(CoordSet * I)
   char *has_atom_state0, *has_atom_state1;
   obj = I->Obj;
 
-  PRINTFD(I->State.G, FB_CoordSet)
+  PRINTFD(G, FB_CoordSet)
     " CoordSetPurge-Debug: entering..." ENDFD;
 
-  c0 = c1 = I->Coord;
-  r0 = r1 = I->RefPos;
-  l0 = l1 = I->LabPos;
-  atom_state0 = atom_state1 = I->atom_state_setting_id;
-  has_atom_state0 = has_atom_state1 = I->has_atom_state_settings;
+  c0 = c1 = I->Coord.data();
+  r0 = r1 = I->RefPos.data();
+  l0 = l1 = I->LabPos.data();
+  atom_state0 = atom_state1 = I->atom_state_setting_id.data();
+  has_atom_state0 = has_atom_state1 = I->has_atom_state_settings.data();
 
   /* This loop slides down the atoms that are not deleted (deleteFlag)
      it moves the Coord, RefPos, and LabPos */
@@ -541,13 +557,16 @@ void CoordSetPurge(CoordSet * I)
       VLASize(I->atom_state_setting_id, int, I->NIndex);
     }
     VLASize(I->IdxToAtm, int, I->NIndex);
-    PRINTFD(I->State.G, FB_CoordSet)
+    PRINTFD(G, FB_CoordSet)
       " CoordSetPurge-Debug: I->IdxToAtm shrunk to %d\n", I->NIndex ENDFD;
     I->invalidateRep(cRepAll, cRepInvAtoms);      /* this will free Color */
   }
-  PRINTFD(I->State.G, FB_CoordSet)
+  PRINTFD(G, FB_CoordSet)
     " CoordSetPurge-Debug: leaving NAtIndex %d NIndex %d...\n",
     I->NAtIndex, I->NIndex ENDFD;
+
+#ifdef _PYMOL_IP_PROPERTIES
+#endif
 }
 
 
@@ -555,12 +574,10 @@ void CoordSetPurge(CoordSet * I)
 int CoordSetTransformAtomTTTf(CoordSet * I, int at, const float *TTT)
 {
   int a1 = I->atmToIdx(at);
-  float *v1;
-
   if(a1 < 0)
     return false;
 
-  v1 = I->Coord + 3 * a1;
+  auto* v1 = I->coordPtr(a1);
   MatrixTransformTTTfN3f(1, v1, TTT, v1);
   return true;
 }
@@ -570,12 +587,10 @@ int CoordSetTransformAtomTTTf(CoordSet * I, int at, const float *TTT)
 int CoordSetTransformAtomR44f(CoordSet * I, int at, const float *matrix)
 {
   int a1 = I->atmToIdx(at);
-  float *v1;
-
   if(a1 < 0)
     return false;
 
-  v1 = I->Coord + 3 * a1;
+  auto* v1 = I->coordPtr(a1);
   MatrixTransformR44fN3f(1, v1, matrix, v1);
   return true;
 }
@@ -592,7 +607,7 @@ void CoordSetRecordTxfApplied(CoordSet * I, const float *matrix, int homogenous)
     convert44f44d(matrix, temp);
   }
 
-  ObjectStateLeftCombineMatrixR44d(&I->State, temp);
+  ObjectStateLeftCombineMatrixR44d(I, temp);
 }
 
 
@@ -600,12 +615,10 @@ void CoordSetRecordTxfApplied(CoordSet * I, const float *matrix, int homogenous)
 int CoordSetMoveAtom(CoordSet * I, int at, const float *v, int mode)
 {
   int a1 = I->atmToIdx(at);
-  float *v1;
-
   if(a1 < 0)
     return false;
 
-  v1 = I->Coord + 3 * a1;
+  auto* v1 = I->coordPtr(a1);
   if(mode) {
     add3f(v, v1, v1);
   } else {
@@ -618,6 +631,7 @@ int CoordSetMoveAtom(CoordSet * I, int at, const float *v, int mode)
 /*========================================================================*/
 int CoordSetMoveAtomLabel(CoordSet * I, int at, const float *v, const float *diff)
 {
+  auto G = I->G;
   ObjectMolecule *obj = I->Obj;
   int a1 = I->atmToIdx(at);
   int result = 0;
@@ -630,10 +644,10 @@ int CoordSetMoveAtomLabel(CoordSet * I, int at, const float *v, const float *dif
     int at_label_relative_mode = 0;
     AtomInfoType *ai = obj->AtomInfo + at;
 
-    AtomStateGetSetting_i(I->State.G, obj, I, a1, ai, cSetting_label_relative_mode, &at_label_relative_mode);
+    AtomStateGetSetting_i(G, obj, I, a1, ai, cSetting_label_relative_mode, &at_label_relative_mode);
     switch (at_label_relative_mode){
     case 0:
-      AtomStateGetSetting(I->State.G, obj, I, a1, ai, cSetting_label_placement_offset, &at_offset_ptr);
+      AtomStateGetSetting(G, obj, I, a1, ai, cSetting_label_placement_offset, &at_offset_ptr);
       add3f(v, at_offset_ptr, at_offset);
       SettingSet(cSetting_label_placement_offset, at_offset, I, a1);
       break;
@@ -642,7 +656,7 @@ int CoordSetMoveAtomLabel(CoordSet * I, int at, const float *v, const float *dif
       {
 	float voff[3];
 	int width, height;
-	SceneGetWidthHeight(I->State.G, &width, &height);
+	SceneGetWidthHeight(G, &width, &height);
 	if (at_label_relative_mode==1){
 	  voff[0] = 2.f * diff[0] / width;
 	  voff[1] = 2.f * diff[1] / height;
@@ -651,7 +665,7 @@ int CoordSetMoveAtomLabel(CoordSet * I, int at, const float *v, const float *dif
 	  voff[1] = diff[1];
 	}
 	voff[2] = 0.f;
-	AtomStateGetSetting(I->State.G, obj, I, a1, ai, cSetting_label_screen_point, &at_offset_ptr);
+	AtomStateGetSetting(G, obj, I, a1, ai, cSetting_label_screen_point, &at_offset_ptr);
 	add3f(voff, at_offset_ptr, at_offset);
 	SettingSet(cSetting_label_screen_point, at_offset, I, a1);
       }
@@ -664,20 +678,20 @@ int CoordSetMoveAtomLabel(CoordSet * I, int at, const float *v, const float *dif
 
 
 /*========================================================================*/
-int CoordSetGetAtomVertex(CoordSet * I, int at, float *v)
+int CoordSetGetAtomVertex(const CoordSet * I, int at, float *v)
 {
   int a1 = I->atmToIdx(at);
 
   if(a1 < 0)
     return false;
 
-  copy3f(I->Coord + 3 * a1, v);
+  copy3f(I->coordPtr(a1), v);
   return true;
 }
 
 
 /*========================================================================*/
-int CoordSetGetAtomTxfVertex(CoordSet * I, int at, float *v)
+int CoordSetGetAtomTxfVertex(const CoordSet * I, int at, float *v)
 {
   ObjectMolecule *obj = I->Obj;
   int a1 = I->atmToIdx(at);
@@ -685,18 +699,17 @@ int CoordSetGetAtomTxfVertex(CoordSet * I, int at, float *v)
   if(a1 < 0)
     return false;
 
-  copy3f(I->Coord + 3 * a1, v);
+  copy3f(I->coordPtr(a1), v);
 
   /* apply state transformation */
-  if(I->State.Matrix && (SettingGet_i(I->State.G,
-          obj->Obj.Setting, I->Setting,
-          cSetting_matrix_mode) > 0)) {
-    transform44d3f(I->State.Matrix, v, v);
+  if (!I->Matrix.empty() && SettingGet<int>(I->G, obj->Setting, I->Setting,
+                                cSetting_matrix_mode) > 0) {
+    transform44d3f(I->Matrix.data(), v, v);
   }
 
   /* object transformation */
-  if(obj->Obj.TTTFlag) {
-    transformTTT44f3f(obj->Obj.TTT, v, v);
+  if(obj->TTTFlag) {
+    transformTTT44f3f(obj->TTT, v, v);
   }
 
   return true;
@@ -711,7 +724,7 @@ int CoordSetSetAtomVertex(CoordSet * I, int at, const float *v)
   if(a1 < 0)
    return false;
 
-  copy3f(v, I->Coord + 3 * a1);
+  copy3f(v, I->coordPtr(a1));
   return true;
 }
 
@@ -720,8 +733,7 @@ int CoordSetSetAtomVertex(CoordSet * I, int at, const float *v)
 void CoordSetRealToFrac(CoordSet * I, const CCrystal * cryst)
 {
   int a;
-  float *v;
-  v = I->Coord;
+  float* v = I->Coord.data();
   for(a = 0; a < I->NIndex; a++) {
     transform33f3f(cryst->RealToFrac, v, v);
     v += 3;
@@ -733,8 +745,7 @@ void CoordSetRealToFrac(CoordSet * I, const CCrystal * cryst)
 void CoordSetTransform44f(CoordSet * I, const float *mat)
 {
   int a;
-  float *v;
-  v = I->Coord;
+  float* v = I->Coord.data();
   for(a = 0; a < I->NIndex; a++) {
     transform44f3f(mat, v, v);
     v += 3;
@@ -747,8 +758,7 @@ void CoordSetTransform44f(CoordSet * I, const float *mat)
 void CoordSetTransform33f(CoordSet * I, const float *mat)
 {
   int a;
-  float *v;
-  v = I->Coord;
+  float* v = I->Coord.data();
   for(a = 0; a < I->NIndex; a++) {
     transform33f3f(mat, v, v);
     v += 3;
@@ -757,13 +767,12 @@ void CoordSetTransform33f(CoordSet * I, const float *mat)
 
 
 /*========================================================================*/
-void CoordSetGetAverage(CoordSet * I, float *v0)
+void CoordSetGetAverage(const CoordSet * I, float *v0)
 {
   int a;
-  float *v;
   double accum[3];
   if(I->NIndex) {
-    v = I->Coord;
+    const float* v = I->Coord.data();
     accum[0] = *(v++);
     accum[1] = *(v++);
     accum[2] = *(v++);
@@ -783,8 +792,7 @@ void CoordSetGetAverage(CoordSet * I, float *v0)
 void CoordSetFracToReal(CoordSet * I, const CCrystal * cryst)
 {
   int a;
-  float *v;
-  v = I->Coord;
+  float* v = I->Coord.data();
   for(a = 0; a < I->NIndex; a++) {
     transform33f3f(cryst->FracToReal, v, v);
     v += 3;
@@ -813,7 +821,7 @@ bool CoordSetInsureOrthogonal(PyMOLGlobals * G,
     return false;
 
   if (!cryst)
-    cryst = cset->Symmetry->Crystal;
+    cryst = &cset->Symmetry->Crystal;
 
   const float * r2f = cryst->RealToFrac;
 
@@ -851,12 +859,12 @@ bool CoordSetInsureOrthogonal(PyMOLGlobals * G,
 }
 
 /*========================================================================*/
-static char RotateU(const double *matrix, float *anisou)
 /* Rotates the ANISOU vector
  *
  * matrix: flat 4x4, but only rotation (upper left 3x3) is considered
  * anisou: has 6 elements (of symmetric 3x3) and will be rotated in-place
  */
+bool RotateU(const double *matrix, float *anisou)
 {
   int i, j, k;
   float Re[3][3];
@@ -964,7 +972,7 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals * G, char **charVLA, int *c,
       char *atomline = (*charVLA) + (*c);
       char *anisoline = atomline + linelen;
       float anisou[6];
-      memcpy(anisou, ai->anisou, 6 * sizeof(float));
+      std::copy_n(ai->anisou, 6, anisou);
 
       if(matrix && !RotateU(matrix, anisou)) {
         PRINTFB(G, FB_CoordSet, FB_Errors) "RotateU failed\n" ENDFB(G);
@@ -1027,7 +1035,7 @@ PyObject *CoordSetAtomToChemPyAtom(PyMOLGlobals * G, AtomInfoType * ai, const fl
     float tmp_array[6] = { 0.f, 0.f, 0.f, 0.f, 0.f, 0.f };
 
     if (ai->anisou) {
-      memcpy(tmp_array, ai->anisou, 6 * sizeof(float));
+      std::copy_n(ai->anisou, 6, tmp_array);
       if (matrix)
       RotateU(matrix, tmp_array);
     }
@@ -1089,6 +1097,10 @@ PyObject *CoordSetAtomToChemPyAtom(PyMOLGlobals * G, AtomInfoType * ai, const fl
     PConvIntToPyObjAttr(atom, "flags", ai->flags);
     PConvIntToPyObjAttr(atom, "id", ai->id);    /* not necc. unique */
     PConvIntToPyObjAttr(atom, "index", index + 1);      /* fragile */
+
+#ifdef _PYMOL_IP_PROPERTIES
+#endif
+
   }
   if(PyErr_Occurred())
     PyErr_Print();
@@ -1109,7 +1121,7 @@ void CoordSet::invalidateRep(int type, int level)
   /* graphical representations need redrawing */
   if(level == cRepInvVisib) {
     /* cartoon_side_chain_helper */
-    if(SettingGet_b(I->State.G, I->Setting, I->Obj->Obj.Setting,
+    if(SettingGet<bool>(G, I->Setting, I->Obj->Setting,
                     cSetting_cartoon_side_chain_helper)) {
       if((type == cRepCyl) || (type == cRepLine) || (type == cRepSphere))
         invalidateRep(cRepCartoon, cRepInvVisib2);
@@ -1120,7 +1132,7 @@ void CoordSet::invalidateRep(int type, int level)
       }
     }
     /* ribbon_side_chain_helper */
-    if(SettingGet_b(I->State.G, I->Setting, I->Obj->Obj.Setting,
+    if(SettingGet<bool>(G, I->Setting, I->Obj->Setting,
                     cSetting_ribbon_side_chain_helper)) {
       if((type == cRepCyl) || (type == cRepLine) || (type == cRepSphere))
         invalidateRep(cRepRibbon, cRepInvVisib2);
@@ -1131,7 +1143,7 @@ void CoordSet::invalidateRep(int type, int level)
       }
     }
     /* line_stick helper  */
-    if(SettingGet_b(I->State.G, I->Setting, I->Obj->Obj.Setting,
+    if(SettingGet<bool>(G, I->Setting, I->Obj->Setting,
                     cSetting_line_stick_helper)) {
       if(type == cRepCyl)
         invalidateRep(cRepLine, cRepInvVisib2);
@@ -1141,14 +1153,15 @@ void CoordSet::invalidateRep(int type, int level)
     }
   }
 
-  if(I->Spheroid)
-    if(I->NSpheroid != I->NAtIndex * I->SpheroidSphereSize) {
-      FreeP(I->Spheroid);
-      FreeP(I->SpheroidNormal);
+  if(!I->Spheroid.empty())
+    if (I->Spheroid.size() !=
+        I->NAtIndex * GetSpheroidSphereRec(G)->nDot) {
+      I->Spheroid.clear();
+      I->SpheroidNormal.clear();
     }
 
   /* invalidate basd on one representation, 'type' */
-  for (RepIterator iter(I->State.G, type); iter.next(); ){
+  for (RepIterator iter(G, type); iter.next(); ){
     int eff_level = level;
     a = iter.rep;
     if(level == cRepInvPick) {
@@ -1179,8 +1192,8 @@ void CoordSet::invalidateRep(int type, int level)
   if(level >= cRepInvCoord) {   /* if coordinates change, then this map becomes invalid */
     MapFree(I->Coord2Idx);
     I->Coord2Idx = NULL;
-    ExecutiveInvalidateSelectionIndicatorsCGO(I->State.G);
-    SceneInvalidatePicking(I->State.G);
+    ExecutiveInvalidateSelectionIndicatorsCGO(G);
+    SceneInvalidatePicking(G);
     /* invalidate distances */
   }
 
@@ -1193,7 +1206,7 @@ void CoordSet::invalidateRep(int type, int level)
   }
 #endif
 
-  SceneChanged(I->State.G);
+  SceneChanged(G);
 }
 
 
@@ -1214,7 +1227,7 @@ void CoordSet::invalidateRep(int type, int level)
          I->Rep[rep] = I->Rep[rep]->fUpdate(I->Rep[rep],I,state,rep);\
     }\
   }\
-OrthoBusyFast(I->State.G,rep,cRepCnt);\
+OrthoBusyFast(G,rep,cRepCnt);\
 }
 
 /*========================================================================*/
@@ -1222,10 +1235,10 @@ void CoordSet::update(int state)
 {
   CoordSet * I = this;
   int a;
-  PyMOLGlobals *G = I->Obj->Obj.G;
+  assert(G == I->Obj->G);
 
   PRINTFB(G, FB_CoordSet, FB_Blather) " CoordSetUpdate-Entered: object %s state %d cset %p\n",
-    I->Obj->Obj.Name, state, (void *) I
+    I->Obj->Name, state, (void *) I
     ENDFB(G);
 
   OrthoBusyFast(G, 0, cRepCnt);
@@ -1250,7 +1263,7 @@ void CoordSet::update(int state)
   OrthoBusyFast(G, 1, 1);
   if(Feedback(G, FB_CoordSet, FB_Blather)) {
     printf(" CoordSetUpdate-Leaving: object %s state %d cset %p\n",
-           I->Obj->Obj.Name, state, (void *) I);
+           I->Obj->Name, state, (void *) I);
   }
 }
 
@@ -1271,7 +1284,7 @@ void CoordSetUpdateCoord2IdxMap(CoordSet * I, float cutoff)
     if(I->NIndex && (!I->Coord2Idx)) {  /* NOTE: map based on stored coords */
       I->Coord2IdxReq = cutoff;
       I->Coord2IdxDiv = cutoff * 1.25F;
-      I->Coord2Idx = MapNew(I->State.G, I->Coord2IdxDiv, I->Coord, I->NIndex, NULL);
+      I->Coord2Idx = MapNew(I->G, I->Coord2IdxDiv, I->Coord, I->NIndex, NULL);
       if(I->Coord2IdxDiv < I->Coord2Idx->Div)
         I->Coord2IdxDiv = I->Coord2Idx->Div;
     }
@@ -1305,12 +1318,11 @@ static int RepToTransparencySetting(const int rep_id){
 void CoordSet::render(RenderInfo * info)
 {
   CoordSet * I = this;
-  PyMOLGlobals *G = I->State.G;
   PRINTFD(G, FB_CoordSet)
     " CoordSetRender: entered (%p).\n", (void *) I ENDFD;
 
   if(!(info->ray || info->pick) &&
-     (SettingGet_i(G, I->Setting, I->Obj->Obj.Setting,
+     (SettingGet_i(G, I->Setting, I->Obj->Setting,
                    cSetting_defer_builds_mode) == 5)) {
     if(!info->pass) {
       ObjectUseColor((CObject *) I->Obj);
@@ -1332,13 +1344,13 @@ void CoordSet::render(RenderInfo * info)
     int a, aa, abit, aastart = 0, aaend = cRepCnt;
     ::Rep *r;
     int sculpt_vdw_vis_mode = SettingGet_i(G, I->Setting,
-					   I->Obj->Obj.Setting,
+					   I->Obj->Setting,
 					   cSetting_sculpt_vdw_vis_mode);
     if((!pass) && sculpt_vdw_vis_mode && 
-       I->SculptCGO && (I->Obj->Obj.visRep & cRepCGOBit)) {
+       I->SculptCGO && (I->Obj->visRep & cRepCGOBit)) {
       if(ray) {
         int ok = CGORenderRay(I->SculptCGO, ray, info,
-			      ColorGet(G, I->Obj->Obj.Color), NULL, I->Setting, I->Obj->Obj.Setting);
+			      ColorGet(G, I->Obj->Color), NULL, I->Setting, I->Obj->Setting);
 	if (!ok){
 	  CGOFree(I->SculptCGO);
 	  CGOFree(I->SculptShaderCGO);
@@ -1361,10 +1373,10 @@ void CoordSet::render(RenderInfo * info)
 	  }
 	  if (I->SculptShaderCGO){
 	    CGORenderGL(I->SculptShaderCGO, NULL,
-			I->Setting, I->Obj->Obj.Setting, info, NULL);
+			I->Setting, I->Obj->Setting, info, NULL);
 	  } else {
 	    CGORenderGL(I->SculptCGO, NULL,
-			I->Setting, I->Obj->Obj.Setting, info, NULL);
+			I->Setting, I->Obj->Setting, info, NULL);
 	  }
         }
       }
@@ -1372,7 +1384,7 @@ void CoordSet::render(RenderInfo * info)
 
     if (pick){
       int pick_labels = SettingGet_i(G, I->Setting,
-				     I->Obj->Obj.Setting,
+				     I->Obj->Setting,
 				     cSetting_pick_labels);
       if (pick_labels == 2){ // only pick labels
 	aastart = cRepLabel;
@@ -1398,10 +1410,10 @@ void CoordSet::render(RenderInfo * info)
           if(I->Obj)
             ray->wobble(
                          SettingGet_i(G, I->Setting,
-                                      I->Obj->Obj.Setting,
+                                      I->Obj->Setting,
                                       cSetting_ray_texture),
                          SettingGet_3fv(G, I->Setting,
-                                        I->Obj->Obj.Setting,
+                                        I->Obj->Setting,
                                         cSetting_ray_texture_settings));
           else
             ray->wobble(
@@ -1409,7 +1421,7 @@ void CoordSet::render(RenderInfo * info)
                                       NULL, cSetting_ray_texture),
                          SettingGet_3fv(G, I->Setting, NULL,
                                         cSetting_ray_texture_settings));
-          ray->color3fv(ColorGet(G, I->Obj->Obj.Color));
+          ray->color3fv(ColorGet(G, I->Obj->Color));
         }
 
         if(r->fRender) {        /* do OpenGL rendering in three passes */
@@ -1435,7 +1447,7 @@ void CoordSet::render(RenderInfo * info)
             case cRepLabel:
               {
                 int t_mode_3 = SettingGetGlobal_i(G, cSetting_transparency_mode) == 3;
-                if (pass == -1 || (t_mode_3 && pass == 1))
+                if (pass == -1 || (t_mode_3 && pass == 1 /* TODO_OPENVR 0 */))
                   r->fRender(r, info);
               }
               break;
@@ -1507,62 +1519,50 @@ void CoordSet::render(RenderInfo * info)
 
 
 /*========================================================================*/
-CoordSet *CoordSetNew(PyMOLGlobals * G)
+CoordSet::CoordSet(PyMOLGlobals* G)
+    : CObjectState(G)
 {
-  OOCalloc(G, CoordSet);        /* NULL-initializes all fields */
-  ObjectStateInit(G, &I->State);
-  I->State.G = G;
-  I->PeriodicBoxType = cCSet_NoPeriodicity;
-
-  I->SpheroidSphereSize = I->State.G->Sphere->Sphere[1]->nDot;  /* does this make any sense? */
-
-  return (I);
 }
 
 /*========================================================================*/
-CoordSet *CoordSetCopy(const CoordSet * cs)
+CoordSet::CoordSet(const CoordSet& cs)
+    : CObjectState(cs)
 {
-  if (!cs)
-    return NULL;
-
-  PyMOLGlobals * G = const_cast<PyMOLGlobals*>(cs->State.G);
-
-  /* OOAlloc declares and defines, I:
-   * I = ... */
-  OOCalloc(G, CoordSet);
-  /* shallow copy */
-  (*I) = (*cs);                 /* NOTE: must deep-copy all pointers in this struct */
-  /* deep copy state struct */
-  ObjectStateCopy(&I->State, &cs->State);
+  this->Obj = cs.Obj;
+  this->Coord = cs.Coord;
+  this->IdxToAtm = cs.IdxToAtm;
+  this->NIndex = cs.NIndex;
+  this->NAtIndex = cs.NAtIndex;
+  this->prevNIndex = cs.prevNIndex;
+  this->prevNAtIndex = cs.prevNAtIndex;
+  std::copy(std::begin(cs.Rep), std::end(cs.Rep), std::begin(this->Rep));
+  std::copy(std::begin(cs.Active), std::end(cs.Active), std::begin(this->Active));
+  this->NTmpBond = cs.NTmpBond;
+  this->NTmpLinkBond = cs.NTmpLinkBond;
   /* deep copy & return ptr to new symmetry */
-  I->Symmetry = SymmetryCopy(cs->Symmetry);
-
-  if(I->PeriodicBox)
-    I->PeriodicBox = CrystalCopy(I->PeriodicBox);
+  if(cs.Symmetry) {
+    this->Symmetry = pymol::make_unique<CSymmetry>(*cs.Symmetry);
+  }
+  if(cs.PeriodicBox) {
+    this->PeriodicBox = pymol::make_unique<CCrystal>(*cs.PeriodicBox);
+  }
+  std::copy(std::begin(cs.Name), std::end(cs.Name), std::begin(this->Name));
+  this->PeriodicBoxType = cs.PeriodicBoxType;
+  this->tmp_index = cs.tmp_index;
+  this->Coord2IdxReq = cs.Coord2IdxReq;
+  this->Coord2IdxDiv = cs.Coord2IdxDiv;
+  this->objMolOpInvalidated = cs.objMolOpInvalidated;
 
   // copy VLAs
-  I->Coord      = VLACopy2(cs->Coord);
-  I->LabPos     = VLACopy2(cs->LabPos);
-  I->RefPos     = VLACopy2(cs->RefPos);
-  I->AtmToIdx   = VLACopy2(cs->AtmToIdx);
-  I->IdxToAtm   = VLACopy2(cs->IdxToAtm);
+  this->LabPos     = cs.LabPos;
+  this->RefPos     = cs.RefPos;
+  this->AtmToIdx   = cs.AtmToIdx;
 
-  UtilZeroMem(I->Rep, sizeof(::Rep *) * cRepCnt);
+  UtilZeroMem(this->Rep, sizeof(::Rep *) * cRepCnt);
 
 #ifdef _PYMOL_IP_PROPERTIES
 #endif
 
-  I->Setting = NULL;
-  I->atom_state_setting_id = NULL;
-  I->has_atom_state_settings = NULL;
-  I->SculptCGO = NULL;
-  I->SculptShaderCGO = NULL;
-  I->TmpLinkBond = NULL;
-  I->TmpBond = NULL;
-  I->Spheroid = NULL;
-  I->SpheroidNormal = NULL;
-  I->Coord2Idx = NULL;
-  return (I);
 }
 
 
@@ -1577,8 +1577,7 @@ int CoordSet::extendIndices(int nAtom)
     ok = obj->setNDiscrete(nAtom);
 
     if(I->AtmToIdx) {           /* convert to discrete if necessary */
-      VLAFree(I->AtmToIdx);
-      I->AtmToIdx = NULL;
+      I->AtmToIdx.freeP();
       if (ok){
 	for(a = 0; a < I->NIndex; a++) {
 	  b = I->IdxToAtm[a];
@@ -1598,7 +1597,7 @@ int CoordSet::extendIndices(int nAtom)
       }
       I->NAtIndex = nAtom;
     } else if(!obj->DiscreteFlag) {
-      I->AtmToIdx = VLACalloc(int, nAtom);
+      I->AtmToIdx = pymol::vla<int>(nAtom);
       CHECKOK(ok, I->AtmToIdx);
       if (ok){
 	for(a = 0; a < nAtom; a++)
@@ -1618,9 +1617,9 @@ void CoordSet::appendIndices(int offset)
   int a, b;
   ObjectMolecule *obj = I->Obj;
 
-  I->IdxToAtm = VLACalloc(int, I->NIndex);
+  I->IdxToAtm = pymol::vla<int>(I->NIndex);
   if(I->NIndex) {
-    ErrChkPtr(I->State.G, I->IdxToAtm);
+    ErrChkPtr(G, I->IdxToAtm);
     for(a = 0; a < I->NIndex; a++)
       I->IdxToAtm[a] = a + offset;
   }
@@ -1633,9 +1632,9 @@ void CoordSet::appendIndices(int offset)
       obj->DiscreteCSet[b] = I;
     }
   } else {
-    I->AtmToIdx = VLACalloc(int, I->NIndex + offset);
+    I->AtmToIdx = pymol::vla<int>(I->NIndex + offset);
     if(I->NIndex + offset) {
-      ErrChkPtr(I->State.G, I->AtmToIdx);
+      ErrChkPtr(G, I->AtmToIdx);
       for(a = 0; a < offset; a++)
         I->AtmToIdx[a] = -1;
       for(a = 0; a < I->NIndex; a++)
@@ -1652,11 +1651,11 @@ void CoordSet::enumIndices()
   CoordSet * I = this;
   /* set up for simple case where 1 = 1, etc. */
   int a;
-  I->AtmToIdx = VLACalloc(int, I->NIndex);
-  I->IdxToAtm = VLACalloc(int, I->NIndex);
+  I->AtmToIdx = pymol::vla<int>(I->NIndex);
+  I->IdxToAtm = pymol::vla<int>(I->NIndex);
   if(I->NIndex) {
-    ErrChkPtr(I->State.G, I->AtmToIdx);
-    ErrChkPtr(I->State.G, I->IdxToAtm);
+    ErrChkPtr(G, I->AtmToIdx);
+    ErrChkPtr(G, I->IdxToAtm);
     for(a = 0; a < I->NIndex; a++) {
       I->AtmToIdx[a] = a;
       I->IdxToAtm[a] = a;
@@ -1667,7 +1666,7 @@ void CoordSet::enumIndices()
 
 
 /*========================================================================*/
-void CoordSet::fFree()
+CoordSet::~CoordSet()
 {
   CoordSet * I = this;
   int a;
@@ -1678,12 +1677,10 @@ void CoordSet::fFree()
 
     if (I->has_atom_state_settings){
       for(a = 0; a < I->NIndex; a++){
-	if (I->has_atom_state_settings[a]){
-	  SettingUniqueDetachChain(I->State.G, I->atom_state_setting_id[a]);
-	}
+        if (I->has_atom_state_settings[a]){
+          SettingUniqueDetachChain(G, I->atom_state_setting_id[a]);
+        }
       }
-      VLAFreeP(I->has_atom_state_settings);
-      VLAFreeP(I->atom_state_setting_id);
     }
     for(a = 0; a < cRepCnt; a++)
       if(I->Rep[a])
@@ -1695,26 +1692,15 @@ void CoordSet::fFree()
           obj->DiscreteAtmToIdx[I->IdxToAtm[a]] = -1;
           obj->DiscreteCSet[I->IdxToAtm[a]] = NULL;
         }
-    VLAFreeP(I->AtmToIdx);
-    VLAFreeP(I->IdxToAtm);
     MapFree(I->Coord2Idx);
-    VLAFreeP(I->Coord);
-    VLAFreeP(I->TmpBond);
-    if(I->Symmetry)
-      SymmetryFree(I->Symmetry);
-    if(I->PeriodicBox)
-      CrystalFree(I->PeriodicBox);
-    FreeP(I->Spheroid);
-    FreeP(I->SpheroidNormal);
     SettingFreeP(I->Setting);
-    ObjectStatePurge(&I->State);
     CGOFree(I->SculptCGO);
-    VLAFreeP(I->LabPos);
-    VLAFreeP(I->RefPos);
-    /* free and make null */
-    OOFreeP(I);
   }
 }
+void CoordSet::fFree()
+{
+  delete this;
+}
 void LabPosTypeCopy(const LabPosType * src, LabPosType * dst){
   dst->mode = src->mode;
   copy3f(src->pos, dst->pos);
@@ -1764,10 +1750,10 @@ PyObject *SettingGetIfDefinedPyObject(PyMOLGlobals * G, CoordSet *cs, int at, in
 
 int CoordSetCheckUniqueID(PyMOLGlobals * G, CoordSet *I, int at){
   if (!I->atom_state_setting_id){
-    I->atom_state_setting_id = VLACalloc(int, I->NIndex);
+    I->atom_state_setting_id = pymol::vla<int>(I->NIndex);
   }
   if (!I->has_atom_state_settings){
-    I->has_atom_state_settings = VLACalloc(char, I->NIndex);
+    I->has_atom_state_settings = pymol::vla<char>(I->NIndex);
   }
   if (!I->atom_state_setting_id[at]){
     I->atom_state_setting_id[at] = AtomInfoGetNewUniqueID(G);
@@ -1785,7 +1771,7 @@ void AtomStateGetSetting(ATOMSTATEGETSETTINGARGS, V * out) {
   if (AtomSettingGetIfDefined(G, ai, setting_id, out))
     return;
 
-  *out = SettingGet<V>(G, cs->Setting, obj->Obj.Setting, setting_id);
+  *out = SettingGet<V>(G, cs->Setting, obj->Setting, setting_id);
 }
 
 template void AtomStateGetSetting(ATOMSTATEGETSETTINGARGS, int * out);
