@@ -101,8 +101,9 @@ static int ObjectCGOAllStatesFromPyList(ObjectCGO * I, PyObject * list, int vers
     ok = PyList_Check(list);
   if(ok) {
     for(a = 0; a < I->NState; a++) {
+      auto *val = PyList_GetItem(list, a);
       ok =
-        ObjectCGOStateFromPyList(I->Obj.G, I->State + a, PyList_GetItem(list, a),
+        ObjectCGOStateFromPyList(I->G, I->State + a, val,
                                  version);
       if(!ok)
         break;
@@ -123,12 +124,14 @@ int ObjectCGONewFromPyList(PyMOLGlobals * G, PyObject * list, ObjectCGO ** resul
   if(ok)
     ok = PyList_Check(list);
 
-  I = ObjectCGONew(G);
+  I = new ObjectCGO(G);
   if(ok)
     ok = (I != NULL);
 
-  if(ok)
-    ok = ObjectFromPyList(G, PyList_GetItem(list, 0), &I->Obj);
+  if(ok){
+    auto *val = PyList_GetItem(list, 0);
+    ok = ObjectFromPyList(G, val, I);
+  }
   if(ok)
     ok = PConvPyIntToInt(PyList_GetItem(list, 1), &I->NState);
   if(ok)
@@ -147,7 +150,7 @@ PyObject *ObjectCGOAsPyList(ObjectCGO * I)
   PyObject *result = NULL;
 
   result = PyList_New(3);
-  PyList_SetItem(result, 0, ObjectAsPyList(&I->Obj));
+  PyList_SetItem(result, 0, ObjectAsPyList(I));
   PyList_SetItem(result, 1, PyInt_FromLong(I->NState));
   PyList_SetItem(result, 2, ObjectCGOAllStatesAsPyList(I));
 
@@ -157,16 +160,14 @@ PyObject *ObjectCGOAsPyList(ObjectCGO * I)
 
 /*========================================================================*/
 
-void ObjectCGOFree(ObjectCGO * I)
+ObjectCGO::~ObjectCGO()
 {
-  int a;
-  for(a = 0; a < I->NState; a++) {
+  auto I = this;
+  for(int a = 0; a < I->NState; a++) {
     CGOFree(I->State[a].renderCGO);
     CGOFree(I->State[a].origCGO);
   }
   VLAFreeP(I->State);
-  ObjectPurge(&I->Obj);
-  OOFreeP(I);
 }
 
 
@@ -188,11 +189,11 @@ void ObjectCGORecomputeExtent(ObjectCGO * I)
       if(CGOGetExtent(cgo, mn, mx)) {
         if(!extent_flag) {
           extent_flag = true;
-          copy3f(mx, I->Obj.ExtentMax);
-          copy3f(mn, I->Obj.ExtentMin);
+          copy3f(mx, I->ExtentMax);
+          copy3f(mn, I->ExtentMin);
         } else {
-          max3f(mx, I->Obj.ExtentMax, I->Obj.ExtentMax);
-          min3f(mn, I->Obj.ExtentMin, I->Obj.ExtentMin);
+          max3f(mx, I->ExtentMax, I->ExtentMax);
+          min3f(mn, I->ExtentMin, I->ExtentMin);
         }
       }
       if (!has_normals && cgo && CGOHasNormals(cgo)){
@@ -200,33 +201,28 @@ void ObjectCGORecomputeExtent(ObjectCGO * I)
       }
     }
   }
-  I->Obj.ExtentFlag = extent_flag;
-  SettingCheckHandle(I->Obj.G, &I->Obj.Setting);
-  SettingSet_i(I->Obj.Setting, cSetting_cgo_lighting, has_normals);
+  I->ExtentFlag = extent_flag;
+  SettingCheckHandle(I->G, &I->Setting);
+  SettingSet_i(I->Setting, cSetting_cgo_lighting, has_normals);
 }
 
 
 /*========================================================================*/
-static void ObjectCGOInvalidate(ObjectCGO * I, int rep, int level, int state)
+void ObjectCGO::invalidate(int rep, int level, int state)
 {
+  auto I = this;
   ObjectCGOState *sobj = NULL;
 
   if(state < 0) {
     int a;
     for(a = 0; a < I->NState; a++) {
       sobj = I->State + a;
-      if (sobj->renderCGO){
-	CGOFree(sobj->renderCGO);	      
-	sobj->renderCGO = 0;
-      }
+      CGOFree(sobj->renderCGO);
     }
   } else {
     if((state >= 0) && (state < I->NState)) {
       sobj = I->State + state;
-      if (sobj->renderCGO){
-	CGOFree(sobj->renderCGO);	      
-	sobj->renderCGO = 0;
-      }
+      CGOFree(sobj->renderCGO);
     }
   }
 }
@@ -234,25 +230,20 @@ static void ObjectCGOInvalidate(ObjectCGO * I, int rep, int level, int state)
 
 /*========================================================================*/
 
-static void ObjectCGOUpdate(ObjectCGO * I)
+void ObjectCGO::update()
 {
-  int a;
-  for(a = 0; a < I->NState; a++) {
-    ObjectCGOState *ocs = I->State + a;
-    if (ocs->renderCGO){
-      CGOFree(ocs->renderCGO);	      
-      ocs->renderCGO = 0;
-    }
+  for (int a = 0; a < NState; ++a) {
+    CGOFree(State[a].renderCGO);
   }
-  SceneInvalidate(I->Obj.G);    /* needed ? */
+  SceneInvalidate(G);    /* needed ? */
 }
 
 
 /*========================================================================*/
 
-static int ObjectCGOGetNState(ObjectCGO * I)
+int ObjectCGO::getNFrame() const
 {
-  return (I->NState);
+  return NState;
 }
 
 static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
@@ -266,7 +257,7 @@ static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
         if (cgo_lighting && CGOHasAnyTriangleVerticesWithoutNormals(cgo)) {
           cgo = cgo_copy = CGOGenerateNormalsForTriangles(cgo);
         }
-        CGORenderRay(cgo, ray, info, color, ramp, I->Obj.Setting, NULL);
+        CGORenderRay(cgo, ray, info, color, ramp, I->Setting, NULL);
         CGOFree(cgo_copy);
       }
     }
@@ -277,8 +268,8 @@ static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
       if(sobj && ((sobj->hasTransparency ^ pass_is_opaque) || (sobj->hasOpaque == pass_is_opaque))){
 	{
 	  CShaderPrg *shaderPrg;
-	  int two_sided_lighting = SettingGet_i(G, I->Obj.Setting, NULL, cSetting_two_sided_lighting);
-          bool backface_cull = SettingGet_i(G, I->Obj.Setting, NULL, cSetting_backface_cull);
+	  int two_sided_lighting = SettingGet_i(G, I->Setting, NULL, cSetting_two_sided_lighting);
+          bool backface_cull = SettingGet_i(G, I->Setting, NULL, cSetting_backface_cull);
 	  if (two_sided_lighting<0){
 	    two_sided_lighting = !cgo_lighting;
 	  }
@@ -292,11 +283,11 @@ static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
 	  if (two_sided_lighting){
 	    if (use_shader)
 	      glEnable(GL_VERTEX_PROGRAM_TWO_SIDE);
-	    GLLIGHTMODELI(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+	    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
 	  } else {
 	    if (use_shader)
 	      glDisable(GL_VERTEX_PROGRAM_TWO_SIDE);
-	    GLLIGHTMODELI(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+	    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
 	  }
 #endif
 	  if (backface_cull){
@@ -311,12 +302,12 @@ static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
 	    shaderPrg->Set1i("two_sided_lighting_enabled", two_sided_lighting);
 	    sobj->renderCGO->use_shader = use_shader;
 	    sobj->renderCGO->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
-	    CGORenderGL(sobj->renderCGO, color, I->Obj.Setting, NULL, info, NULL);
+	    CGORenderGL(sobj->renderCGO, color, I->Setting, NULL, info, NULL);
 	    shaderPrg->Disable();
 	  } else {
 	    sobj->renderCGO->use_shader = use_shader;
 	    sobj->renderCGO->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
-	    CGORenderGL(sobj->renderCGO, color, I->Obj.Setting, NULL, info, NULL);
+	    CGORenderGL(sobj->renderCGO, color, I->Setting, NULL, info, NULL);
 	  }
 
 	    if (backface_cull){
@@ -326,7 +317,7 @@ static void ObjectCGORenderState(PyMOLGlobals* G, int pass, CRay* ray,
 	    if (two_sided_lighting){
 	      if (use_shader)
 		glDisable(GL_VERTEX_PROGRAM_TWO_SIDE);
-	      GLLIGHTMODELI(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+	      glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
 	    }
 	    if (!cgo_lighting){
 	      glEnable(GL_LIGHTING);
@@ -345,55 +336,48 @@ static void ObjectCGOGenerateCGO(PyMOLGlobals * G, ObjectCGO * I, ObjectCGOState
        (cgo_lighting ^ sobj->cgo_lighting))){
     // if renderWithShaders doesn't match use_shader, clear CGO and re-generate
     CGOFree(sobj->renderCGO);
-    sobj->renderCGO = 0;
   }
   if (!sobj->renderCGO){
-    CGO *convertcgo = NULL;
     float colorWithA[4];
     short someLinesWithoutNormals = 0;
-    CGO *preOpt = NULL;
     if (color){
       colorWithA[0] = color[0]; colorWithA[1] = color[1]; colorWithA[2] = color[2];
     } else {
       colorWithA[0] = 1.f; colorWithA[1] = 1.f; colorWithA[2] = 1.f;
     }
-    colorWithA[3] = 1.f - SettingGet_f(G, I->Obj.Setting, NULL, cSetting_cgo_transparency);
-    preOpt = sobj->origCGO;
+    colorWithA[3] = 1.f - SettingGet_f(G, I->Setting, NULL, cSetting_cgo_transparency);
 
-    bool hasTransparency = (colorWithA[3] < 1.f || CGOHasTransparency(preOpt));
-    bool hasOpaque = (colorWithA[3] == 1.f || CGOHasOpaque(preOpt));
+    bool hasTransparency = (colorWithA[3] < 1.f || CGOHasTransparency(sobj->origCGO));
+    bool hasOpaque = (colorWithA[3] == 1.f || CGOHasOpaque(sobj->origCGO));
 
     CGO *allCylinders = NULL;
     CGO *allSpheres = NULL;
+    std::unique_ptr<CGO> preOpt;
+
     {
-      CGO *convertcgo = NULL;
-      bool freePreOpt = false;
+      std::unique_ptr<CGO> inputWithLighting;
+      const CGO* inputCGO = sobj->origCGO;
+
       if (cgo_lighting){
-	if (CGOHasAnyTriangleVerticesWithoutNormals(preOpt)){
+	if (CGOHasAnyTriangleVerticesWithoutNormals(inputCGO)){
 	  // we only need normals if cgo_lighting is on
-	  preOpt = CGOGenerateNormalsForTriangles(preOpt);
-	  freePreOpt = true;
+          inputWithLighting.reset(CGOGenerateNormalsForTriangles(inputCGO));
+          inputCGO = inputWithLighting.get();
 	}
-	someLinesWithoutNormals = CGOHasAnyLineVerticesWithoutNormals(preOpt);
+	someLinesWithoutNormals = CGOHasAnyLineVerticesWithoutNormals(inputCGO);
 	if (!use_shader && someLinesWithoutNormals){
 	  // if some lines without normals, turn lighting off on lines
-	  CGO *preOpt2 = preOpt;
-          preOpt->use_shader = use_shader;
-	  preOpt = CGOTurnLightingOnLinesOff(preOpt);
-          CGOStop(preOpt);
-	  if (freePreOpt){
-	    CGOFree(preOpt2);
-	  }
-	  freePreOpt = true;
+          inputWithLighting.reset(CGOTurnLightingOnLinesOff(inputCGO, use_shader));
+          inputCGO = inputWithLighting.get();
 	}
       }
-      convertcgo = CGONew(G);
+
+      CGO* convertcgo = CGONew(G);
       CGOColorv(convertcgo, colorWithA);
       CGOAlpha(convertcgo, colorWithA[3]);
-      CGOAppend(convertcgo, preOpt);
-      if (freePreOpt){
-        CGOFree(preOpt);
-      }
+      CGOAppend(convertcgo, inputCGO);
+      inputWithLighting.reset();
+
       if (use_shader){
         bool t_mode_3 = SettingGetGlobal_i(G, cSetting_transparency_mode)==3;
         if ((t_mode_3 || !hasTransparency)
@@ -401,12 +385,11 @@ static void ObjectCGOGenerateCGO(PyMOLGlobals * G, ObjectCGO * I, ObjectCGOState
             && G->ShaderMgr->Get_CylinderShader(0))
         {
           if (CGOHasCylinderOperations(convertcgo)){
-            CGO *newCGO = NULL;
             allCylinders = CGONew(G);
             CGOEnable(allCylinders, GL_CYLINDER_SHADER);
-            newCGO = CGOConvertShaderCylindersToCylinderShader(convertcgo, allCylinders);
-            CGOAppendNoStop(allCylinders, newCGO);
-            CGOFreeWithoutVBOs(newCGO);
+            CGO* newCGO = CGOConvertShaderCylindersToCylinderShader(convertcgo, allCylinders);
+            allCylinders->free_append(newCGO);
+            assert(newCGO == nullptr);
             CGODisable(allCylinders, GL_CYLINDER_SHADER);
             CGOStop(allCylinders);
             CGO *allButCylinders = CGONew(G);
@@ -426,72 +409,71 @@ static void ObjectCGOGenerateCGO(PyMOLGlobals * G, ObjectCGO * I, ObjectCGOState
               CGOFree(allButSpheres);
             }
           }
-          preOpt = CGOSimplify(convertcgo, 0);
+          preOpt.reset(CGOSimplify(convertcgo, 0));
         } else {
-          preOpt = CGOSimplifyNoCompress(convertcgo, 0) ;
+          preOpt.reset(CGOSimplifyNoCompress(convertcgo, 0));
         }
       } else {
-	preOpt = CGOSimplifyNoCompress(convertcgo, 0) ;
+        preOpt.reset(CGOSimplifyNoCompress(convertcgo, 0));
       }
       CGOFree(convertcgo);
     }
 
     if (ramp){
-      convertcgo = CGOColorByRamp(G, preOpt, ramp, state, I->Obj.Setting);
-      CGOFree(preOpt);
-      preOpt = convertcgo;
+      preOpt.reset(CGOColorByRamp(G, preOpt.get(), ramp, state, I->Setting));
     }
+
+    sobj->hasTransparency = hasTransparency;
+    sobj->hasOpaque = hasOpaque;
+
     if (use_shader){
       if(preOpt && preOpt->has_begin_end){
-	CGO *convertcgo = CGOCombineBeginEnd(preOpt, 0);
-	CGOFree(preOpt);
-	preOpt = convertcgo;
+	preOpt.reset(CGOCombineBeginEnd(preOpt.get(), 0));
       }
-      sobj->hasTransparency = hasTransparency;
-      sobj->hasOpaque = hasOpaque;
-      convertcgo = CGOOptimizeToVBOIndexedWithColorEmbedTransparentInfo(preOpt, 0, colorWithA, false);
+
+      preOpt.reset(CGOOptimizeToVBOIndexedWithColorEmbedTransparentInfo(
+          preOpt.get(), 0, colorWithA, false));
+
       if (someLinesWithoutNormals){
         // if some lines without normals, turn lighting off on lines
-        convertcgo->use_shader = use_shader;
-        CGO *convertcgo2 = CGOTurnLightingOnLinesOff(convertcgo);
-        CGOStop(convertcgo2);
+        CGO* convertcgo = preOpt.release();
+        preOpt.reset(CGOTurnLightingOnLinesOff(convertcgo, use_shader));
         CGOFreeWithoutVBOs(convertcgo);
-        convertcgo = convertcgo2;
       }
+
       if (allCylinders){
-        CGOAppendNoStop(convertcgo, allCylinders);
-        CGOFreeWithoutVBOs(allCylinders);
+        preOpt->free_append(allCylinders);
       }
+
       if (allSpheres){
-        CGOAppendNoStop(convertcgo, allSpheres);
-        CGOFreeWithoutVBOs(allSpheres);
+        preOpt->free_append(allSpheres);
       }
-      CGOStop(convertcgo);
-      sobj->renderCGO = convertcgo;
+
+      sobj->renderCGO = preOpt.release();
     } else {
-      unsigned char hasTrans = CGOHasTransparency(preOpt);
-      CGOFree(convertcgo);
-      if (hasTrans){
-	CGO *convertcgo ;
-	convertcgo = CGOConvertTrianglesToAlpha(preOpt);
-	sobj->renderCGO = convertcgo;
+      assert(sobj->hasTransparency == CGOHasTransparency(preOpt.get()));
+      assert(sobj->hasOpaque == CGOHasOpaque(preOpt.get()));
+
+      if (sobj->hasTransparency) {
+        sobj->renderCGO = CGOConvertTrianglesToAlpha(preOpt.get());
 	sobj->renderCGO->render_alpha = 2;
       } else {
-	sobj->renderCGO = CGOSimplify(preOpt, 0); // copy, for now
+        sobj->renderCGO = CGOSimplify(preOpt.get(), 0);
       }
-      sobj->hasTransparency = hasTrans;
-      sobj->hasOpaque = CGOHasOpaque(preOpt);
     }
-    CGOFree(preOpt);
+
+    assert(allCylinders == nullptr);
+    assert(allSpheres == nullptr);
+
     sobj->renderWithShaders = use_shader;
     sobj->cgo_lighting = cgo_lighting;
   }
 }
 /*========================================================================*/
 
-static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
+void ObjectCGO::render(RenderInfo * info)
 {
-  PyMOLGlobals *G = I->Obj.G;
+  auto I = this;
   int state = info->state;
   CRay *ray = info->ray;
   int pass = info->pass;
@@ -502,18 +484,18 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
   
   use_shader = SettingGetGlobal_b(G, cSetting_cgo_use_shader) &
     SettingGetGlobal_b(G, cSetting_use_shaders);
-  cgo_lighting = SettingGet_i(G, I->Obj.Setting, NULL, cSetting_cgo_lighting);
+  cgo_lighting = SettingGet_i(G, I->Setting, NULL, cSetting_cgo_lighting);
 
-  ObjectPrepareContext(&I->Obj, info);
-  ramp = ColorGetRamp(G, I->Obj.Color);
-  color = ColorGet(G, I->Obj.Color);
+  ObjectPrepareContext(I, info);
+  ramp = ColorGetRamp(G, I->Color);
+  color = ColorGet(G, I->Color);
 
   if(!I->State)
     return;
 
   if(pass || info->ray) {
-    if((I->Obj.visRep & cRepCGOBit)) {
-      for(StateIterator iter(G, I->Obj.Setting, state, I->NState); iter.next();) {
+    if((I->visRep & cRepCGOBit)) {
+      for(StateIterator iter(G, I->Setting, state, I->NState); iter.next();) {
         sobj = I->State + iter.state;
         if (!sobj->origCGO)
           continue;
@@ -527,23 +509,10 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 
 
 /*========================================================================*/
-ObjectCGO *ObjectCGONew(PyMOLGlobals * G)
+ObjectCGO::ObjectCGO(PyMOLGlobals * G) : CObject(G)
 {
-  OOAlloc(G, ObjectCGO);
-
-  ObjectInit(G, (CObject *) I);
-
-  I->State = VLACalloc(ObjectCGOState, 10);
-  I->NState = 0;
-  I->Obj.type = cObjectCGO;
-  I->Obj.fFree = (void (*)(CObject *)) ObjectCGOFree;
-  I->Obj.fUpdate = (void (*)(CObject *)) ObjectCGOUpdate;
-  I->Obj.fInvalidate = (void (*)(CObject *, int rep, int level, int state))
-    ObjectCGOInvalidate;
-  I->Obj.fRender = (void (*)(CObject *, RenderInfo *)) ObjectCGORender;
-  I->Obj.fGetNFrame = (int (*)(CObject *)) ObjectCGOGetNState;
-
-  return (I);
+  State = VLACalloc(ObjectCGOState, 10);
+  type = cObjectCGO;
 }
 
 
@@ -606,11 +575,11 @@ ObjectCGO *ObjectCGOFromCGO(PyMOLGlobals * G, ObjectCGO * obj, CGO * cgo, int st
   ObjectCGO *I = NULL;
 
   if(obj) {
-    if(obj->Obj.type != cObjectCGO)     /* TODO: handle this */
+    if(obj->type != cObjectCGO)     /* TODO: handle this */
       obj = NULL;
   }
   if(!obj) {
-    I = ObjectCGONew(G);
+    I = new ObjectCGO(G);
   } else {
     I = obj;
   }
@@ -662,11 +631,11 @@ ObjectCGO *ObjectCGODefine(PyMOLGlobals * G, ObjectCGO * obj, PyObject * pycgo,
   int est;
 
   if(obj) {
-    if(obj->Obj.type != cObjectCGO)     /* TODO: handle this */
+    if(obj->type != cObjectCGO)     /* TODO: handle this */
       obj = NULL;
   }
   if(!obj) {
-    I = ObjectCGONew(G);
+    I = new ObjectCGO(G);
   } else {
     I = obj;
   }
@@ -716,11 +685,11 @@ ObjectCGO *ObjectCGOFromFloatArray(PyMOLGlobals * G, ObjectCGO * obj,
   int est;
 
   if(obj) {
-    if(obj->Obj.type != cObjectCGO)     /* TODO: handle this */
+    if(obj->type != cObjectCGO)     /* TODO: handle this */
       obj = NULL;
   }
   if(!obj) {
-    I = ObjectCGONew(G);
+    I = new ObjectCGO(G);
   } else {
     I = obj;
   }
