@@ -34,10 +34,10 @@ Z* -------------------------------------------------------------------
 #include"Feedback.h"
 #include"DistSet.h"
 #include"ListMacros.h"
+#ifdef _PYMOL_INCENTIVE
+#endif
 
 static void ObjectDistFree(ObjectDist * I);
-static void ObjectDistUpdate(ObjectDist * I);
-static int ObjectDistGetNFrames(ObjectDist * I);
 static void ObjectDistUpdateExtents(ObjectDist * I);
 
 int ObjectDistGetLabelTxfVertex(ObjectDist * I, int state, int index, float *v)
@@ -45,15 +45,15 @@ int ObjectDistGetLabelTxfVertex(ObjectDist * I, int state, int index, float *v)
   int result = 0;
   if(I->DSet) {
     if(state < 0)
-      state = SettingGet_i(I->Obj.G, NULL, I->Obj.Setting, cSetting_state) - 1;
+      state = SettingGet_i(I->G, NULL, I->Setting, cSetting_state) - 1;
     if(state < 0)
-      state = SceneGetState(I->Obj.G);
+      state = SceneGetState(I->G);
     if(I->NDSet == 1)
       state = 0;                /* static singletons always active here it seems */
     state = state % I->NDSet;
     {
       DistSet *ds = I->DSet[state];
-      if((!ds) && (SettingGet_b(I->Obj.G, I->Obj.Setting, NULL, cSetting_all_states))) {
+      if((!ds) && (SettingGet_b(I->G, I->Setting, NULL, cSetting_all_states))) {
         state = 0;
         ds = I->DSet[state];
       }
@@ -76,7 +76,7 @@ int ObjectDistMoveLabel(ObjectDist * I, int state, int index, float *v, int mode
     state = 0;
   state = state % I->NDSet;
   if((!I->DSet[state])
-     && (SettingGet_b(I->Obj.G, I->Obj.Setting, NULL, cSetting_all_states)))
+     && (SettingGet_b(I->G, I->Setting, NULL, cSetting_all_states)))
     state = 0;
   /* find the corresponding distance set, for this state */
   ds = I->DSet[state];
@@ -85,7 +85,7 @@ int ObjectDistMoveLabel(ObjectDist * I, int state, int index, float *v, int mode
     /* force this object to redraw itself; invalidate the Label's coordinates
      * with the new data set, ds */
     ds->invalidateRep(cRepLabel, cRepInvCoord);
-    /*      ExecutiveUpdateCoordDepends(I->Obj.G,I); */
+    /*      ExecutiveUpdateCoordDepends(I->G,I); */
   }
   return (result);
 }
@@ -120,187 +120,12 @@ int ObjectDistMoveWithObject(ObjectDist * I, struct ObjectMolecule * O) {
     }
   }
 	
-  PRINTFD(I->Obj.G, FB_ObjectDist) " ObjectDist-Move: Out of Move\n" ENDFD;
+  PRINTFD(I->G, FB_ObjectDist) " ObjectDist-Move: Out of Move\n" ENDFD;
   return result;
 }
 /* -- JV end */
 
 
-
-		
-static DistSet *ObjectDistGetDistSetFromM4XBond(PyMOLGlobals * G,
-                                                ObjectMolecule * obj,
-                                                M4XBondType * hb, int n_hb,
-                                                int state, int nbr_sele)
-{
-  int min_id, max_id, range, *lookup = NULL;
-  int nv = 0;
-  float *vv = NULL;
-  DistSet *ds;
-  ds = DistSetNew(G);
-  vv = VLAlloc(float, 10);
-
-  /* this routine only works if IDs cover a reasonable range --
-     should rewrite using a hash table */
-
-  if(obj->NAtom) {
-
-    /* determine range */
-
-    {
-      int a, cur_id;
-      cur_id = obj->AtomInfo[0].id;
-      min_id = cur_id;
-      max_id = cur_id;
-      for(a = 1; a < obj->NAtom; a++) {
-        cur_id = obj->AtomInfo[a].id;
-        if(min_id > cur_id)
-          min_id = cur_id;
-        if(max_id < cur_id)
-          max_id = cur_id;
-      }
-    }
-
-    /* create cross-reference table */
-
-    {
-      int a, offset;
-
-      range = max_id - min_id + 1;
-      lookup = Calloc(int, range);
-      for(a = 0; a < obj->NAtom; a++) {
-        offset = obj->AtomInfo[a].id - min_id;
-        if(lookup[offset])
-          lookup[offset] = -1;
-        else {
-          lookup[offset] = a + 1;
-        }
-      }
-    }
-
-    /* iterate through IDs and get pairs */
-    {
-      AtomInfoType *ai1, *ai2;
-      int at1, at2;
-      CoordSet *cs;
-
-      float *vv0, *vv1;
-      int idx1, idx2;
-
-      int i, offset1, offset2;
-      int sele_flag = false;
-
-      for(i = 0; i < n_hb; i++) {
-        offset1 = hb[i].atom1 - min_id;
-        offset2 = hb[i].atom2 - min_id;
-        if((offset1 >= 0) && (offset1 < range) && (offset2 >= 0) && (offset2 < range)) {
-          at1 = lookup[offset1] - 1;
-          at2 = lookup[offset2] - 1;
-          if((at1 >= 0) && (at2 >= 0) && (at1 != at2) && (state < obj->NCSet)) {
-            cs = obj->CSet[state];
-
-            if(cs) {
-              ai1 = obj->AtomInfo + at1;
-              ai2 = obj->AtomInfo + at2;
-
-              sele_flag = false;
-
-              if(nbr_sele >= 0) {
-                if(SelectorIsMember(G, ai1->selEntry, nbr_sele))
-                  sele_flag = true;
-                if(SelectorIsMember(G, ai2->selEntry, nbr_sele))
-                  sele_flag = true;
-              } else {
-                sele_flag = true;
-              }
-
-              if(sele_flag) {
-                if(obj->DiscreteFlag) {
-                  if(cs == obj->DiscreteCSet[at1]) {
-                    idx1 = obj->DiscreteAtmToIdx[at1];
-                  } else {
-                    idx1 = -1;
-                  }
-                } else {
-                  idx1 = cs->AtmToIdx[at1];
-                }
-
-                if(obj->DiscreteFlag) {
-                  if(cs == obj->DiscreteCSet[at2]) {
-                    idx2 = obj->DiscreteAtmToIdx[at2];
-                  } else {
-                    idx2 = -1;
-                  }
-
-                } else {
-                  idx2 = cs->AtmToIdx[at2];
-                }
-
-                if((idx1 >= 0) && (idx2 >= 0)) {
-                  VLACheck(vv, float, (nv * 3) + 5);
-                  vv0 = vv + (nv * 3);
-                  vv1 = cs->Coord + 3 * idx1;
-                  *(vv0++) = *(vv1++);
-                  *(vv0++) = *(vv1++);
-                  *(vv0++) = *(vv1++);
-                  vv1 = cs->Coord + 3 * idx2;
-                  *(vv0++) = *(vv1++);
-                  *(vv0++) = *(vv1++);
-                  *(vv0++) = *(vv1++);
-                  nv += 2;
-                }
-              }
-            }
-          }
-        }
-      }
-    }
-  }
-
-  FreeP(lookup);
-  ds->NIndex = nv;
-  ds->Coord = vv;
-  return (ds);
-
-}
-
-ObjectDist *ObjectDistNewFromM4XBond(PyMOLGlobals * G, ObjectDist * oldObj,
-                                     struct ObjectMolecule * objMol,
-                                     struct M4XBondType * hbond,
-                                     int n_hbond, int nbr_sele)
-{
-  int a;
-  ObjectDist *I;
-  int n_state;
-  if(!oldObj)
-    I = ObjectDistNew(G);
-  else {
-    I = oldObj;
-    for(a = 0; a < I->NDSet; a++)
-      if(I->DSet[a]) {
-        I->DSet[a]->fFree();
-        I->DSet[a] = NULL;
-      }
-    I->NDSet = 0;
-  }
-  n_state = objMol->NCSet;
-  for(a = 0; a < n_state; a++) {
-    VLACheck(I->DSet, DistSet *, a);
-
-    I->DSet[a] = ObjectDistGetDistSetFromM4XBond(G, objMol, hbond, n_hbond, a, nbr_sele);
-
-    if(I->DSet[a]) {
-      I->DSet[a]->Obj = I;
-      I->NDSet = a + 1;
-    }
-  }
-  ObjectDistUpdateExtents(I);
-
-  SceneChanged(G);
-  return (I);
-}
-
-
 /*========================================================================*/
 
 void ObjectDistUpdateExtents(ObjectDist * I)
@@ -311,14 +136,14 @@ void ObjectDistUpdateExtents(ObjectDist * I)
   DistSet *ds;
 
   /* update extents */
-  copy3f(maxv, I->Obj.ExtentMin);
-  copy3f(minv, I->Obj.ExtentMax);
-  I->Obj.ExtentFlag = false;
+  copy3f(maxv, I->ExtentMin);
+  copy3f(minv, I->ExtentMax);
+  I->ExtentFlag = false;
   for(a = 0; a < I->NDSet; a++) {
     ds = I->DSet[a];
     if(ds) {
-      if(DistSetGetExtent(ds, I->Obj.ExtentMin, I->Obj.ExtentMax))
-        I->Obj.ExtentFlag = true;
+      if(DistSetGetExtent(ds, I->ExtentMin, I->ExtentMax))
+        I->ExtentFlag = true;
     }
   }
 }
@@ -347,8 +172,10 @@ static int ObjectDistDSetFromPyList(ObjectDist * I, PyObject * list)
   if(ok) {
     VLACheck(I->DSet, DistSet *, I->NDSet);
     for(a = 0; a < I->NDSet; a++) {
-      if(ok)
-        ok = DistSetFromPyList(I->Obj.G, PyList_GetItem(list, a), &I->DSet[a]);
+      if(ok){
+        auto *val = PyList_GetItem(list, a);
+        ok = DistSetFromPyList(I->G, val, &I->DSet[a]);
+      }
       if(ok)
         I->DSet[a]->Obj = I;
     }
@@ -364,7 +191,7 @@ PyObject *ObjectDistAsPyList(ObjectDist * I)
   /* first, dump the atoms */
 
   result = PyList_New(4);
-  PyList_SetItem(result, 0, ObjectAsPyList(&I->Obj));
+  PyList_SetItem(result, 0, ObjectAsPyList(I));
   PyList_SetItem(result, 1, PyInt_FromLong(I->NDSet));
   PyList_SetItem(result, 2, ObjectDistDSetAsPyList(I));
   PyList_SetItem(result, 3, PyInt_FromLong(0));
@@ -381,12 +208,14 @@ int ObjectDistNewFromPyList(PyMOLGlobals * G, PyObject * list, ObjectDist ** res
   if(ok)
     ok = PyList_Check(list);
 
-  I = ObjectDistNew(G);
+  I = new ObjectDist(G);
   if(ok)
     ok = (I != NULL);
 
-  if(ok)
-    ok = ObjectFromPyList(G, PyList_GetItem(list, 0), &I->Obj);
+  if(ok){
+    auto *val = PyList_GetItem(list, 0);
+    ok = ObjectFromPyList(G, val, I);
+  }
   if(ok)
     ok = PConvPyIntToInt(PyList_GetItem(list, 1), &I->NDSet);
   if(ok)
@@ -404,21 +233,22 @@ int ObjectDistNewFromPyList(PyMOLGlobals * G, PyObject * list, ObjectDist ** res
 }
 
 /*========================================================================*/
-int ObjectDistGetNFrames(ObjectDist * I)
+int ObjectDist::getNFrame() const
 {
-  return I->NDSet;
+  return NDSet;
 }
 
 
 /*========================================================================*/
-void ObjectDistUpdate(ObjectDist * I)
+void ObjectDist::update()
 {
+  auto I = this;
   int a;
-  OrthoBusyPrime(I->Obj.G);
+  OrthoBusyPrime(I->G);
   for(a = 0; a < I->NDSet; a++)
     if(I->DSet[a]) {
-      OrthoBusySlow(I->Obj.G, a, I->NDSet);
-      /*           printf(" ObjectDist: updating state %d of \"%s\".\n" , a+1, I->Obj.Name); */
+      OrthoBusySlow(I->G, a, I->NDSet);
+      /*           printf(" ObjectDist: updating state %d of \"%s\".\n" , a+1, I->Name); */
       I->DSet[a]->update(a);
     }
 }
@@ -428,7 +258,7 @@ void ObjectDistUpdate(ObjectDist * I)
 void ObjectDistInvalidateRep(ObjectDist * I, int rep)
 {
   int a;
-  PRINTFD(I->Obj.G, FB_ObjectDist)
+  PRINTFD(I->G, FB_ObjectDist)
     " ObjectDistInvalidateRep: entered.\n" ENDFD;
 
   for(a = 0; a < I->NDSet; a++)
@@ -439,8 +269,9 @@ void ObjectDistInvalidateRep(ObjectDist * I, int rep)
 
 
 /*========================================================================*/
-static void ObjectDistRender(ObjectDist * I, RenderInfo * info)
+void ObjectDist::render(RenderInfo * info)
 {
+  auto I = this;
   int state = info->state;
   int pass = info->pass;
   CRay *ray = info->ray;
@@ -458,30 +289,30 @@ static void ObjectDistRender(ObjectDist * I, RenderInfo * info)
   if (!shouldRender)
     return;
 
-  ObjectPrepareContext(&I->Obj, info);
-
-    for(StateIterator iter(I->Obj.G, I->Obj.Setting, state, I->NDSet);
-        iter.next();) {
-      DistSet * ds = I->DSet[iter.state];
-      if(ds)
-        ds->render(info);
-    }
+  ObjectPrepareContext(I, info);
+  
+  for(StateIterator iter(I->G, I->Setting, state, I->NDSet);
+      iter.next();) {
+    DistSet * ds = I->DSet[iter.state];
+    if(ds)
+      ds->render(info);
+  }
 }
 
 #if 0
 static CSetting **ObjectDistGetSettingHandle(ObjectDist * I, int state)
 {
   if(state < 0) {
-    return (&I->Obj.Setting);
+    return (&I->Setting);
   } else {
     return (NULL);
   }
 }
 #endif
 
-static void ObjectDistInvalidate(CObject * Iarg, int rep, int level, int state){
-  ObjectDist * I = (ObjectDist*)Iarg;
-  for(StateIterator iter(I->Obj.G, I->Obj.Setting, state, I->NDSet);
+void ObjectDist::invalidate(int rep, int level, int state){
+  auto I = this;
+  for(StateIterator iter(I->G, I->Setting, state, I->NDSet);
       iter.next();) {
     DistSet * ds = I->DSet[iter.state];
     if(ds)
@@ -490,25 +321,12 @@ static void ObjectDistInvalidate(CObject * Iarg, int rep, int level, int state){
 }
 
 /*========================================================================*/
-ObjectDist *ObjectDistNew(PyMOLGlobals * G)
+ObjectDist::ObjectDist(PyMOLGlobals * G) : CObject(G)
 {
-  OOAlloc(G, ObjectDist);
-  ObjectInit(G, (CObject *) I);
-  I->Obj.type = cObjectMeasurement;
-  I->DSet = VLACalloc(DistSet *, 10);  /* auto-zero */
-  I->NDSet = 0;
-  I->Obj.fRender = (void (*)(CObject *, RenderInfo * info)) ObjectDistRender;
-  I->Obj.fFree = (void (*)(CObject *)) ObjectDistFree;
-  I->Obj.fUpdate = (void (*)(CObject *)) ObjectDistUpdate;
-  I->Obj.fInvalidate = (void (*)(CObject *, int, int, int)) ObjectDistInvalidate;
-  I->Obj.fGetNFrame = (int (*)(CObject *)) ObjectDistGetNFrames;
-#if 0
-  I->Obj.fGetSettingHandle = (CSetting ** (*)(CObject *, int state))
-    ObjectDistGetSettingHandle;
-#endif
-  I->Obj.fDescribeElement = NULL;
-  I->Obj.Color = ColorGetIndex(G, "dash");
-  return (I);
+  auto I = this;
+  I->type = cObjectMeasurement;
+  I->DSet.reserve(10);  /* auto-zero */
+  I->Color = ColorGetIndex(G, "dash");
 }
 
 
@@ -518,10 +336,7 @@ static void ObjectDistReset(PyMOLGlobals * G, ObjectDist * I)
 	/* This wipes out all the distance sets and clears the state */
   int a;
   for(a = 0; a < I->NDSet; a++)
-    if(I->DSet[a]) {
-      I->DSet[a]->fFree();
-      I->DSet[a] = NULL;
-    }
+    DeleteP(I->DSet[a]);
   I->NDSet = 0;
 }
 
@@ -561,7 +376,7 @@ ObjectDist *ObjectDistNewFromSele(PyMOLGlobals * G, ObjectDist * oldObj,
    * overwrite it by resetting it; otherwise intialize the
    * objectDistance and its base class */
   if(!oldObj)
-    I = ObjectDistNew(G);
+    I = new ObjectDist(G);
   else {
     I = oldObj;
     if(reset)
@@ -614,8 +429,18 @@ ObjectDist *ObjectDistNewFromSele(PyMOLGlobals * G, ObjectDist * oldObj,
 
       /* this does the actual work of creating the distances for this state */
       /* I->DSet[a] = new DistSet(G, selections, states, etc) -- created this new DistSet */
-      I->DSet[a] = SelectorGetDistSet(G, I->DSet[a], sele1, state1, sele2, state2, mode, cutoff, &dist);
-
+      if (5 <= mode && mode <= 7) {
+#ifdef _PYMOL_INCENTIVE
+#else
+        PRINTFB(G, FB_ObjectDist, FB_Errors)
+          " ObjectDist-Error: modes 5-7 only available in Incentive PyMOL\n"
+          ENDFB(G);
+        I->DSet[a] = nullptr;
+#endif
+      } else {
+        I->DSet[a] = SelectorGetDistSet(
+            G, I->DSet[a], sele1, state1, sele2, state2, mode, cutoff, &dist);
+      }
 
       /* if the distances are valid, then tally the total and set the ObjectMolecule pointer as necessary */
       if(I->DSet[a]) {
@@ -654,7 +479,7 @@ ObjectDist *ObjectDistNewFromAngleSele(PyMOLGlobals * G, ObjectDist * oldObj,
 
   int frozen1=-1, frozen2=-1, frozen3=-1;
   if(!oldObj)                   /* create object if new */
-    I = ObjectDistNew(G);
+    I = new ObjectDist(G);
   else {                        /* otherwise, use existing object */
     I = oldObj;
     if(reset) {                 /* if reseting, then clear out all existing coordinate sets */
@@ -752,7 +577,7 @@ ObjectDist *ObjectDistNewFromDihedralSele(PyMOLGlobals * G, ObjectDist * oldObj,
   int frozen1=-1, frozen2=-1, frozen3=-1, frozen4=-1;
 
   if(!oldObj)                   /* create object if new */
-    I = ObjectDistNew(G);
+    I = new ObjectDist(G);
   else {                        /* otherwise, use existing object */
     I = oldObj;
     if(reset) {                 /* if reseting, then clear out all existing coordinate sets */
@@ -836,15 +661,10 @@ ObjectDist *ObjectDistNewFromDihedralSele(PyMOLGlobals * G, ObjectDist * oldObj,
 
 
 /*========================================================================*/
-void ObjectDistFree(ObjectDist * I)
+ObjectDist::~ObjectDist()
 {
-  int a;
-  for(a = 0; a < I->NDSet; a++)
-    if(I->DSet[a]) {
-      I->DSet[a]->fFree();
-      I->DSet[a] = NULL;
-    }
+  auto I = this;
+  for(int a = 0; a < I->NDSet; a++)
+    DeleteP(I->DSet[a]);
   VLAFreeP(I->DSet);
-  ObjectPurge(&I->Obj);
-  OOFreeP(I); /* from OOAlloc */
 }
