@@ -28,9 +28,10 @@ Z* -------------------------------------------------------------------
 #include"Symmetry.h"
 #include"DistSet.h"
 #include "Executive_pre.h"
-#include "CifFile.h"
 #include "vla.h"
+#include "Result.h"
 
+#include "Sculpt.h"
 #include <memory>
 
 #ifdef _WEBGL
@@ -53,55 +54,79 @@ typedef struct ObjectMoleculeBPRec {
   int n_atom;
 } ObjectMoleculeBPRec;
 
-typedef struct ObjectMolecule {
-	/* base Object class */
-  CObject Obj;
+struct ObjectMolecule : public CObject {
 	/* array of pointers to coordinate sets; one set per state */
-  struct CoordSet **CSet;
+  pymol::vla<CoordSet*> CSet;
 	/* number of coordinate sets */
-  int NCSet;
-  struct CoordSet *CSTmpl;      /* template for trajectories, etc. */
+  int NCSet = 0;
+  struct CoordSet *CSTmpl = nullptr;      /* template for trajectories, etc. */
 	/* array of bonds */
-  BondType *Bond;
+  pymol::vla<BondType> Bond;
 	/* array of atoms (infos) */
   pymol::vla<AtomInfoType> AtomInfo;
 	/* number of atoms and bonds */
-  int NAtom, prevNAtom;
-  int NBond, prevNBond;
+  int NAtom = 0, prevNAtom = 0;
+  int NBond = 0, prevNBond = 0;
 	/* is this object loaded as a discrete object? if so, number of states */
-  int DiscreteFlag;
-  int *DiscreteAtmToIdx;
-  struct CoordSet **DiscreteCSet;
+  int DiscreteFlag = 0;
+  pymol::vla<int> DiscreteAtmToIdx;
+  pymol::vla<CoordSet*> DiscreteCSet;
   /* proposed, for storing uniform trajectory data more efficiently:
      int *UniformAtmToIdx, *UniformIdxToAtm;  */
-  int CurCSet;                  /* Current state number */
-  int SeleBase;                 /* for internal usage by  selector & only valid during selection process */
-  CSymmetry *Symmetry;
-  int *Neighbor;
-  float *UndoCoord[cUndoMask + 1];
-  int UndoState[cUndoMask + 1];
-  int UndoNIndex[cUndoMask + 1];
-  int UndoIter;
-  CGO *UnitCellCGO;
-  int BondCounter;
-  int AtomCounter;
+  int SeleBase = 0;                 /* for internal usage by  selector & only valid during selection process */
+  CSymmetry *Symmetry = 0;
+  int *Neighbor = 0;
+  float *UndoCoord[cUndoMask + 1] {};
+  int UndoState[cUndoMask + 1] {};
+  int UndoNIndex[cUndoMask + 1] {};
+  int UndoIter = 0;
+  CGO *UnitCellCGO = nullptr;
+  int BondCounter = 0;
+  int AtomCounter = 0;
   /* not stored */
-  struct CSculpt *Sculpt;
-  int RepVisCacheValid;
-  int RepVisCache;     /* for transient storage during updates */
+  struct CSculpt *Sculpt =  nullptr;
+  int RepVisCacheValid = 0;
+  int RepVisCache = 0;     /* for transient storage during updates */
+
+#ifndef _PYMOL_NO_UNDO
+#endif
 
   // for reporting available assembly ids after mmCIF loading - SUBJECT TO CHANGE
-  std::shared_ptr<cif_file> m_ciffile;
-  const cif_data * m_cifdata;
+  std::shared_ptr<pymol::cif_file> m_ciffile;
+  const pymol::cif_data * m_cifdata = nullptr;
 
   // hetatm and ignore-flag by non-polymer classification
-  bool need_hetatm_classification;
+  bool need_hetatm_classification = false;
 
   // methods
-  int getState() const;
+  ObjectMolecule(PyMOLGlobals* G, int discreteFlag);
+  ~ObjectMolecule();
   bool setNDiscrete(int natom);
   bool updateAtmToIdx();
-} ObjectMolecule;
+  bool atomHasAnyCoordinates(size_t atm) const;
+
+  /// Typed version of getObjectState
+  CoordSet* getCoordSet(int state);
+  const CoordSet* getCoordSet(int state) const;
+
+  // virtual methods
+  void update() override;
+  void render(RenderInfo* info) override;
+  void invalidate(int rep, int level, int state) override;
+  int getNFrame() const override;
+  void describeElement(int index, char* buffer) const override;
+  char* getCaption(char* ch, int len) const override;
+  CSetting **getSettingHandle(int state) override;
+  CObject* clone() const override;
+  CSymmetry const* getSymmetry(int state = 0) const override
+  {
+    return Symmetry;
+  }
+  bool setSymmetry(CSymmetry const& symmetry, int state = 0) override;
+
+protected:
+  CObjectState* _getObjectState(int state) override;
+};
 
 /* this is a record that holds information for specific types of Operatations on Molecules, eg. translation/rotation/etc */
 typedef struct ObjectMoleculeOpRec {
@@ -178,7 +203,9 @@ typedef struct {
 #define OMOP_AlterState 16
 #define OMOP_Flag 17
 #define OMOP_LABL 18
+#if 0
 #define OMOP_Identify    19
+#endif
 #define OMOP_Remove 20
 #define OMOP_Protect 21
 #define OMOP_Mask 22
@@ -228,7 +255,6 @@ typedef struct {
 #define OMOP_ReferenceSwap 65
 #define OMOP_RenameAtoms 66
 
-#include"CoordSet.h"
 
 typedef struct {
   ObjectMolecule *trg_obj, *mbl_obj;    /* target and mobile objects */
@@ -239,45 +265,6 @@ typedef struct {
 void ObjMolPairwiseInit(ObjMolPairwise * pairwise);
 void ObjMolPairwisePurge(ObjMolPairwise * pairwise);
 
-
-/* Metaphorics Annotated PDB stuff */
-
-typedef struct M4XBondType {    /* now used for non-bonds as well as h-bonds */
-  int atom1;
-  int atom2;
-  float strength;
-} M4XBondType;
-
-typedef struct {
-  WordType name;
-  int *site, n_site;
-  int *ligand, n_ligand;
-  int *water, n_water;
-  M4XBondType *hbond, *nbond;
-  int n_hbond, n_nbond;
-} M4XContextType;
-
-typedef struct {
-  ObjectNameType target;
-  int n_point;
-  int *id_at_point;
-  float *fitness;
-} M4XAlignType;
-
-typedef struct {
-  int annotated_flag;
-  int invisible;
-  int n_context;
-  M4XContextType *context;
-  int xname_flag;
-  ObjectNameType xname;
-  M4XAlignType *align;
-} M4XAnnoType;
-
-typedef struct {
-  ObjectNameType name;
-} ObjMolMultiplexType;
-
 /**
  * loop iterators for the ObjectMolecule::Neighbor array
  *
@@ -295,14 +282,6 @@ typedef struct {
 #define ITERNEIGHBORATOMS(N, a, n, i) for(i = N[a] + 1; (n = N[i]) > -1; i += 2)
 #define ITERNEIGHBORBONDS(N, a, b, i) for(i = N[a] + 1; (b = N[i + 1]), N[i] > -1; i += 2)
 
-void M4XAnnoInit(M4XAnnoType * m4x);
-void M4XAnnoPurge(M4XAnnoType * m4x);
-
-void M4XAlignInit(M4XAlignType * align);
-void M4XAlignPurge(M4XAlignType * align);
-
-int ObjectMoleculeGetMatrix(ObjectMolecule * I, int state, double **history);
-int ObjectMoleculeSetMatrix(ObjectMolecule * I, int state, double *matrix);
 int ObjectMoleculeGetTopNeighbor(PyMOLGlobals * G,
                                  ObjectMolecule * I, int start, int excluded);
 
@@ -313,10 +292,7 @@ int ObjectMoleculeGetNearestBlendedColor(ObjectMolecule * I, const float *point,
                                          int sub_vdw);
 
 int *ObjectMoleculeGetPrioritizedOtherIndexList(ObjectMolecule * I, struct CoordSet *cs);
-int ObjectMoleculeGetPrioritizedOther(int *other, int a1, int a2, int *double_sided);
-
-void ObjectMoleculeM4XAnnotate(ObjectMolecule * I, M4XAnnoType * m4x, const char *script_file,
-                               int match_colors, int nbr_sele);
+int ObjectMoleculeGetPrioritizedOther(const int *other, int a1, int a2, int *double_sided);
 
 
 /* */
@@ -327,10 +303,9 @@ void ObjectMoleculeOpRecInit(ObjectMoleculeOpRec * op);
 int ObjectMoleculeNewFromPyList(PyMOLGlobals * G, PyObject * list,
                                 ObjectMolecule ** result);
 PyObject *ObjectMoleculeAsPyList(ObjectMolecule * I);
-int ObjectMoleculeSetStateTitle(ObjectMolecule * I, int state, const char *text);
+pymol::Result<> ObjectMoleculeSetStateTitle(ObjectMolecule * I, int state, const char *text);
 const char *ObjectMoleculeGetStateTitle(ObjectMolecule * I, int state);
 int ObjectMoleculeCheckFullStateSelection(ObjectMolecule * I, int sele, int state);
-void ObjectMoleculeFree(ObjectMolecule * I);    /* only for friends of ObjectMolecule */
 
 int ObjectMoleculeSetStateOrder(ObjectMolecule * I, int * order, int len);
 
@@ -338,11 +313,11 @@ int ObjectMoleculeAddPseudoatom(ObjectMolecule * I, int sele_index, const char *
                                 const char *resn, const char *resi, const char *chain,
                                 const char *segi, const char *elem, float vdw,
                                 int hetatm, float b, float q, const char *label,
-                                float *pos, int color, int state, int more, int quiet);
+                                const float *pos, int color, int state, int more, int quiet);
 
-ObjectMolecule *ObjectMoleculeNew(PyMOLGlobals * G, int discreteFlag);
 int ObjectMoleculeSort(ObjectMolecule * I);
 ObjectMolecule *ObjectMoleculeCopy(const ObjectMolecule * obj);
+void ObjectMoleculeCopyNoAlloc(const ObjectMolecule * src, ObjectMolecule * dst);
 void ObjectMoleculeFixChemistry(ObjectMolecule * I, int sele1, int sele2, int invalidate);
 
 ObjectMolecule *ObjectMoleculeLoadTOPFile(PyMOLGlobals * G, ObjectMolecule * obj,
@@ -374,33 +349,31 @@ ObjectMolecule *ObjectMoleculeReadStr(PyMOLGlobals * G, ObjectMolecule * I,
 
 ObjectMolecule *ObjectMoleculeReadPDBStr(PyMOLGlobals * G, ObjectMolecule * obj,
                                          const char *molstr, int frame, int discrete,
-                                         M4XAnnoType * m4x, char *pdb_name,
+                                         char *pdb_name,
                                          const char **next_pdb, PDBInfoRec * pdb_info,
                                          int quiet, int *model_number);
 
 int ObjectMoleculeExtendIndices(ObjectMolecule * I, int state);
 
-void ObjectMoleculeInvalidate(ObjectMolecule * I, int rep, int level, int state);
 void ObjectMoleculeInvalidateAtomType(ObjectMolecule *I, int state);
 
 void ObjectMoleculeRenderSele(ObjectMolecule * I, int curState, int sele, int vis_only SELINDICATORARG);
 
 void ObjectMoleculeSeleOp(ObjectMolecule * I, int sele, ObjectMoleculeOpRec * op);
 
-struct CoordSet *ObjectMoleculeGetCoordSet(ObjectMolecule * I, int setIndex);
 int ObjectMoleculeMerge(ObjectMolecule * I, pymol::vla<AtomInfoType>&& ai,
 			struct CoordSet *cs, int bondSearchFlag,
 			int aic_mask, int invalidate);
 void ObjectMoleculeUpdateNonbonded(ObjectMolecule * I);
 int ObjectMoleculeUpdateNeighbors(ObjectMolecule * I);
-int ObjectMoleculeMoveAtom(ObjectMolecule * I, int state, int index, float *v, int mode,
+int ObjectMoleculeMoveAtom(ObjectMolecule * I, int state, int index, const float *v, int mode,
                            int log);
 int ObjectMoleculeMoveAtomLabel(ObjectMolecule * I, int state, int index, float *v, int log, float *diff);
 int ObjectMoleculeGetAtomVertex(ObjectMolecule * I, int state, int index, float *v);
 int ObjectMoleculeGetAtomTxfVertex(ObjectMolecule * I, int state, int index, float *v);
 int ObjectMoleculeGetAtomIndex(ObjectMolecule * I, int sele);
 int ObjectMoleculeTransformSelection(ObjectMolecule * I, int state,
-                                     int sele, float *TTT, int log,
+                                     int sele, const float *TTT, int log,
                                      const char *sname, int homogenous, int global);
 int ObjectMoleculeDoesAtomNeighborSele(ObjectMolecule * I, int index, int sele);
 void ObjectMoleculeInferChemFromNeighGeom(ObjectMolecule * I, int state);
@@ -414,6 +387,8 @@ int ObjectMoleculeXferValences(ObjectMolecule * Ia, int sele1, int sele2,
 void ObjectMoleculeGuessValences(ObjectMolecule * I, int state, int *flag1, int *flag2,
                                  int reset);
 int ObjectMoleculeAddBond(ObjectMolecule * I, int sele0, int sele1, int order);
+pymol::Result<> ObjectMoleculeAddBondByIndices(
+    ObjectMolecule* I, unsigned atm1, unsigned atm2, int order);
 int ObjectMoleculeRemoveBonds(ObjectMolecule * I, int sele1, int sele2);
 
 int ObjectMoleculeAutoDisableAtomNameWildcard(ObjectMolecule * I);
@@ -439,15 +414,23 @@ int ObjectMoleculeFuse(ObjectMolecule * I, int index0, ObjectMolecule * src, int
                         int mode, int move_flag);
 int ObjectMoleculeRenameAtoms(ObjectMolecule * I, int *flag, int force);
 int ObjectMoleculeAreAtomsBonded(ObjectMolecule * I, int i0, int i1);
-void ObjectGotoState(ObjectMolecule * I, int state);
+void ObjectGotoState(CObject* I, int state);
 float ObjectMoleculeGetAvgHBondVector(ObjectMolecule * I, int atom, int state, float *v,
                                       float *incoming);
 int ObjectMoleculeCheckBondSep(ObjectMolecule * I, int a0, int a1, int dist);
 int ObjectMoleculeGetPhiPsi(ObjectMolecule * I, int ca, float *phi, float *psi,
                             int state);
-void ObjectMoleculeGetAtomSele(ObjectMolecule * I, int index, char *buffer);
-void ObjectMoleculeGetAtomSeleFast(ObjectMolecule * I, int index, char *buffer);
-void ObjectMoleculeGetAtomSeleLog(ObjectMolecule * I, int index, char *buffer, int quote);
+void ObjectMoleculeGetAtomSele(const ObjectMolecule * I, int index, char *buffer);
+
+/**
+ * Retrives selection string of an Object Molecule's atom
+ * @param I ObjectMolecule whose info is retrieved
+ * @param index atom index of I whose info is retrieved
+ * @return selection string unique to the atom
+ */
+std::string ObjectMoleculeGetAtomSeleFast(const ObjectMolecule * I, int index);
+
+void ObjectMoleculeGetAtomSeleLog(const ObjectMolecule * I, int index, char *buffer, int quote);
 
 void ObjectMoleculeUpdateIDNumbers(ObjectMolecule * I);
 
@@ -465,7 +448,9 @@ int ObjectMoleculePurgeBondPath(ObjectMolecule * I, ObjectMoleculeBPRec * bp);
 int ***ObjectMoleculeGetBondPrint(ObjectMolecule * I, int max_bond, int max_type,
                                   int *dim);
 
-int ObjectMoleculeConnect(ObjectMolecule * I, int *nbond, BondType ** bond, AtomInfoType * ai,
+bool ObjectMoleculeConnect(ObjectMolecule* I, CoordSet* cs,
+    bool searchFlag = true, int connectModeOverride = -1);
+bool ObjectMoleculeConnect(ObjectMolecule* I, int& nbond, pymol::vla<BondType>& bond,
                           struct CoordSet *cs, int searchFlag, int connectModeOverride);
 int ObjectMoleculeSetDiscrete(PyMOLGlobals * G, ObjectMolecule * I, int discrete);
 
@@ -486,7 +471,7 @@ int ObjectMoleculeConvertIDsToIndices(ObjectMolecule * I, int *id, int n_id);
 ObjectMolecule *ObjectMoleculeDummyNew(PyMOLGlobals * G, int mode);
 void ObjectMoleculeDummyUpdate(ObjectMolecule * I, int mode);
 
-void ObjectMoleculeTransformState44f(ObjectMolecule * I, int state, float *matrix,
+void ObjectMoleculeTransformState44f(ObjectMolecule * I, int state, const float *matrix,
                                      int log_trans, int homogenous, int transformed);
 
 
@@ -497,7 +482,6 @@ struct CoordSet *ObjectMoleculePDBStr2CoordSet(PyMOLGlobals * G,
                                                AtomInfoType ** atInfoPtr,
                                                const char **restart_model,
                                                char *segi_override,
-                                               M4XAnnoType * m4x,
                                                char *pdb_name,
                                                const char **next_pdb,
                                                PDBInfoRec * pdb_info,
@@ -520,7 +504,9 @@ void ObjectMoleculeAdjustDiscreteAtmIdx(ObjectMolecule *I, int *lookup, int nAto
 
 void AtomInfoSettingGenerateSideEffects(PyMOLGlobals * G, ObjectMolecule *obj, int index, int id);
 
-int *AtomInfoGetSortedIndex(PyMOLGlobals * G, ObjectMolecule * obj, AtomInfoType * rec, int n,
+int *AtomInfoGetSortedIndex(PyMOLGlobals * G,
+    const ObjectMolecule* obj,
+    const AtomInfoType* rec, int n,
                             int **outdex);
 
 ObjectMolecule *ObjectMoleculeReadMmtfStr(PyMOLGlobals * G, ObjectMolecule * I,
@@ -533,4 +519,8 @@ template <typename V> void SettingSet(int index, V value, ObjectMolecule * I, in
   SettingSet(index, value, (CObject*)I, state);
 }
 
+std::unique_ptr<int[]> LoadTrajSeleHelper(
+    const ObjectMolecule* obj, CoordSet* cs, const char* selection);
+
+int ObjectMoleculeSetGeometry(PyMOLGlobals* G, ObjectMolecule* I, int sele, int geom, int valence);
 #endif
