@@ -312,13 +312,14 @@ static int RepZeroOrderBond(RepCylBond *I, CGO *cgo, bool s1, bool s2, float *vv
 
 static int RepValence(RepCylBond *I, CGO *cgo, bool s1, bool s2, bool isRamped,
 		      float *v1, float *v2, int *other,
-		      int a1, int a2, float *coord,
+		      int a1, int a2, const float *coord,
 		      float *color1, float *color2, int ord,
 		      float tube_size,
 		      bool fixed_r, float scale_r,
 		      Pickable pickdata[] = NULL)
 {
-  float d[3], t[3], p0[3], p1[3], p2[3], *vv;
+  float d[3], t[3], p0[3], p1[3], p2[3];
+  const float* vv;
   float v1t[3], v2t[3];
   int a3;
   int double_sided;
@@ -537,13 +538,13 @@ static int RepValence(RepCylBond *I, CGO *cgo, bool s1, bool s2, bool isRamped,
 
 Rep *RepCylBondNew(CoordSet * cs, int state)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj;
   int a1, a2; // can be -1 for missing atoms
   int a;
   int c1, c2, s1, s2;
   unsigned int b1, b2;
-  BondType *b;
+  const BondType *b;
   float radius;
   float valence;
   int half_bonds, *other = NULL;
@@ -551,7 +552,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
   int ord;
   int stick_ball, stick_ball_color = -1;
   float stick_ball_ratio = 1.0F;
-  AtomInfoType *ai1;
+  const AtomInfoType *ai1;
   bool fixed_radius = false;
   int valence_flag = false;
   int hide_long = false;
@@ -596,31 +597,31 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
     return (NULL);              /* skip if no sticks are visible */
   }
 
-  capdrawn = Calloc(float, obj->NAtom); // max radius of caps
-  marked = Calloc(bool, obj->NAtom);
+  capdrawn = pymol::calloc<float>(obj->NAtom); // max radius of caps
+  marked = pymol::calloc<bool>(obj->NAtom);
   CHECKOK(ok, marked);
   if (!ok){
     RepCylBondFree(I);
     return NULL;
   }
 
-  valence = SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_valence);
+  valence = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_valence);
   valence_flag = (valence != 0.0F);
 
   alpha =
-    SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_transparency);
+    SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_transparency);
   alpha = 1.0F - alpha;
   if(fabs(alpha - 1.0) < R_SMALL4)
     alpha = 1.0F;
 
-  stick_color = SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_stick_color);
-  cartoon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Obj.Setting,
+  stick_color = SettingGet_color(G, cs->Setting, obj->Setting, cSetting_stick_color);
+  cartoon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Setting,
                                            cSetting_cartoon_side_chain_helper);
-  ribbon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Obj.Setting,
+  ribbon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Setting,
                                           cSetting_ribbon_side_chain_helper);
 
-  transp = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_transparency);
-  hide_long = SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_hide_long_bonds);
+  transp = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_transparency);
+  hide_long = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_hide_long_bonds);
 
   std::set<int> all_zero_order_bond_atoms;
   b = obj->Bond;
@@ -653,24 +654,24 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
     b++;
     ok &= !G->Interrupt;
   }
-  radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_radius);
-  half_bonds = SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_half_bonds);
+  radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_radius);
+  half_bonds = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_half_bonds);
   na_mode =
-    SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_cartoon_nucleic_acid_mode);
+    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_cartoon_nucleic_acid_mode);
   int na_mode_ribbon =
-    SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_nucleic_acid_mode);
-  h_scale = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_h_scale);
+    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_nucleic_acid_mode);
+  h_scale = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_h_scale);
   auto valence_zero_scale =
-    SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_valence_zero_scale);
+    SettingGet_f(G, cs->Setting, obj->Setting, cSetting_valence_zero_scale);
   auto valence_zero_mode =
-    SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_valence_zero_mode);
+    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_valence_zero_mode);
 
   RepInit(G, &I->R);
   I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepCylBondRender;
   I->R.fFree = (void (*)(struct Rep *)) RepCylBondFree;
   I->R.obj = (CObject *) obj;
   I->R.cs = cs;
-  I->R.context.object = (void *) obj;
+  I->R.context.object = obj;
   I->R.context.state = state;
 
   I->renderCGO = 0;
@@ -680,7 +681,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
     CGOAlpha(I->primitiveCGO, alpha);
   }
   if(ok && obj->NBond) {
-    stick_ball = SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_stick_ball);
+    stick_ball = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_stick_ball);
 
     shader_mode = SettingGetGlobal_b(G, cSetting_use_shaders) && SettingGetGlobal_b(G, cSetting_stick_as_cylinders) 
       && SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_stick_use_shader);
@@ -695,14 +696,14 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
       other = ObjectMoleculeGetPrioritizedOtherIndexList(obj, cs);
       CHECKOK(ok, other);
       if (ok){
-        fixed_radius = SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_stick_fixed_radius);
-        scale_r = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_valence_scale);
+        fixed_radius = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_stick_fixed_radius);
+        scale_r = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_valence_scale);
       }
     }
 
     /* spheres for stick & balls */
-    stick_ball_ratio = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_ball_ratio);
-    stick_ball_color = SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_stick_ball_color);
+    stick_ball_ratio = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_ball_ratio);
+    stick_ball_color = SettingGet_color(G, cs->Setting, obj->Setting, cSetting_stick_ball_color);
 
     b = obj->Bond;
     for(a = 0; ok && a < obj->NBond; ++a, ++b) {
@@ -750,7 +751,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
 
         if(bd_stick_color < 0) {
           if(bd_stick_color == cColorObject) {
-            c1 = (c2 = obj->Obj.Color);
+            c1 = (c2 = obj->Color);
           } else if(ColorCheckRamped(G, bd_stick_color)) {
             c1 = (c2 = bd_stick_color);
           } else {
@@ -937,7 +938,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
 }
 
 #ifndef PURE_OPENGL_ES_2
-static void RepCylinderImmediate(float *v1arg, float *v2arg, int nEdge,
+static void RepCylinderImmediate(const float *v1arg, const float *v2arg, int nEdge,
                                  int frontCapArg, int endCapArg,
                                  float overlap, float nub, float radius, float **dir)
 {
@@ -974,7 +975,7 @@ static void RepCylinderImmediate(float *v1arg, float *v2arg, int nEdge,
 
   if (dir){
     if (!*dir){
-      *dir = Alloc(float, 3);
+      *dir = pymol::malloc<float>(3);
       (*dir)[0] = d[0]; (*dir)[1] = d[1]; (*dir)[2] = d[2];
     } else {
       if (get_angle3f(d, *dir)>=(cPI/2.)){
@@ -1109,34 +1110,34 @@ void RepCylBondRenderImmediate(CoordSet * cs, RenderInfo * info)
 
    */
 
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   if(info->ray || info->pick || (!(G->HaveGUI && G->ValidContext)))
     return;
   else {
     int active = false;
     ObjectMolecule *obj = cs->Obj;
-    int nEdge = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_stick_quality);
+    int nEdge = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_stick_quality);
     float radius =
-      fabs(SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_radius));
+      fabs(SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_radius));
     float overlap =
-      SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_overlap);
-    float nub = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_stick_nub);
+      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_overlap);
+    float nub = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_nub);
     float overlap_r = radius * overlap;
     float nub_r = radius * nub;
 
     {
       int a;
       int nBond = obj->NBond;
-      BondType *bd = obj->Bond;
-      AtomInfoType *ai = obj->AtomInfo;
+      const BondType *bd = obj->Bond.data();
+      const AtomInfoType *ai = obj->AtomInfo.data();
       int last_color = -9;
-      float *coord = cs->Coord;
+      const float *coord = cs->Coord.data();
       const float _pt5 = 0.5F;
 
       for(a = 0; a < nBond; a++) {
         int b1 = bd->index[0];
         int b2 = bd->index[1];
-        AtomInfoType *ai1, *ai2;
+        const AtomInfoType *ai1, *ai2;
         bd++;
 
         if( ((ai1 = ai + b1)->visRep & cRepCylBit) &&
@@ -1150,8 +1151,8 @@ void RepCylBondRenderImmediate(CoordSet * cs, RenderInfo * info)
             int c1 = ai1->color;
             int c2 = ai2->color;
 
-            float *v1 = coord + 3 * a1;
-            float *v2 = coord + 3 * a2;
+            const float *v1 = coord + 3 * a1;
+            const float *v2 = coord + 3 * a2;
 
             if(c1 == c2) {      /* same colors -> one cylinder */
               if(c1 != last_color) {
