@@ -1216,7 +1216,6 @@ static void RepLabelRender(RepLabel * I, RenderInfo * info)
 	return;
       } else {
         Pickable *p = I->R.P;
-        unsigned int i;
         TextSetIsPicking(G, true);
         SceneSetupGLPicking(G);
         if(c) {
@@ -1228,7 +1227,6 @@ static void RepLabelRender(RepLabel * I, RenderInfo * info)
           if (!I->shaderCGO){
             SceneGetWidthHeight(G, &screenwidth, &screenheight);
           }
-          i = pick->begin()->src.index;
 
           while(c--) {
             if(*l) {
@@ -1255,14 +1253,14 @@ static void RepLabelRender(RepLabel * I, RenderInfo * info)
                   copy3f(offpt, tCenterPt);
                 }
               }
-              i++;
               TextSetPosNColor(G, tCenterPt, v);
               TextSetTargetPos(G, v + 3);
               TextSetLabelBkgrdInfo(G, *(v + 16), *(v + 17), (v + 18));
 
               if (p) {
                 p++;
-                AssignNewPickColor(NULL, i, pick, &I->R.context, TextGetColorUChar4uv(G), p->index, p->bond);
+                AssignNewPickColor(nullptr, pick, TextGetColorUChar4uv(G),
+                    &I->R.context, p->index, p->bond);
               }
 
               TextSetColorFromUColor(G);
@@ -1278,7 +1276,6 @@ static void RepLabelRender(RepLabel * I, RenderInfo * info)
           }
           if(float_text)
             glEnable(GL_DEPTH_TEST);
-          (*pick)[0].src.index = i;       /* pass the count */
         }
         TextSetIsPicking(G, false);
       }
@@ -1506,10 +1503,10 @@ static void RepLabelRender(RepLabel * I, RenderInfo * info)
 
 Rep *RepLabelNew(CoordSet * cs, int state)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj;
   int a, a1, c1;
-  float *v, *v0;
+  float *v;
   const float *vc;
   lexidx_t *l;
   int label_color;
@@ -1524,7 +1521,7 @@ Rep *RepLabelNew(CoordSet * cs, int state)
   RepLabelInit(I);
   obj = cs->Obj;
 
-  label_color = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_label_color);
+  label_color = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_label_color);
   RepInit(G, &I->R);
 
   I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepLabelRender;
@@ -1532,21 +1529,21 @@ Rep *RepLabelNew(CoordSet * cs, int state)
   I->R.fRecolor = NULL;
   I->R.obj = (CObject *) obj;
   I->R.cs = cs;
-  I->R.context.object = (void *) obj;
+  I->R.context.object = obj;
   I->R.context.state = state;
 
   /* raytracing primitives */
 
-  I->L = Calloc(lexidx_t, cs->NIndex);
+  I->L = pymol::calloc<lexidx_t>(cs->NIndex);
   ErrChkPtr(G, I->L);
-  I->V = Calloc(float, cs->NIndex * 28);
+  I->V = pymol::calloc<float>(cs->NIndex * 28);
   ErrChkPtr(G, I->V);
 
   I->OutlineColor =
-    SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_label_outline_color);
+    SettingGet_color(G, cs->Setting, obj->Setting, cSetting_label_outline_color);
 
-  if(SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_pickable)) {
-    I->R.P = Alloc(Pickable, cs->NIndex + 1);
+  if(SettingGet_b(G, cs->Setting, obj->Setting, cSetting_pickable)) {
+    I->R.P = pymol::malloc<Pickable>(cs->NIndex + 1);
     ErrChkPtr(G, I->R.P);
     rp = I->R.P + 1;            /* skip first record! */
   }
@@ -1575,7 +1572,7 @@ Rep *RepLabelNew(CoordSet * cs, int state)
       *(v++) = *(vc++);
       *(v++) = *(vc++);
 
-      v0 = cs->Coord + 3 * a;
+      const float* v0 = cs->coordPtr(a);
       *(v++) = *(v0++);
       *(v++) = *(v0++);
       *(v++) = *(v0++);
