@@ -37,9 +37,10 @@ Z* -------------------------------------------------------------------
 
 typedef struct RepMesh {
   Rep R;
-  int *N;
+  pymol::vla<int> N;
   int NTot;
-  float *V, *VC;
+  pymol::vla<float> V;
+  float* VC;
   int NDot;
   float *Dot;
   float Radius, Width;
@@ -62,8 +63,6 @@ void RepMeshFree(RepMesh * I)
     I->shaderCGO = 0;
   }
   FreeP(I->VC);
-  VLAFreeP(I->V);
-  VLAFreeP(I->N);
   FreeP(I->LastColor);
   FreeP(I->LastVisib);
   OOFreeP(I);
@@ -76,9 +75,9 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
 static int RepMeshCGOGenerate(RepMesh * I, RenderInfo * info)
 {
   PyMOLGlobals *G = I->R.G;
-  float *v = I->V;
+  float *v = I->V.data();
   float *vc = I->VC;
-  int *n = I->N;
+  int *n = I->N.data();
   int ok = true;
   short use_shader;
   short mesh_as_cylinders;
@@ -302,9 +301,9 @@ static void RepMeshRender(RepMesh * I, RenderInfo * info)
   CRay *ray = info->ray;
   auto pick = info->pick;
   PyMOLGlobals *G = I->R.G;
-  float *v = I->V;
+  float *v = I->V.data();
   float *vc = I->VC;
-  int *n = I->N;
+  int *n = I->N.data();
   int c;
   const float *col = NULL;
   float line_width = SceneGetDynamicLineWidth(info, I->Width);
@@ -561,7 +560,7 @@ int RepMeshSameVis(RepMesh * I, CoordSet * cs)
 static
 void RepMeshColor(RepMesh * I, CoordSet * cs)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   MapType *map;
   int a, i0, i, j, h, k, l, c1;
   float *v0, *vc;
@@ -580,16 +579,16 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
 
   obj = cs->Obj;
 
-  probe_radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_solvent_radius);
-  mesh_color = SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_color);
-  mesh_mode = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_mode);
+  probe_radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_solvent_radius);
+  mesh_color = SettingGet_color(G, cs->Setting, obj->Setting, cSetting_mesh_color);
+  mesh_mode = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_mode);
   cullByFlag = (mesh_mode == cRepMesh_by_flags);
   inclH = !(mesh_mode == cRepMesh_heavy_atoms);
 
   if(!I->LastVisib)
-    I->LastVisib = Alloc(int, cs->NIndex);
+    I->LastVisib = pymol::malloc<int>(cs->NIndex);
   if(!I->LastColor)
-    I->LastColor = Alloc(int, cs->NIndex);
+    I->LastColor = pymol::malloc<int>(cs->NIndex);
   lv = I->LastVisib;
   lc = I->LastColor;
   for(a = 0; a < cs->NIndex; a++) {
@@ -599,11 +598,11 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
   }
 
   if(I->mesh_type != 1) {
-    I->Width = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_width);
-    I->Radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_radius);
+    I->Width = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_width);
+    I->Radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_radius);
   } else {
-    I->Width = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_dot_width);
-    I->Radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_dot_radius);
+    I->Width = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_dot_width);
+    I->Radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_dot_radius);
   }
 
   if(I->NTot) {
@@ -615,7 +614,7 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
     }
     first_color = -1;
     if(!I->VC)
-      I->VC = Alloc(float, 3 * I->NTot);
+      I->VC = pymol::malloc<float>(3 * I->NTot);
     vc = I->VC;
     /* now, assign colors to each point */
     map = MapNew(G, I->max_vdw + probe_radius, cs->Coord, cs->NIndex, NULL);
@@ -624,9 +623,9 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
       for(a = 0; a < I->NTot; a++) {
         AtomInfoType *ai0 = NULL;
         c1 = 1;
-        minDist = MAXFLOAT;
+        minDist = FLT_MAX;
         i0 = -1;
-        v0 = I->V + 3 * a;
+        v0 = I->V.data() + 3 * a;
         MapLocus(map, v0, &h, &k, &l);
 
         i = *(MapEStart(map, h, k, l));
@@ -636,7 +635,7 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
             ai2 = obj->AtomInfo + cs->IdxToAtm[j];
             if((inclH || (!ai2->isHydrogen())) &&
                ((!cullByFlag) || (!(ai2->flags & cAtomFlag_ignore)))) {
-              dist = (float) diff3f(v0, cs->Coord + j * 3) - ai2->vdw;
+              dist = (float) diff3f(v0, cs->coordPtr(j)) - ai2->vdw;
               if(dist < minDist) {
                 i0 = j;
                 ai0 = ai2;
@@ -701,7 +700,7 @@ void RepMeshColor(RepMesh * I, CoordSet * cs)
 
 Rep *RepMeshNew(CoordSet * cs, int state)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj;
   CoordSet *ccs;
   int a, b, c, d, h, k, l, *n;
@@ -738,17 +737,17 @@ Rep *RepMeshNew(CoordSet * cs, int state)
   " RepMeshNew-DEBUG: entered with coord-set %p\n", (void *) cs ENDFD;
   if (ok){
     obj = cs->Obj;
-    I->R.context.object = (void *) obj;
+    I->R.context.object = obj;
     I->R.context.state = state;
   }
   if (ok){
-    probe_radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_solvent_radius);
+    probe_radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_solvent_radius);
     probe_radius2 = probe_radius * probe_radius;
-    solv_acc = (SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_solvent));
-    mesh_type = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_type);
-    mesh_skip = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_skip);
+    solv_acc = (SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_solvent));
+    mesh_type = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_type);
+    mesh_skip = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_skip);
     
-    mesh_mode = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_mode);
+    mesh_mode = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_mode);
     cullByFlag = (mesh_mode == cRepMesh_by_flags);
     inclH = !(mesh_mode == cRepMesh_heavy_atoms);
   }
@@ -773,7 +772,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
 
   RepInit(G, &I->R);
 
-  min_spacing = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_min_mesh_spacing);
+  min_spacing = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_min_mesh_spacing);
 
   I->N = NULL;
   I->NTot = 0;
@@ -790,14 +789,14 @@ Rep *RepMeshNew(CoordSet * cs, int state)
   I->LastVisib = NULL;
   I->LastColor = NULL;
   I->mesh_type = mesh_type;
-  I->Radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_radius);
+  I->Radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_radius);
   I->shaderCGO = 0;
 
   meshFlag = true;
 
   if(meshFlag) {
     float trim_cutoff =
-      SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_cutoff);
+      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_cutoff);
     int trim_flag = false;
     float *trim_vla = NULL;
     MapType *trim_map = NULL;
@@ -805,7 +804,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     int carve_state = 0;
     int carve_flag = false;
     float carve_cutoff =
-      SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_carve_cutoff);
+      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_carve_cutoff);
     const char *carve_selection = NULL;
     float *carve_vla = NULL;
     MapType *carve_map = NULL;
@@ -813,12 +812,12 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     int clear_state = 0;
     int clear_flag = false;
     float clear_cutoff =
-      SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_clear_cutoff);
+      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_mesh_clear_cutoff);
     const char *clear_selection = NULL;
     float *clear_vla = NULL;
     MapType *clear_map = NULL;
 
-    int mesh_max = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_grid_max);
+    int mesh_max = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_grid_max);
     if(mesh_max < 1)
       mesh_max = 1;
 
@@ -839,7 +838,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
           VLACheck(trim_vla, float, nc * 3 + 2);
 	  CHECKOK(ok, trim_vla);
           if (ok) {
-            float *src = cs->Coord + 3 * c;
+            const float* src = cs->coordPtr(c);
             float *dst = trim_vla + 3 * nc;
             *(dst++) = *(src++);
             *(dst++) = *(src++);
@@ -860,9 +859,9 @@ Rep *RepMeshNew(CoordSet * cs, int state)
 
     if(ok && carve_cutoff > 0.0F) {
       carve_state =
-        SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_carve_state) - 1;
+        SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_carve_state) - 1;
       carve_selection =
-        SettingGet_s(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_carve_selection);
+        SettingGet_s(G, cs->Setting, obj->Setting, cSetting_mesh_carve_selection);
       if(carve_selection)
         carve_map = SelectorGetSpacialMapFromSeleCoord(G,
                                                        SelectorIndexByName(G,
@@ -876,9 +875,9 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     }
     if(ok && clear_cutoff > 0.0F) {
       clear_state =
-        SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_clear_state) - 1;
+        SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_clear_state) - 1;
       clear_selection =
-        SettingGet_s(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_clear_selection);
+        SettingGet_s(G, cs->Setting, obj->Setting, cSetting_mesh_clear_selection);
       if(clear_selection)
         clear_map = SelectorGetSpacialMapFromSeleCoord(G,
                                                        SelectorIndexByName(G,
@@ -892,18 +891,18 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     }
 
     if (ok)
-      I->V = (float*) VLAMalloc(1000, sizeof(float), 9, false);
+      I->V = pymol::vla_take_ownership((float*)VLAMalloc(1000, sizeof(float), 9, false));
     CHECKOK(ok, I->V);
     if (ok)
-      I->N = (int*) VLAMalloc(100, sizeof(int), 9, false);
+      I->N = pymol::vla_take_ownership((int*)VLAMalloc(100, sizeof(int), 9, false));
     CHECKOK(ok, I->N);
     if (ok)
       I->N[0] = 0;
 
     if (ok){
       for(c = 0; c < 3; c++) {
-	minE[c] = MAXFLOAT;
-	maxE[c] = -(MAXFLOAT);
+	minE[c] = FLT_MAX;
+	maxE[c] = -FLT_MAX;
       }
       for(b = 0; b < obj->NCSet; b++) {
 	ccs = obj->CSet[b];
@@ -946,7 +945,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
       
       for(c = 0; c < 3; c++)
 	dims[c] = (int) ((sizeE[c] / gridSize) + 1.5F);
-      field = IsosurfFieldAlloc(G, dims);
+      field = new Isofield(G, dims);
       CHECKOK(ok, field);
     }
 
@@ -979,8 +978,8 @@ Rep *RepMeshNew(CoordSet * cs, int state)
             point[2] = minE[2] + c * gridSize;
             copy3f(point, F3Ptr(field->points, a, b, c));
             aNear = -1;
-            bestDist = MAXFLOAT;
-            aLen = MAXFLOAT;
+            bestDist = FLT_MAX;
+            aLen = FLT_MAX;
             MapLocus(map, point, &h, &k, &l);
             d = *(MapEStart(map, h, k, l));
             if(d) {
@@ -989,7 +988,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
                 ai1 = obj->AtomInfo + cs->IdxToAtm[cur];
                 if((inclH || (!ai1->isHydrogen())) &&
                    ((!cullByFlag) || (!(ai1->flags & cAtomFlag_ignore)))) {
-                  vLen = (float) diff3f(point, cs->Coord + (cur * 3));
+                  vLen = (float) diff3f(point, cs->coordPtr(cur));
                   dist2vdw = vLen - (ai1->vdw + vdw_add);
                   if(dist2vdw < bestDist) {
                     bestDist = dist2vdw;
@@ -1008,7 +1007,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
                 inSolvFlag = false;
                 /* this point lies within a water radius of the atom, so
                    lets see if it is actually near a water */
-                aLen = MAXFLOAT;
+                aLen = FLT_MAX;
 
                 MapLocus(smap, point, &h, &k, &l);
                 d = *(MapEStart(smap, h, k, l));
@@ -1057,19 +1056,18 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     FreeP(I->Dot);
     OrthoBusyFast(G, 2, 3);
     if(ok) {
-      ok &= IsosurfVolume(G, NULL, NULL, field, 1.0, &I->N, &I->V, NULL, mesh_type, mesh_skip,
+      ok &= IsosurfVolume(G, NULL, NULL, field, 1.0, I->N, I->V, NULL, mesh_type, mesh_skip,
                     1.0F);
     }
-    if (field)
-      IsosurfFieldFree(G, field);
-    if(ok && (I->N && I->V && (carve_flag || clear_flag || trim_flag))) {
+    DeleteP(field);
+    if(ok && (I->N.data() && I->V.data() && (carve_flag || clear_flag || trim_flag))) {
       int cur_size = VLAGetSize(I->N);
       if((mesh_type == 0) && cur_size) {
-        int *n = I->N;
+        int *n = I->N.data();
         int *new_n = VLACalloc(int, cur_size);
         int new_size = 0;
-        float *new_v = I->V;
-        float *v = I->V;
+        float *new_v = I->V.data();
+        float *v = I->V.data();
 	CHECKOK(ok, new_n);
         while(ok && (c = *(n++))) {
           int new_c = 0;
@@ -1165,8 +1163,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
             new_c = 0;
           }
         }
-        VLAFreeP(I->N);
-        I->N = new_n;
+        I->N = pymol::vla_take_ownership(new_n);
       }
     }
     MapFree(trim_map);
@@ -1175,7 +1172,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
     VLAFreeP(trim_vla);
     VLAFreeP(carve_vla);
     VLAFreeP(clear_vla);
-    n = I->N;
+    n = I->N.data();
     I->NTot = 0;
     if (ok){
       while(*n)
@@ -1195,7 +1192,7 @@ Rep *RepMeshNew(CoordSet * cs, int state)
 int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
                           float probe_radius)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj = cs->Obj;
   int a, b, c, a1, a2, flag, i, h, k, l, j;
   int ok = true;
@@ -1209,7 +1206,7 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
   int cnt;
   int inclH, mesh_mode, cullByFlag;
   AtomInfoType *ai1, *ai2;
-  int ds = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_quality);
+  int ds = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_quality);
 
   if(ds < 0)
     ds = 0;
@@ -1217,13 +1214,13 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
     ds = 4;
   sp = G->Sphere->Sphere[ds];
 
-  cavity_cull = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_cavity_cull);
+  cavity_cull = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_cavity_cull);
 
-  mesh_mode = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_mesh_mode);
+  mesh_mode = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_mesh_mode);
   cullByFlag = (mesh_mode == cRepMesh_by_flags);
   inclH = !(mesh_mode == cRepMesh_heavy_atoms);
 
-  I->Dot = (float *) mmalloc(sizeof(float) * cs->NIndex * 3 * sp->nDot);
+  I->Dot = pymol::malloc<float>(cs->NIndex * 3 * sp->nDot);
   ErrChkPtr(G, I->Dot);
 
   probe_radius_plus = probe_radius * 1.5F;
@@ -1242,7 +1239,7 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
         OrthoBusyFast(G, a, cs->NIndex * 3);
         dotCnt = 0;
         a1 = cs->IdxToAtm[a];
-        v0 = cs->Coord + 3 * a;
+        v0 = cs->coordPtr(a);
         vdw = cs->Obj->AtomInfo[a1].vdw + probe_radius;
         inFlag = true;
         for(c = 0; c < 3; c++) {
@@ -1273,7 +1270,7 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
                   if(j != a) {
                     a2 = cs->IdxToAtm[j];
                     if(within3f
-                       (cs->Coord + 3 * j, v, cs->Obj->AtomInfo[a2].vdw + probe_radius)) {
+                       (cs->coordPtr(j), v, cs->Obj->AtomInfo[a2].vdw + probe_radius)) {
                       flag = false;
                       break;
                     }
@@ -1301,7 +1298,7 @@ int RepMeshGetSolventDots(RepMesh * I, CoordSet * cs, float *min, float *max,
   }
 
   if(ok && (cavity_cull > 0)) {
-    dot_flag = Alloc(int, I->NDot);
+    dot_flag = pymol::malloc<int>(I->NDot);
     ErrChkPtr(G, dot_flag);
     for(a = 0; a < I->NDot; a++) {
       dot_flag[a] = 0;
