@@ -140,7 +140,7 @@ static void RepRibbonRender(RepRibbon * I, RenderInfo * info)
 
 Rep *RepRibbonNew(CoordSet * cs, int state)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj;
   int a, b, a1, a2, c1, c2, *i, *s, *at, *seg, nAt, *atp;
   float *v, *v1, *v2, *v3;
@@ -174,35 +174,35 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
   obj = cs->Obj;
 
   RepInit(G, &I->R);
-  power_a = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_power);
-  power_b = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_power_b);
-  throw_ = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_throw);
-  int trace_ostate = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_trace_atoms);
-  trace_mode = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_trace_atoms_mode);
+  power_a = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_power);
+  power_b = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_power_b);
+  throw_ = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_throw);
+  int trace_ostate = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_trace_atoms);
+  trace_mode = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_trace_atoms_mode);
   na_mode =
-    SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_nucleic_acid_mode);
+    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_nucleic_acid_mode);
 
   ribbon_color =
-    SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_color);
+    SettingGet_color(G, cs->Setting, obj->Setting, cSetting_ribbon_color);
 
-  sampling = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_sampling);
+  sampling = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_sampling);
   if(sampling < 1)
     sampling = 1;
-  I->radius = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_radius);
+  I->radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_radius);
   I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepRibbonRender;
   I->R.fFree = (void (*)(struct Rep *)) RepRibbonFree;
   I->R.fRecolor = NULL;
   I->R.obj = (CObject *) obj;
   I->R.cs = cs;
-  I->ribbon_width = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_width);
-  I->R.context.object = (void *) obj;
+  I->ribbon_width = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_width);
+  I->R.context.object = obj;
   I->R.context.state = state;
 
   /* find all of the CA points */
 
-  at = Alloc(int, cs->NAtIndex * 2);
-  pv = Alloc(float, cs->NAtIndex * 6);
-  seg = Alloc(int, cs->NAtIndex * 2);
+  at = pymol::malloc<int>(cs->NAtIndex * 2);
+  pv = pymol::malloc<float>(cs->NAtIndex * 6);
+  seg = pymol::malloc<int>(cs->NAtIndex * 2);
 
   i = at;
   v = pv;
@@ -212,13 +212,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
   nSeg = 0;
   a2 = -1;
   for(a1 = 0; a1 < cs->NAtIndex; a1++) {
-    if(obj->DiscreteFlag) {
-      if(cs == obj->DiscreteCSet[a1])
-        a = obj->DiscreteAtmToIdx[a1];
-      else
-        a = -1;
-    } else
-      a = cs->AtmToIdx[a1];
+    a = cs->atmToIdx(a1);
     if(a >= 0) {
       ai = obj->AtomInfo + a1;
       if(GET_BIT(obj->AtomInfo[a1].visRep,cRepRibbon)) {
@@ -254,7 +248,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
           *(s++) = nSeg;
           nAt++;
           *(i++) = a;
-          v1 = cs->Coord + 3 * a;
+          v1 = cs->coordPtr(a);
           *(v++) = *(v1++);
           *(v++) = *(v1++);
           *(v++) = *(v1++);
@@ -273,7 +267,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
                 *(s++) = nSeg;
                 nAt++;
                 *(i++) = trailing_O3p_a;
-                v1 = cs->Coord + 3 * trailing_O3p_a;
+                v1 = cs->coordPtr(trailing_O3p_a);
                 *(v++) = *(v1++);
                 *(v++) = *(v1++);
                 *(v++) = *(v1++);
@@ -291,7 +285,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
               *(s++) = nSeg;
               nAt++;
               *(i++) = leading_O5p_a;
-              v1 = cs->Coord + 3 * leading_O5p_a;
+              v1 = cs->coordPtr(leading_O5p_a);
               *(v++) = *(v1++);
               *(v++) = *(v1++);
               *(v++) = *(v1++);
@@ -305,7 +299,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
           *(s++) = nSeg;
           nAt++;
           *(i++) = a;
-          v1 = cs->Coord + 3 * a;
+          v1 = cs->coordPtr(a);
           *(v++) = *(v1++);
           *(v++) = *(v1++);
           *(v++) = *(v1++);
@@ -339,7 +333,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
     *(s++) = nSeg;
     nAt++;
     *(i++) = trailing_O3p_a;
-    v1 = cs->Coord + 3 * trailing_O3p_a;
+    v1 = cs->coordPtr(trailing_O3p_a);
     *(v++) = *(v1++);
     *(v++) = *(v1++);
     *(v++) = *(v1++);
@@ -353,9 +347,9 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
     s = seg;
     v = pv;
 
-    dv = Alloc(float, nAt * 6);
-    nv = Alloc(float, nAt * 6);
-    dl = Alloc(float, nAt * 2);
+    dv = pymol::malloc<float>(nAt * 6);
+    nv = pymol::malloc<float>(nAt * 6);
+    dl = pymol::malloc<float>(nAt * 2);
     v1 = dv;
     v2 = nv;
     d = dl;
@@ -386,7 +380,7 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
     s = seg;
     v = nv;
 
-    tv = Alloc(float, nAt * 6 + 6);
+    tv = pymol::malloc<float>(nAt * 6 + 6);
     v1 = tv;
 
     *(v1++) = *(v++);           /* first segment */
@@ -443,7 +437,6 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
     // This is required for immediate mode rendering
     CGOBegin(I->primitiveCGO, GL_LINES);
 
-    bool first = true;
     int atm1, atm2;
     float origV1[14], origV2[14], *origV = origV2;
     bool origVis1 = false;
@@ -522,13 +515,11 @@ Rep *RepRibbonNew(CoordSet * cs, int state)
 
           isRamped = ColorGetCheckRamped(G, c2, origV + 11, origV + 8, state) || isRamped;
 
-          if (first || I->primitiveCGO->interpolated!=isRamped)
-            CGOInterpolated(I->primitiveCGO, isRamped );
           int atm1pk = ai1->masked ? cPickableNoPick : cPickableAtom;
           int atm2pk = ai2->masked ? cPickableNoPick : cPickableAtom;
           CGOPickColor(I->primitiveCGO, at1, atm1pk);
           CGOColorv(I->primitiveCGO, origV + 1);
-          I->primitiveCGO->add<cgo::draw::splitline>(origV + 4, origV + 11, origV + 8, at2, atm2pk, false, false, false);
+          I->primitiveCGO->add<cgo::draw::splitline>(origV + 4, origV + 11, origV + 8, at2, atm2pk, isRamped, false, false);
         }
       }
       v1 += 3;
@@ -563,24 +554,23 @@ void RepRibbonRenderImmediate(CoordSet * cs, RenderInfo * info)
 #ifndef PURE_OPENGL_ES_2
   /* performance optimized to provide a simple C-alpha trace -- no smoothing */
 
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   if(info->ray || info->pick || (!(G->HaveGUI && G->ValidContext)))
     return;
   else {
     ObjectMolecule *obj = cs->Obj;
     int active = false;
     int nAtIndex = cs->NAtIndex;
-    int a;
-    AtomInfoType *obj_AtomInfo = obj->AtomInfo;
-    AtomInfoType *ai, *last_ai = NULL;
+    const AtomInfoType *obj_AtomInfo = obj->AtomInfo.data();
+    const AtomInfoType *ai, *last_ai = NULL;
     int trace, trace_ostate =
-      SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_trace_atoms);
+      SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_trace_atoms);
     int trace_mode =
-      SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_trace_atoms_mode);
+      SettingGet_i(G, cs->Setting, obj->Setting, cSetting_trace_atoms_mode);
     int na_mode =
-      SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_nucleic_acid_mode);
+      SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_nucleic_acid_mode);
     float ribbon_width =
-      SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_width);
+      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_ribbon_width);
     int a1, a2 = -1;
     int color, last_color = -9;
 
@@ -590,13 +580,7 @@ void RepRibbonRenderImmediate(CoordSet * cs, RenderInfo * info)
       glDisable(GL_LIGHTING);
     glBegin(GL_LINE_STRIP);
     for(a1 = 0; a1 < nAtIndex; a1++) {
-      if(obj->DiscreteFlag) {
-        if(cs == obj->DiscreteCSet[a1])
-          a = obj->DiscreteAtmToIdx[a1];
-        else
-          a = -1;
-      } else
-        a = cs->AtmToIdx[a1];
+      auto a = cs->atmToIdx(a1);
       if(a >= 0) {
         ai = obj_AtomInfo + a1;
         if(GET_BIT(ai->visRep,cRepRibbon)) {
@@ -624,7 +608,7 @@ void RepRibbonRenderImmediate(CoordSet * cs, RenderInfo * info)
               last_color = color;
               glColor3fv(ColorGet(G, color));
             }
-            glVertex3fv(cs->Coord + 3 * a);
+            glVertex3fv(cs->coordPtr(a));
             active = true;
             last_ai = ai;
             a2 = a1;
@@ -649,7 +633,7 @@ void RepRibbonRenderImmediate(CoordSet * cs, RenderInfo * info)
               last_color = color;
               glColor3fv(ColorGet(G, color));
             }
-            glVertex3fv(cs->Coord + 3 * a);
+            glVertex3fv(cs->coordPtr(a));
             active = true;
             last_ai = ai;
             a2 = a1;
