@@ -189,6 +189,11 @@ static void RepSphereRender(RepSphere * I, RenderInfo * info)
     if(pick) {
       RepSphereRenderPick(I, info, sphere_mode);
     } else {                    /* not pick, render! */
+      if (I->spheroidCGO) {
+        CGORenderGL(I->spheroidCGO, NULL, NULL, NULL, info, &I->R);
+        return;
+      }
+
 #ifdef _PYMOL_ARB_SHADERS
       if (sphere_mode == 5){
         // we need to check sphere_mode 5 here until
@@ -296,10 +301,10 @@ static void RepSphereAddAtomVisInfoToStoredVC(RepSphere *I, ObjectMolecule *obj,
     float sphere_scale, int sphere_color, float transp,
     int *variable_alpha, float sphere_add)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   float at_transp = transp;
   int c1;
-  float *v0, vc[3];
+  float vc[3];
   const float *vcptr;
 
   float at_sphere_scale = AtomSettingGetWD(G, ati1, cSetting_sphere_scale, sphere_scale);
@@ -307,14 +312,25 @@ static void RepSphereAddAtomVisInfoToStoredVC(RepSphere *I, ObjectMolecule *obj,
 
   if(AtomSettingGetIfDefined(G, ati1, cSetting_sphere_transparency, &at_transp))
     *variable_alpha = true;
-  
-  CGOPickColor(I->primitiveCGO, a1, ati1->masked ? cPickableNoPick : cPickableAtom);
-  
+
+  int trans_pick_mode = SettingGet<int>(
+      G, cs->Setting, obj->Setting, cSetting_transparency_picking_mode);
+
+  int pickmode = cPickableAtom;
+  if (trans_pick_mode != cTransparencyPickingModePickable &&
+      at_transp > PICKABLE_THROUGH_CUTOFF) {
+    pickmode = cPickableThrough;
+  } else if (ati1->masked) {
+    pickmode = cPickableNoPick;
+  }
+
+  CGOPickColor(I->primitiveCGO, a1, pickmode);
+
   if(at_sphere_color == -1)
     c1 = ati1->color;
   else
     c1 = at_sphere_color;
-  v0 = cs->Coord + 3 * a;
+  const float* v0 = cs->coordPtr(a);
 
   if(ColorCheckRamped(G, c1)) {
     ColorGetRamped(G, c1, v0, vc, state);
@@ -350,7 +366,7 @@ static float SphereComputeCutMultiplier(SphereRec *sr){
  *
  */
 static void RepSphereCGOSetSphereColorAndPick(ObjectMolecule *obj, CoordSet * cs, CGO *cgo, int idx, int state, float transp, int sphere_color){
-  PyMOLGlobals *G = obj->Obj.G;
+  PyMOLGlobals *G = obj->G;
   int a1 = cs->IdxToAtm[idx];
   float at_transp;
   AtomInfoType *ati1 = obj->AtomInfo + a1;
@@ -367,7 +383,7 @@ static void RepSphereCGOSetSphereColorAndPick(ObjectMolecule *obj, CoordSet * cs
   else
     c1 = at_sphere_color;
   if(ColorCheckRamped(G, c1)) {
-    float *v0 = cs->Coord + 3 * idx;
+    const float* v0 = cs->coordPtr(idx);
     float color[3];
     ColorGetRamped(G, c1, v0, color, state);
     CGOColorv(cgo, color);
@@ -384,12 +400,12 @@ CGO *RepSphereGeneratespheroidCGO(ObjectMolecule * I, CoordSet *cs, SphereRec *s
   int *q, *s;
   bool ok = true;
   float spheroid_scale =
-    SettingGet_f(I->Obj.G, cs->Setting, I->Obj.Setting, cSetting_spheroid_scale);
+    SettingGet_f(I->G, cs->Setting, I->Setting, cSetting_spheroid_scale);
   int sphere_color =
-    SettingGet_color(I->Obj.G, cs->Setting, I->Obj.Setting, cSetting_sphere_color);
-  float transp = SettingGet_f(I->Obj.G, cs->Setting, I->Obj.Setting, cSetting_sphere_transparency);
+    SettingGet_color(I->G, cs->Setting, I->Setting, cSetting_sphere_color);
+  float transp = SettingGet_f(I->G, cs->Setting, I->Setting, cSetting_sphere_transparency);
 
-  CGO *cgo = CGONew(I->Obj.G);
+  CGO *cgo = CGONew(I->G);
   for(idx = 0; idx < cs->NIndex; idx++) {
     float *v0 = &cs->Coord[3 * idx];
     a = cs->IdxToAtm[idx];
@@ -397,8 +413,8 @@ CGO *RepSphereGeneratespheroidCGO(ObjectMolecule * I, CoordSet *cs, SphereRec *s
     s = sp->StripLen;
     RepSphereCGOSetSphereColorAndPick(I, cs, cgo, idx, state, transp, sphere_color);
     for(b = 0; ok && b < sp->NStrip; b++) {
-      float *sphLen = cs->Spheroid + (sp->nDot * a);
-      float *sphNorm = cs->SpheroidNormal + (3 * sp->nDot * a);
+      float *sphLen = cs->Spheroid.data() + (sp->nDot * a);
+      float *sphNorm = cs->SpheroidNormal.data() + (3 * sp->nDot * a);
       CGOBegin(cgo, GL_TRIANGLE_STRIP);
       for(c = 0; c < (*s); c++) {
         float sphTmp, *sphTmpN = sphNorm + 3 * (*q);
@@ -412,7 +428,7 @@ CGO *RepSphereGeneratespheroidCGO(ObjectMolecule * I, CoordSet *cs, SphereRec *s
       }
       CGOEnd(cgo);
       s++;
-      ok &= !I->Obj.G->Interrupt;
+      ok &= !I->G->Interrupt;
     }
   }
   CGOStop(cgo);
@@ -485,7 +501,7 @@ static void RepSphereSetNormalForSphere(RepSphere *I, MapType *map, float *v_tmp
 
 Rep *RepSphereNew(CoordSet * cs, int state)
 {
-  PyMOLGlobals *G = cs->State.G;
+  PyMOLGlobals *G = cs->G;
   ObjectMolecule *obj;
   int ok = true;
   int a, a1;
@@ -512,30 +528,30 @@ Rep *RepSphereNew(CoordSet * cs, int state)
     return NULL;
   obj = cs->Obj;
 
-  marked = Calloc(bool, obj->NAtom);
+  marked = pymol::calloc<bool>(obj->NAtom);
   CHECKOK(ok, marked);
   if (ok)
     RepInit(G, &I->R);
   I->renderCGO = NULL;
   I->primitiveCGO = NULL;
-  if (cs->Spheroid)
-    I->spheroidCGO = RepSphereGeneratespheroidCGO(obj, cs, G->Sphere->Sphere[1], state);
+  if (!cs->Spheroid.empty())
+    I->spheroidCGO = RepSphereGeneratespheroidCGO(obj, cs, GetSpheroidSphereRec(G), state);
 
   if (ok){
-    sphere_mode = SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_sphere_mode);
+    sphere_mode = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sphere_mode);
     if (!use_shader && (sphere_mode == 5 || sphere_mode == 9)){
       sphere_mode = 0;
     }
   }
   if (ok){
     sphere_color =
-      SettingGet_color(G, cs->Setting, obj->Obj.Setting, cSetting_sphere_color);
+      SettingGet_color(G, cs->Setting, obj->Setting, cSetting_sphere_color);
     cartoon_side_chain_helper =
-      SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_cartoon_side_chain_helper);
+      SettingGet_b(G, cs->Setting, obj->Setting, cSetting_cartoon_side_chain_helper);
     ribbon_side_chain_helper =
-      SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_ribbon_side_chain_helper);
-    transp = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_sphere_transparency);
-    sphere_scale = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_sphere_scale);
+      SettingGet_b(G, cs->Setting, obj->Setting, cSetting_ribbon_side_chain_helper);
+    transp = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sphere_transparency);
+    sphere_scale = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sphere_scale);
   }
 
   if (ok){
@@ -544,18 +560,18 @@ Rep *RepSphereNew(CoordSet * cs, int state)
     I->R.fSameVis = (int (*)(struct Rep *, struct CoordSet *)) RepSphereSameVis;
     I->R.obj = (CObject *) obj;
     I->R.cs = cs;
-    I->R.context.object = (void *) obj;
+    I->R.context.object = obj;
     I->R.context.state = state;
   }
   /* raytracing primitives */
 
   if (ok){
-    if(SettingGet_i(G, cs->Setting, obj->Obj.Setting, cSetting_sphere_solvent)) { /* are we generating a solvent surface? */
-      sphere_add = SettingGet_f(G, cs->Setting, obj->Obj.Setting, cSetting_solvent_radius);       /* if so, get solvent radius */
+    if(SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sphere_solvent)) { /* are we generating a solvent surface? */
+      sphere_add = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_solvent_radius);       /* if so, get solvent radius */
     }
     
-    if(SettingGet_b(G, cs->Setting, obj->Obj.Setting, cSetting_pickable)) {
-      I->R.P = Alloc(Pickable, cs->NIndex + 1);
+    if(SettingGet_b(G, cs->Setting, obj->Setting, cSetting_pickable)) {
+      I->R.P = pymol::malloc<Pickable>(cs->NIndex + 1);
       CHECKOK(ok, I->R.P);
     }
   }
@@ -576,7 +592,7 @@ Rep *RepSphereNew(CoordSet * cs, int state)
         int cnc = nspheres * 3;
         nspheres++;
         VLACheck(v_tmp, float, cnc + 3);
-        copy3f(cs->Coord + 3 * a, &v_tmp[cnc]);
+        copy3f(cs->coordPtr(a), &v_tmp[cnc]);
     }
     ok &= !G->Interrupt;
   }
@@ -584,7 +600,7 @@ Rep *RepSphereNew(CoordSet * cs, int state)
     float cut_mult = SphereComputeCutMultiplier(G->Sphere->Sphere[1]);
     float *dot = G->Sphere->Sphere[1]->dot[0];
     int n_dot = G->Sphere->Sphere[1]->nDot;
-    int *active = Alloc(int, 2 * n_dot);
+    int *active = pymol::malloc<int>(2 * n_dot);
 
 	ok &= MapSetupExpress(map);
     for(a = 0; ok && a < cs->NIndex; a++) {
@@ -618,16 +634,16 @@ Rep *RepSphereNew(CoordSet * cs, int state)
 
   if(ok) {
     if(!I->LastVisib)
-      I->LastVisib = Alloc(bool, cs->NIndex);
+      I->LastVisib = pymol::malloc<bool>(cs->NIndex);
     CHECKOK(ok, I->LastVisib);
     if(ok && !I->LastColor)
-      I->LastColor = Alloc(int, cs->NIndex);
+      I->LastColor = pymol::malloc<int>(cs->NIndex);
     CHECKOK(ok, I->LastColor);
     if (ok){
       lv = I->LastVisib;
       lc = I->LastColor;
       obj = cs->Obj;
-      AtomInfoType *ai2 = obj->AtomInfo;
+      const AtomInfoType *ai2 = obj->AtomInfo.data();
       if(sphere_color == -1){
 	for(a = 0; a < cs->NIndex; a++) {
           int at = cs->IdxToAtm[a];
