@@ -39,13 +39,22 @@
 #include "Property.h"
 #endif
 
-/*
+#ifdef _PYMOL_IP_PROPERTIES
+#define _PYMOL_MAE_PROP_EXPORT
+#endif
+
+/**
  * Get the capitalized element symbol. Assume that ai->elem is either all
  * caps (e.g. loaded from PDB) or capitalized.
  */
 class ElemCanonicalizer {
   ElemName m_buffer;
 public:
+  /**
+   * Returns either a pointer to AtomInfoType::elem, if it's already in
+   * canonical form, or to an internal buffer which stays valid until
+   * operator()() is called again.
+   */
   const char * operator() (const AtomInfoType * ai) {
     const char * elem = ai->elem;
     if (ai->protons < 1 || !elem[0] || !elem[1] || islower(elem[1]))
@@ -56,11 +65,15 @@ public:
   }
 };
 
-/*
+/**
  * "sprintf" into VLA string buffer. Will grow (and reallocate) the VLA if
  * needed.
  *
- * Returns the number of characters written (excluding the null byte).
+ * @param vla Buffer to write to (at `offset`)
+ * @param offset Buffer offset
+ * @param format See `printf`
+ *
+ * @return Number of characters written (excluding the null byte)
  */
 static
 int VLAprintf(pymol::vla<char>& vla, int offset, const char * format, ...) {
@@ -113,23 +126,24 @@ struct AtomRef {
   int id;
 };
 
-/*
+/**
  * Abstract base class for exporting molecular selections
  */
 struct MoleculeExporter {
-  pymol::vla<char> m_buffer; // out
+  pymol::vla<char> m_buffer; //!< Out buffer and final result
 
 protected:
-  int m_offset;
-  CoordSet        * m_last_cs;
-  ObjectMolecule  * m_last_obj;
-  int m_last_state;
+  int m_offset = 0; //!< Offset into `m_buffer`
+
+  CoordSet* m_last_cs = nullptr;
+  ObjectMolecule* m_last_obj = nullptr;
+  int m_last_state = -1;
 
   PyMOLGlobals * G;
   SeleCoordIterator m_iter;
 
-  bool m_retain_ids;
-  int m_id;
+  bool m_retain_ids = false;
+  int m_id = 0;
 
   struct matrix_t {
     double storage[16];
@@ -138,20 +152,20 @@ protected:
     m_mat_full, // for ANISOU
     m_mat_move; // for coordinates (TTT)
 
-  // atom coordinate
 private:
   float m_coord_tmp[3];
 protected:
+  /// Current atom coordinates
   const float *m_coord;
 
-  // how to restart models
+  /// How to restart models
   int m_multi;
 
   std::vector<BondRef> m_bonds;
   std::vector<int> m_tmpids;
 
 public:
-  // quasi constructor (easier to inherit than a real constructor)
+  /// Quasi constructor (easier to inherit than a real constructor)
   virtual void init(PyMOLGlobals * G_) {
     G = G_;
 
@@ -159,12 +173,6 @@ public:
     m_buffer[0] = '\0';
 
     m_mat_ref.ptr = nullptr;
-    m_offset = 0;
-    m_last_cs  = nullptr;
-    m_last_obj = nullptr;
-    m_last_state = -1;
-    m_retain_ids = false;
-    m_id = 0;
 
     setMulti(getMultiDefault());
   }
@@ -172,7 +180,7 @@ public:
   // destructor
   virtual ~MoleculeExporter() = default;
 
-  /*
+  /**
    * Do the export (e.g. populate "m_buffer" with file contents)
    */
   void execute(int sele, int state);
@@ -186,7 +194,7 @@ public:
   }
 
 private:
-  /*
+  /**
    * Reset the "index" fields in the selector table
    */
   void resetTmpIDs() {
@@ -195,7 +203,7 @@ private:
   }
 
 protected:
-  /*
+  /**
    * Get the current atom's running index (1-based). Most formats use
    * this index to reference atoms from the bonds table. If "retain_ids"
    * is true, then this is AtomInfoType::id.
@@ -204,36 +212,36 @@ protected:
     return m_tmpids[m_iter.getAtm()];
   }
 
-  /*
+  /**
    * Get the running index for the given atom.
    */
   int getTmpID(int atm) {
     return m_tmpids[atm];
   }
 
-  /*
+  /**
    * Get the current state title if not empty, or the object name otherwise.
    */
   const char * getTitleOrName() {
     if (!m_iter.cs)
       return "untitled";
-    return m_iter.cs->Name[0] ? m_iter.cs->Name : m_iter.obj->Obj.Name;
+    return m_iter.cs->Name[0] ? m_iter.cs->Name : m_iter.obj->Name;
   }
 
 public:
-  /*
+  /**
    * Set the reference coordinate frame to the inverse of the given
    * object's transformation matrix.
    */
   void setRefObject(const char * ref_object, int ref_state);
 
 private:
-  /*
+  /**
    * Update a transformation matrix for the current state
    */
   void updateMatrix(matrix_t& matrix, bool history);
 
-  /*
+  /**
    * Populates the "m_bonds" vector
    */
   void populateBondRefs();
@@ -253,7 +261,7 @@ protected:
   virtual void beginFile() {}
 };
 
-/*
+/**
  * Return true if this bond should not be exported
  */
 bool MoleculeExporter::isExcludedBond(int atm1, int atm2) {
@@ -301,7 +309,7 @@ void MoleculeExporter::endCoordSet() {
 }
 
 void MoleculeExporter::execute(int sele, int state) {
-  m_iter.init(G, sele, state);
+  m_iter = SeleCoordIterator(G, sele, state);
   m_iter.setPerObject(m_multi != cMolExportGlobal);
 
   beginFile();
@@ -373,8 +381,8 @@ void MoleculeExporter::setRefObject(const char * ref_object, int ref_state) {
   if (!base)
     return;
 
-  if(ref_state < 0) {
-    ref_state = ObjectGetCurrentState(base, true);
+  if (ref_state == -1 /* all */) {
+    ref_state = -2; // current
   }
 
   if(ObjectGetTotalMatrix(base, ref_state, true, matrix)) {
@@ -400,7 +408,7 @@ void MoleculeExporter::populateBondRefs() {
   auto& obj = m_last_obj;
   int id1, id2;
 
-  for (auto bond = obj->Bond, bond_end = obj->Bond + obj->NBond;
+  for (auto bond = obj->Bond.data(), bond_end = obj->Bond.data() + obj->NBond;
       bond != bond_end; ++bond) {
 
     auto atm1 = bond->index[0];
@@ -424,9 +432,9 @@ void MoleculeExporter::populateBondRefs() {
 // ---------------------------------------------------------------------------------- //
 
 struct MoleculeExporterPDB : public MoleculeExporter {
-  bool m_conect_all;
+  bool m_conect_all = false;
   bool m_conect_nodup;
-  bool m_mdl_written;
+  bool m_mdl_written = false;
   bool m_use_ter_records;
   const AtomInfoType * m_pre_ter = nullptr;
   PDBInfoRec m_pdb_info;
@@ -437,8 +445,6 @@ struct MoleculeExporterPDB : public MoleculeExporter {
 
     UtilZeroMem((void *) &m_pdb_info, sizeof(PDBInfoRec));
 
-    m_conect_all    = false;
-    m_mdl_written   = false;
     m_conect_nodup  = SettingGetGlobal_b(G, cSetting_pdb_conect_nodup);
     m_retain_ids    = SettingGetGlobal_b(G, cSetting_pdb_retain_ids);
     m_use_ter_records = SettingGetGlobal_b(G, cSetting_pdb_use_ter_records);
@@ -463,7 +469,7 @@ struct MoleculeExporterPDB : public MoleculeExporter {
     }
   }
 
-  /*
+  /**
    * Write a TER record if the previous atom was polymer and `ai`
    * is NULL, non-polymer, or has a different chain identifier.
    */
@@ -521,11 +527,11 @@ struct MoleculeExporterPDB : public MoleculeExporter {
   }
 
   void writeCryst1() {
-    const auto& sym = m_iter.cs->Symmetry ? m_iter.cs->Symmetry : m_iter.obj->Symmetry;
+    const auto& sym = m_iter.cs->Symmetry ? m_iter.cs->Symmetry.get(): m_iter.obj->Symmetry;
 
-    if (sym && sym->Crystal) {
-      const auto& dim   = sym->Crystal->Dim;
-      const auto& angle = sym->Crystal->Angle;
+    if (sym) {
+      const auto& dim   = sym->Crystal.Dim;
+      const auto& angle = sym->Crystal.Angle;
       m_offset += VLAprintf(m_buffer, m_offset,
           "CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4d\n",
           dim[0], dim[1], dim[2], angle[0], angle[1], angle[2],
@@ -536,10 +542,10 @@ struct MoleculeExporterPDB : public MoleculeExporter {
   void beginObject() override {
     MoleculeExporter::beginObject();
 
-    m_conect_all = SettingGet_b(G, m_iter.obj->Obj.Setting, nullptr, cSetting_pdb_conect_all);
+    m_conect_all = SettingGet_b(G, m_iter.obj->Setting, nullptr, cSetting_pdb_conect_all);
 
     if (m_multi == cMolExportByObject) {
-      m_offset += VLAprintf(m_buffer, m_offset, "HEADER    %.40s\n", m_iter.obj->Obj.Name);
+      m_offset += VLAprintf(m_buffer, m_offset, "HEADER    %.40s\n", m_iter.obj->Name);
       writeCryst1();
     }
   }
@@ -587,6 +593,11 @@ struct MoleculeExporterPQR: public MoleculeExporterPDB {
     m_pdb_info.pqr_workarounds = SettingGetGlobal_b(G, cSetting_pqr_workarounds);
   }
 
+  // don't write MODEL records
+  void beginCoordSet() override {
+    MoleculeExporter::beginCoordSet();
+  }
+
   bool isExcludedBond(int atm1, int atm2) override {
     // no bonds for PQR format
     return true;
@@ -596,7 +607,7 @@ struct MoleculeExporterPQR: public MoleculeExporterPDB {
 // ---------------------------------------------------------------------------------- //
 
 struct MoleculeExporterCIF : public MoleculeExporter {
-  const char * m_molecule_name;
+  const char* m_molecule_name = "multi";
   CifDataValueFormatter cifrepr;
 
   // quasi constructor
@@ -609,7 +620,6 @@ struct MoleculeExporterCIF : public MoleculeExporter {
     cifrepr.m_buf.resize(10);
 
     m_retain_ids    = SettingGetGlobal_b(G, cSetting_pdb_retain_ids);
-    m_molecule_name = "multi";
 
     m_offset += VLAprintf(m_buffer, m_offset,
         "# generated by PyMOL " _PyMOL_VERSION "\n");
@@ -620,11 +630,11 @@ struct MoleculeExporterCIF : public MoleculeExporter {
   }
 
   void writeCellSymmetry() {
-    const auto& sym = m_iter.cs->Symmetry ? m_iter.cs->Symmetry : m_iter.obj->Symmetry;
+    const auto& sym = m_iter.cs->Symmetry.get() ? m_iter.cs->Symmetry.get() : m_iter.obj->Symmetry;
 
-    if (sym && sym->Crystal) {
-      const auto& dim   = sym->Crystal->Dim;
-      const auto& angle = sym->Crystal->Angle;
+    if (sym) {
+      const auto& dim   = sym->Crystal.Dim;
+      const auto& angle = sym->Crystal.Angle;
       m_offset += VLAprintf(m_buffer, m_offset, "#\n"
           "_cell.entry_id %s\n"
           "_cell.length_a %.3f\n"
@@ -646,7 +656,7 @@ struct MoleculeExporterCIF : public MoleculeExporter {
     MoleculeExporter::beginMolecule();
 
     switch (m_multi) {
-      case cMolExportByObject:   m_molecule_name = m_iter.obj->Obj.Name; break;
+      case cMolExportByObject:   m_molecule_name = m_iter.obj->Name; break;
       case cMolExportByCoordSet: m_molecule_name = getTitleOrName(); break;
     }
 
@@ -717,25 +727,29 @@ struct MoleculeExporterCIF : public MoleculeExporter {
         m_iter.state + 1);
   }
 
+  /**
+   * @todo TODO Not implemented
+   * Bond categories:
+   * 1) within residue -> _chem_comp_bond
+   * 2) polymer links -> implicit
+   * 3) others -> _struct_conn
+   */
   void writeBonds() override {
-    // TODO
-    // Bond categories:
-    // 1) within residue -> _chem_comp_bond
-    // 2) polymer links -> implicit
-    // 3) others -> _struct_conn
     m_bonds.clear();
   }
 
+  /**
+   * @todo TODO Not implemented
+   * Exclude polymer links
+   */
   bool isExcludedBond(int atm1, int atm2) override {
-    // TODO
-    // polymer links
     return true;
   }
 };
 
 // ---------------------------------------------------------------------------------- //
 
-/*
+/**
  * Experimental PyMOL style annotated mmCIF. Exports colors, representation,
  * and secondary structure.
  */
@@ -1061,14 +1075,61 @@ struct MoleculeExporterMOL2 : public MoleculeExporter {
 struct MoleculeExporterMAE : public MoleculeExporter {
   int m_n_atoms;
   int m_n_atoms_offset;
-  int m_n_arom_bonds;
+  int m_n_arom_bonds = 0;
   std::map<int, const AtomInfoType *> m_atoms;
+  bool m_has_anisou;
+
+#ifdef _PYMOL_MAE_PROP_EXPORT
+#endif
+
+  /** Check if current object has any ANISOU data
+   */
+  bool currentObjectHasAnisou() const {
+    for (auto i = 0; i < m_iter.obj->NAtom; ++i) {
+      if (m_iter.obj->AtomInfo[i].has_anisou()) {
+        return true;
+      }
+    }
+
+    return false;
+  }
+
+  /** Write a single string value to the output buffer
+   */
+  void writeMaeValue(const char * s) {
+    auto s_quoted = MaeExportStrRepr(s);
+    m_offset += VLAprintf(m_buffer, m_offset, "%s\n", s_quoted.c_str());
+  }
+
+  /** Write the given keys to the output buffer.
+   * Add type prefix and make key unique if necessary.
+   */
+  void writeMaeKeys(const std::vector<std::string> &keys) {
+    std::set<std::string> unique_keys;
+
+    for (auto key : keys) {
+      // check for "<type>_" prefix or add one
+      if (key.size() < 2 || key[1] != '_' || !strchr("irsb", key[0])) {
+        key = "s_pymol_" + key;
+      }
+
+      auto key_size = key.size();
+
+      // make key unique (append numbers)
+      for (unsigned i = 1; unique_keys.count(key); ++i) {
+        key.resize(key_size);
+        key += std::to_string(i);
+      }
+
+      unique_keys.insert(key);
+
+      writeMaeValue(key.c_str());
+    }
+  }
 
   // quasi constructor
   void init(PyMOLGlobals * G_) override {
     MoleculeExporter::init(G_);
-
-    m_n_arom_bonds = 0;
   }
 
   int getMultiDefault() const override {
@@ -1089,43 +1150,91 @@ struct MoleculeExporterMAE : public MoleculeExporter {
 
     m_offset += VLAprintf(m_buffer, m_offset,
         "\nf_m_ct {\n"
-        "s_m_subgroupid\n"
-        "s_m_title\n"
+        );
+
+    std::vector<std::string> keys {
+      "s_m_title",
+    };
+
+    if (!groupid.empty()) {
+      keys.emplace_back("s_m_subgroupid");
+    }
+
+#ifdef _PYMOL_MAE_PROP_EXPORT
+#endif
+
+    writeMaeKeys(keys);
+
+    m_offset += VLAprintf(m_buffer, m_offset,
         ":::\n"
-        "\"%s\"\n"
-        "\"%s\"\n",
-        groupid.c_str(),
-        getTitleOrName());
+        );
+
+    // title may contain spaces or quotes
+    writeMaeValue(getTitleOrName());
+
+    if (!groupid.empty()) {
+      m_offset += VLAprintf(m_buffer, m_offset, "\"%s\"\n", groupid.c_str());
+    }
+
+#ifdef _PYMOL_MAE_PROP_EXPORT
+#endif
 
     // defer until number of atoms known
     m_n_atoms_offset = m_offset;
 
+    keys = {
+      "i_m_mmod_type",
+      "r_m_x_coord",
+      "r_m_y_coord",
+      "r_m_z_coord",
+      "i_m_residue_number",
+      "s_m_insertion_code",
+      "s_m_chain_name",
+      "s_m_pdb_residue_name",
+      "s_m_pdb_atom_name",
+      "i_m_atomic_number",
+      "i_m_formal_charge",
+      "s_m_color_rgb",
+      "i_m_secondary_structure",
+      "r_m_pdb_occupancy",
+      "i_pdb_PDB_serial",
+
+      "r_m_pdb_tfactor",
+      "r_m_charge1",
+
+      "i_m_visibility",
+      "i_m_representation",
+      "i_m_ribbon_style",
+      "i_m_ribbon_color",
+      "s_m_ribbon_color_rgb",
+      "s_m_label_format",
+      "i_m_label_color",
+      "s_m_label_user_text",
+    };
+
+    // ANISOU
+    if ((m_has_anisou = currentObjectHasAnisou())) {
+      keys.insert(keys.end(), {
+        "i_pdb_anisou_u11",
+        "i_pdb_anisou_u22",
+        "i_pdb_anisou_u33",
+        "i_pdb_anisou_u12",
+        "i_pdb_anisou_u13",
+        "i_pdb_anisou_u23",
+      });
+    }
+
     m_offset += VLAprintf(m_buffer, m_offset,
         "m_atom[X]            {\n" // place holder
         "# First column is atom index #\n"
-        "i_m_mmod_type\n"
-        "r_m_x_coord\n"
-        "r_m_y_coord\n"
-        "r_m_z_coord\n"
-        "i_m_residue_number\n"
-        "s_m_insertion_code\n"
-        "s_m_chain_name\n"
-        "s_m_pdb_residue_name\n"
-        "s_m_pdb_atom_name\n"
-        "i_m_atomic_number\n"
-        "i_m_formal_charge\n"
-        "s_m_color_rgb\n"
-        "i_m_secondary_structure\n"
-        "r_m_pdb_occupancy\n"
-        "i_pdb_PDB_serial\n"
-        "i_m_visibility\n"
-        "i_m_representation\n"
-        "i_m_ribbon_style\n"
-        "i_m_ribbon_color\n"
-        "s_m_ribbon_color_rgb\n"
-        "s_m_label_format\n"
-        "i_m_label_color\n"
-        "s_m_label_user_text\n"
+        );
+
+#ifdef _PYMOL_MAE_PROP_EXPORT
+#endif
+
+    writeMaeKeys(keys);
+
+    m_offset += VLAprintf(m_buffer, m_offset,
         ":::\n");
 
     m_n_atoms = 0;
@@ -1152,16 +1261,25 @@ struct MoleculeExporterMAE : public MoleculeExporter {
     if (ai->name)
       AtomInfoGetAlignedPDBAtomName(G, ai, resn, name);
 
+    // 4-letter padding for atom name
+    for (auto i = strlen(name); i < 4; ++i) {
+      name[i] = ' ';
+    }
+    name[4] = '\0';
+
+    // 1-letter padding for chain
+    const char* chain = ai->chain ? LexStr(G, ai->chain) : " ";
+
     m_offset += VLAprintf(m_buffer, m_offset,
-        "%d %d %.3f %.3f %.3f %d %s %s \"%-4s\" \"%-4s\" %d %d %02X%02X%02X %d %.2f %d\n",
+        "%d %d %.3f %.3f %.3f %d %s %s \"%-4s\" %s %d %d %02X%02X%02X %d %.2f %d\n",
         getTmpID(),
         getMacroModelAtomType(ai),
         m_coord[0], m_coord[1], m_coord[2],
         ai->resv,
         inscode,
-        ai->chain ? LexStr(G, ai->chain) : "\" \"",
+        MaeExportStrRepr(chain).c_str(),
         resn,
-        name,
+        MaeExportStrRepr(name).c_str(),
         ai->protons,
         ai->formalCharge,
         int(rgb[0] * 255),
@@ -1171,6 +1289,11 @@ struct MoleculeExporterMAE : public MoleculeExporter {
         ai->q,
         ai->id);
 
+    m_offset += VLAprintf(m_buffer, m_offset,
+        "%.2f %.2f ",
+        ai->b,
+        ai->partialCharge);
+
     char ribbon_color_rgb[7] = "<>";
     MaeExportGetRibbonColor(G, m_iter, ribbon_color_rgb);
     std::string label_user_text = MaeExportGetLabelUserText(G, ai);
@@ -1185,6 +1308,28 @@ struct MoleculeExporterMAE : public MoleculeExporter {
         label_user_text.empty() ? "" : "%UT",
         label_user_text.c_str());
 
+    // ANISOU
+    if (m_has_anisou) {
+      if (ai->has_anisou()) {
+        float anisou[6];
+        std::copy_n(ai->get_anisou(), 6, anisou);
+
+        if (m_mat_full.ptr) {
+          RotateU(m_mat_full.ptr, anisou);
+        }
+
+        m_offset += VLAprintf(m_buffer, m_offset,
+            "%.0f %.0f %.0f %.0f %.0f %.0f\n",
+            anisou[0] * 1e4, anisou[1] * 1e4, anisou[2] * 1e4,
+            anisou[3] * 1e4, anisou[4] * 1e4, anisou[5] * 1e4);
+      } else {
+        m_offset += VLAprintf(m_buffer, m_offset, "<> <> <> <> <> <>\n");
+      }
+    }
+
+#ifdef _PYMOL_MAE_PROP_EXPORT
+#endif
+
     m_atoms[getTmpID()] = ai;
 
     ++m_n_atoms;
@@ -1357,6 +1502,11 @@ public:
     m_raw.occupancyList.emplace_back(ai->q);
     m_raw.altLocList.emplace_back(ai->alt[0]);
 
+    // don't assign to 'm_retain_ids', it messes up bonds
+    if (SettingGetGlobal_b(G, cSetting_pdb_retain_ids)) {
+      m_raw.atomIdList.emplace_back(ai->id);
+    }
+
     m_last_ai = ai;
   }
 
@@ -1389,7 +1539,7 @@ public:
     auto buffer = stream.str();
     auto bufferSize = buffer.size();
     m_buffer.resize(bufferSize);
-    std::memcpy(m_buffer, buffer.data(), bufferSize);
+    std::memcpy(m_buffer.data(), buffer.data(), bufferSize);
     m_offset = bufferSize;
   }
 };
@@ -1397,19 +1547,18 @@ public:
 
 /*========================================================================*/
 
-/*
+/**
  * Export the given selection to a molecular file format.
  *
- * Return the file contents as a VLA (must be VLAFree'd by caller) or
- * NULL if the format is not known.
+ * @return File contents or NULL if the format is not known.
  *
- * format:      pdb, sdf, ...
- * selection:   atom selection expression
- * state:       object state (-1 for all, -2/-3 for current)
- * ref_object:  name of a reference object which defines the frame of
+ * @param format      pdb, sdf, ...
+ * @param selection   atom selection expression
+ * @param state       object state (-1 for all, -2/-3 for current)
+ * @param ref_object  name of a reference object which defines the frame of
  *              reference for exported coordinates
- * ref_state:   reference object state
- * multi:       defines how to handle selections which span multiple objects
+ * @param ref_state   reference object state
+ * @param multi       defines how to handle selections which span multiple objects
  *              -1: use format-specific default
  *               0: one global "molecule" (default for PDB)
  *               1: molecules per objects
@@ -1428,7 +1577,7 @@ pymol::vla<char> MoleculeExporterGetStr(PyMOLGlobals * G,
   int sele = tmpsele1.getIndex();
 
   if (sele < 0)
-    return nullptr;
+    return {};
 
   std::unique_ptr<MoleculeExporter> exporter;
 
@@ -1463,12 +1612,12 @@ pymol::vla<char> MoleculeExporterGetStr(PyMOLGlobals * G,
 #else
     PRINTFB(G, FB_ObjectMolecule, FB_Errors)
       " Error: This build has no fast MMTF support.\n" ENDFB(G);
-    return nullptr;
+    return {};
 #endif
   } else {
     PRINTFB(G, FB_ObjectMolecule, FB_Errors)
       " Error: unknown format: '%s'\n", format ENDFB(G);
-    return nullptr;
+    return {};
   }
 
   // Ensure "." decimal point in printf. It's possible to change this from
@@ -1486,8 +1635,8 @@ pymol::vla<char> MoleculeExporterGetStr(PyMOLGlobals * G,
 /*========================================================================*/
 
 #ifndef _PYMOL_NOPY
-/*
- * See MoleculeExporterGetPyBonds
+/**
+ * See ::MoleculeExporterGetPyBonds
  */
 struct MoleculeExporterPyBonds : public MoleculeExporter {
   PyObject *m_bond_list; // out
@@ -1516,7 +1665,7 @@ protected:
 
 /*========================================================================*/
 
-/*
+/**
  * Get all bonds within the given selection.
  *
  * Returns a list of (atm1, atm2, order) tuples, where atm1 and atm2 are
@@ -1548,7 +1697,7 @@ PyObject *MoleculeExporterGetPyBonds(PyMOLGlobals * G,
 
 /*========================================================================*/
 
-/*
+/**
  * Creates a `chempy.models.Indexed` instance from a selection.
  *
  * Changes from the old implementation (SelectorGetChemPyModel):
@@ -1556,21 +1705,17 @@ PyObject *MoleculeExporterGetPyBonds(PyMOLGlobals * G,
  *
  */
 struct MoleculeExporterChemPy : public MoleculeExporter {
-  PyObject *m_model; // out
+  PyObject *m_model = nullptr; //!< Final result (never NULL)
 
 protected:
-  int m_n_cs; // number of coordinate sets
+  int m_n_cs = 0; //!< Number of coordinate sets
   float m_ref_tmp[3];
-  PyObject *m_atom_list;
+  PyObject* m_atom_list = nullptr;
 
 public:
   // quasi constructor
   void init(PyMOLGlobals * G_) override {
     MoleculeExporter::init(G_);
-
-    m_model = nullptr;
-    m_n_cs = 0;
-    m_atom_list = nullptr;
   }
 
 protected:
@@ -1596,7 +1741,7 @@ protected:
   }
 
   const float * getRefPtr() {
-    RefPosType  * ref_pos = m_iter.cs->RefPos;
+    const RefPosType* ref_pos = m_iter.cs->RefPos.data();
     const float * ref_ptr = nullptr;
 
     if (ref_pos) {
@@ -1683,6 +1828,9 @@ protected:
 
 /*========================================================================*/
 
+/**
+ * Implementation of `cmd.get_model`
+ */
 PyObject *ExecutiveSeleToChemPyModel(PyMOLGlobals * G,
     const char *s1, int state,
     const char *ref_object, int ref_state)
