@@ -11,9 +11,20 @@ Supported ways to launch PyMOL:
 
     shell> python /path/to/pymol/__init__.py [args]
 
+  If the 'pymol' module is in PYTHONPATH
+
+    shell> python -m pymol [args]
+
   From a python main thread:
 
+    >>> # blocks the interpreter
+    >>> import pymol
+    >>> pymol.launch()
+
+  From a python main thread, spawning a new thread:
+
     >>> # with GUI
+    >>> # THIS IS NOT SUPPORTED ON macOS
     >>> import pymol
     >>> pymol.finish_launching()
 
@@ -74,16 +85,6 @@ IS_MACOS = sys.platform.startswith('darwin')
 IS_LINUX = sys.platform.startswith('linux')
 
 if IS_PY3:
-    # legacy string API, still used by Pmw for example
-    import string
-    for attr in ['capitalize', 'count', 'find', 'index', 'lower',
-            'replace', 'rstrip', 'split', 'strip', 'upper', 'zfill']:
-        setattr(string, attr, getattr(str, attr))
-    string.letters      = string.ascii_letters
-    string.lowercase    = string.ascii_lowercase
-    string.join = lambda words, sep=' ': sep.join(words)
-    string.atoi = int
-
     import _thread as thread
 else:
     import thread
@@ -161,7 +162,6 @@ def _init_internals(_pymol):
     # single Python interpeter
 
     _pymol.lock_api = threading.RLock() # mutex for API calls from the outside
-    _pymol.lock_api_c = threading.RLock() # mutex for C management of python threads
     _pymol.lock_api_status = threading.RLock() # mutex for PyMOL status info
     _pymol.lock_api_glut = threading.RLock() # mutex for GLUT avoidance
     _pymol.lock_api_data = threading.RLock() # mutex for internal data structures
@@ -177,8 +177,11 @@ def get_version_message(v=None):
     p += "Incentive Product" if invocation.options.incentive_product else \
          "Open-Source"
 
-    if v[5]:
-        p += ", svn rev %s" % v[5]
+    if v[4]:
+        p += ' (' + v[4][:10] + ')'
+
+    if v[3]:
+        p += ', ' + time.strftime('%Y-%m-%d', time.localtime(v[3]))
 
     return p
 
@@ -345,8 +348,7 @@ def adapt_to_hardware(self):
                 print(" Disabling shaders for Intel Express graphics")
             cmd.set("use_shaders", 0)
 
-    elif (vendor == 'nouveau'
-            or ' R300 ' in vendor # V: X.Org R300 Project, R: Gallium 0.4 on ATI RV370
+    elif (' R300 ' in vendor # V: X.Org R300 Project, R: Gallium 0.4 on ATI RV370
             ):
         if invocation.options.show_splash:
             print(" Detected blacklisted graphics driver.  Disabling shaders.")
@@ -452,6 +454,8 @@ def launch(args=None, block_input_hook=0):
 def finish_launching(args=None):
     '''
     Start the PyMOL process in a thread
+
+    THIS IS NOT SUPPORTED ON macOS
     '''
     global glutThreadObject
 
@@ -469,7 +473,7 @@ def finish_launching(args=None):
     if True:
         # run PyMOL in thread
         invocation.options.keep_thread_alive = 1
-        cmd.reaper = threading.currentThread()
+        cmd.reaper = threading.current_thread()
         glutThreadObject = threading.Thread(target=launch,
                 args=(list(args), 1))
         glutThreadObject.start()
@@ -559,7 +563,7 @@ def _colortype(cmd):
 
 ######### VARIABLES ############################
 
-glutThread = 0
+glutThread = None
 
 ######### ENVIRONMENT ##########################
 
@@ -568,9 +572,6 @@ setup_environ()
 # initialize instance-specific module/object internals
 _init_internals(sys.modules[__name__])
 
-# maximize responsiveness
-sys.setcheckinterval(1)
-
 # get X-window support (machine_get_clipboard)
 if 'DISPLAY' in os.environ:
     from .xwin import *
@@ -580,6 +581,8 @@ if 'DISPLAY' in os.environ:
 import pymol._cmd
 _cmd = sys.modules['pymol._cmd']
 
+get_capabilities = _cmd.get_capabilities
+
 from . import cmd
 
 cmd._COb = None
