@@ -28,43 +28,33 @@ from .cmd import DEFAULT_ERROR, DEFAULT_SUCCESS, loadable, _load2str, Shortcut,
 
 def _cache_validate(_self=cmd):
     r = DEFAULT_SUCCESS
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         _pymol = _self._pymol
         if not hasattr(_pymol,"_cache"):
             _pymol._cache = []
         if not hasattr(_pymol,"_cache_memory"):
             _pymol._cache_memory = 0
-    finally:
-        _self.unlock_data(_self)
-        
+
 def _cache_clear(_self=cmd):
     r = DEFAULT_SUCCESS
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         _pymol = _self._pymol
         _pymol._cache = []
         _pymol._cache_memory = 0
-    finally:
-        _self.unlock_data(_self)
     return r
-    
+
 def _cache_mark(_self=cmd):
     r = DEFAULT_SUCCESS
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         _pymol = _self._pymol
         _cache_validate(_self)
-        for entry in _self._pymol._cache: 
+        for entry in _self._pymol._cache:
             entry[5] = 0.0
-    finally:
-        _self.unlock_data(_self)
     return r
 
 def _cache_purge(max_size, _self=cmd):
     r = DEFAULT_SUCCESS
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         _pymol = _self._pymol
         _cache_validate(_self)
         if len(_pymol._cache):
@@ -77,7 +67,7 @@ def _cache_purge(max_size, _self=cmd):
                 new_cache = [x[1] for x in new_cache]
                 # remove oldest entries one by one until size requirement is met
                 while (cur_size>max_size) and (len(new_cache)>1):
-                    entry = new_cache.pop() 
+                    entry = new_cache.pop()
                     cur_size = cur_size - entry[0]
                 _pymol._cache = new_cache
                 _pymol._cache_memory = cur_size
@@ -91,16 +81,13 @@ def _cache_purge(max_size, _self=cmd):
                 _pymol._cache = new_cache
                 _pymol._cache_memory = cur_size
         result = _pymol._cache_memory
-    finally:
-        _self.unlock_data(_self)
     return result
-        
+
 def _cache_get(target, hash_size = None, _self=cmd):
     result = None
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         try:
-            if hash_size == None:
+            if hash_size is None:
                 hash_size = len(target[1])
             key = target[1][0:hash_size]
             # should optimize this with a dictionary lookup, key -> index in _cache
@@ -115,14 +102,11 @@ def _cache_get(target, hash_size = None, _self=cmd):
                         break
         except:
             traceback.print_exc()
-    finally:
-        _self.unlock_data(_self)
     return result
 
 def _cache_set(new_entry, max_size, _self=cmd):
     r = DEFAULT_SUCCESS
-    try:
-        _self.lock_data(_self)
+    with _self.lock_api_data:
         _pymol = _self._pymol
         _cache_validate(_self)
         try:
@@ -132,10 +116,10 @@ def _cache_set(new_entry, max_size, _self=cmd):
             found = 0
             new_entry[4] = new_entry[4] + 1 # incr access count
             new_entry[5] = time.time() # timestamp
-            for entry in _pymol._cache: 
+            for entry in _pymol._cache:
                 if entry[1][0:hash_size] == key:
                     if entry[2] == new_entry[2]: # dupe (shouldn't happen)
-                        entry[3] = new_entry[3] 
+                        entry[3] = new_entry[3]
                         found = 1
                         break
                 count = count + 1
@@ -147,14 +131,12 @@ def _cache_set(new_entry, max_size, _self=cmd):
                         _cache_purge(max_size, _self)
         except:
             traceback.print_exc()
-    finally:
-        _self.unlock_data(_self)
     return r
-        
+
 # ray tracing threads
 
 def _ray_anti_spawn(thread_info,_self=cmd):
-    # WARNING: internal routine, subject to change      
+    # WARNING: internal routine, subject to change
     # internal routine to support multithreaded raytracing
     thread_list = []
     for a in thread_info[1:]:
@@ -169,24 +151,24 @@ def _ray_anti_spawn(thread_info,_self=cmd):
         t.join()
 
 def _ray_hash_spawn(thread_info,_self=cmd):
-    # WARNING: internal routine, subject to change      
+    # WARNING: internal routine, subject to change
     # internal routine to support multithreaded raytracing
     thread_list = []
     for a in thread_info[1:]:
-        if a != None:
+        if a is not None:
             t = threading.Thread(target=_cmd.ray_hash_thread,
                                  args=(_self._COb,a))
             t.setDaemon(1)
             thread_list.append(t)
     for t in thread_list:
         t.start()
-    if thread_info[0] != None:
+    if thread_info[0] is not None:
         _cmd.ray_hash_thread(_self._COb,thread_info[0])
     for t in thread_list:
         t.join()
 
 def _ray_spawn(thread_info,_self=cmd):
-    # WARNING: internal routine, subject to change      
+    # WARNING: internal routine, subject to change
     # internal routine to support multithreaded raytracing
     thread_list = []
     for a in thread_info[1:]:
@@ -272,7 +254,7 @@ def _mpng(prefix, first=-1, last=-1, preserve=0, modal=0,
     format = int(format)
     # WARNING: internal routine, subject to change
     try:
-        _self.lock(_self)   
+        _self.lock(_self)
         fname = prefix
         if re.search("[0-9]*\.png$",fname): # remove numbering, etc.
             fname = re.sub("[0-9]*\.png$","",fname)
@@ -296,7 +278,7 @@ def _mpng(prefix, first=-1, last=-1, preserve=0, modal=0,
 def _copy_image(_self=cmd,quiet=1):
     r = DEFAULT_ERROR
     try:
-        _self.lock(_self)   
+        _self.lock(_self)
         r = _cmd.copy_image(_self._COb,int(quiet))
     finally:
         _self.unlock(r,_self)
@@ -382,13 +364,15 @@ def _load(oname,finfo,state,ftype,finish,discrete,
     # caller must already hold API lock
     # NOTE: state index assumes 1-based state
     r = DEFAULT_ERROR
+    contents = None
     size = 0
     if ftype not in (loadable.model,loadable.brick):
         if True:
             if ftype in _load2str:
-                finfo = _self.file_read(finfo)
+                contents = _self.file_read(finfo)
                 ftype = _load2str[ftype]
-            r = _cmd.load(_self._COb,str(oname),finfo,int(state)-1,int(ftype),
+        return _cmd.load(_self._COb, str(oname), str(finfo), contents,
+                          int(state) - 1, int(ftype),
                           int(finish),int(discrete),int(quiet),
                           int(multiplex),int(zoom), plugin,
                           object_props, atom_props, int(mimic))
@@ -530,14 +514,14 @@ def _cmmd(k,_self=cmd):
     # command-key on macs
     if k in _self.cmmd:
         ak = _self.cmmd[k]
-        if ak[0]!=None:
+        if ak[0] is not None:
             ak[0](*ak[1], **ak[2])
     return None
 
 def _ctsh(k,_self=cmd):
     # WARNING: internal routine, subject to change
     _invoke_key('CTSH-' + k, 0, _self)
-    
+
 
 # writing PNG files (thread-unsafe)
 
@@ -545,7 +529,7 @@ def _png(a,width=0,height=0,dpi=-1.0,ray=0,quiet=1,prior=0,format=-1,_self=cmd):
     # INTERNAL - can only be safely called by GLUT thread (unless prior == 1)
     # WARNING: internal routine, subject to change
     try:
-        _self.lock(_self)   
+        _self.lock(_self)
         fname = a
         if re.search("\.ppm$",fname):
             if format<0:
@@ -567,10 +551,11 @@ def _png(a,width=0,height=0,dpi=-1.0,ray=0,quiet=1,prior=0,format=-1,_self=cmd):
 def _quit(code=0, _self=cmd):
     pymol=_self._pymol
     # WARNING: internal routine, subject to change
+    _self.interrupt()
     try:
         _self.lock(_self)
         try: # flush and close log if possible to avoid threading exception
-            if pymol._log_file!=None:
+            if pymol._log_file is not None:
                 try:
                     pymol._log_file.flush()
                 except:
@@ -579,7 +564,7 @@ def _quit(code=0, _self=cmd):
                 del pymol._log_file
         except:
             pass
-        if _self.reaper!=None:
+        if _self.reaper is not None:
             try:
                 _self.reaper.join()
             except:
@@ -596,14 +581,11 @@ def _refresh(swap_buffers=1,_self=cmd):  # Only call with GLUT thread!
     r = None
     try:
         _self.lock(_self)
-        if hasattr(_self._pymol,'glutThread'):
-            if thread.get_ident() == _self._pymol.glutThread:
+        if _self.is_gui_thread():
                 if swap_buffers:
                     r = _cmd.refresh_now(_self._COb)
                 else:
                     r = _cmd.refresh(_self._COb)
-            else:
-                r = _cmd.refresh_later(_self._COb)                
         else:
             r = _cmd.refresh_later(_self._COb)
     finally:
@@ -626,7 +608,7 @@ def _interpret_color(_self,color):
 
 def _validate_color_sc(_self=cmd):
     # WARNING: internal routine, subject to change
-    if _self.color_sc == None: # update color shortcuts if needed
+    if _self.color_sc is None: # update color shortcuts if needed
         lst = _self.get_color_indices()
         names = [x[0] for x in lst]
         names.extend(['default', 'auto', 'current', 'atomic'])
@@ -657,8 +639,6 @@ def _get_feedback(_self=cmd): # INTERNAL
         l = None
     return l
 
-get_feedback = _get_feedback # for legacy compatibility
-
 def _fake_drag(_self=cmd): # internal
     _self.lock(_self)
     try:
@@ -669,7 +649,7 @@ def _fake_drag(_self=cmd): # internal
 
 def _sdof(tx,ty,tz,rx,ry,rz,_self=cmd):
     _cmd._sdof(_self._COb,tx,ty,tz,rx,ry,rz)
-    
+
 # testing tools
 
 # for comparing floating point numbers calculated using
